##
## Finding Chart plugin for SkyCat
##
## 13/07/2001
##
## S.Marteau (smarteau@eso.org)
##
## $Id: FindingChart_plugin.tcl,v 1.24 2008/06/23 10:44:33 smarteau Exp $

## This plug-in is to be used with Skycat. It adds a dialog box to the application,
## available in the 'File' menu under 'Make finding chart...'. It enables to easily
## create VLT-compliant finding charts by providing the mandatory information in the
## dialog box (PI name, run ID, OB name, image band...). A scale and a N-E compass
## are also automatically added to the displayed image, which can then be saved
## to a jpeg file, in order to be later attached to the corresponding OB(s).
##
## Additionally, the dialog box offers to draw boxes of arbitrary on-sky sizes and
## position angles, thus allowing the user to draw the field-of-view of an instrument
## or a slit.


# Activate namespace dedicated to the plug-in
namespace eval ::FindingChart {

    # Shortcuts to Skycat widgets
    variable main
    variable canvas
    variable imageCtrl
    variable image

    # Variables for the finding chart
    variable startDir [pwd]
    variable runID ""
    variable piName ""
    variable obName ""
    variable imgBand ""
    variable userChosen "false"
    variable textColor ""
    variable textFont  "-*-courier-bold-r-*-*-24-*-*-*-*-*-*-*"
    variable textFont2 "-*-courier-bold-r-*-*-22-*-*-*-*-*-*-*"

    # Variables for the drawing of boxes
    variable box_Xdim 0
    variable box_Ydim 0
    variable box_angle 0
    variable box_counter 1
    variable box_RA ""
    variable box_DEC ""

    # Global array to store plot parameters
    variable plotParams
    array set plotParams {draw "false" \
			      drawbox "false" \
			      jpegResolution "high-res"}

    variable tops                 ;# For hiding other toplevels
}


# Create namespace for math functions
namespace eval ::math {

    # Return max of ( a b )
    proc max { a b } {

        if { $a > $b } {
            return $a
        } else {
            return $b
        }

    }

    # Return min of ( a b )
    proc min { a b } {

        if { $a < $b } {
            return $a
        } else {
            return $b
        }

    }

    # Cartesian distance between points A(xa,ya) and B(xb,yb)
    proc distAB { xa ya xb yb } {

	set dist [expr sqrt(pow(($xb-$xa),2)+pow(($yb-$ya),2))]
	return $dist
    }

    # Return non-signed angle between segments AB and AC
    proc angleABAC { xa ya xb yb xc yc } {

	set a [distAB $xb $yb $xc $yc]
	set b [distAB $xa $ya $xc $yc]
	set c [distAB $xa $ya $xb $yb]

	set theta [expr acos((pow($b,2)+pow($c,2)-pow($a,2))/(2*$b*$c))]
	return $theta
    }

    # Rotate canvas item(s) of a given angle
    # w : canvas widget
    # tagOrId : identifier of item(s) on canvas
    # Ox, Oy : canvas coordinates of the center of rotation
    # angle : angle of rotation in degrees (positive = clockwise)
    #
    proc RotateItem {w tagOrId Ox Oy angle} {

	# Convert angle to radians
	set angle [expr {$angle * atan(1) * 4 / 180.0}] ;# Radians

	# Loop on all concerned canvas items
	foreach id [$w find withtag $tagOrId] {     ;# Do each component separately

	    # list to store new coordinates
	    set xy {}
	    
	    # loop on all coordinates of item ("corners")
	    foreach {x y} [$w coords $id] {

		# rotates vector (Ox,Oy)->(x,y) by 'angle' clockwise
		
		set x [expr {$x - $Ox}]             ;# Shift to origin of rotation
		set y [expr {$y - $Oy}]
		
		set xx [expr {$x * cos($angle) - $y * sin($angle)}] ;# Rotate
		set yy [expr {$x * sin($angle) + $y * cos($angle)}]
		
		set xx [expr {$xx + $Ox}]           ;# Shift back
		set yy [expr {$yy + $Oy}]
		lappend xy $xx $yy
	    }
	    
	    # Set new coordinates for canvas item
	    $w coords $id $xy
	}
    }

}


# Mandatory procedure to let Skycat know
# about the plug-in.
#
# IMPORTANT: the name of this procedure must NOT be changed
#            to ensure integration in Skycat
#
proc SkyCat_plugin {this} {

    variable ::FindingChart::main
    variable ::FindingChart::canvas
    variable ::FindingChart::imageCtrl
    variable ::FindingChart::image

    # Request Img package (used for snapshots)
    package require Img

    # Get main window name
    set w [utilNamespaceTail $this]

    # Set name of main window (= SkyCat, derived from rtd::Rtd)
    set ::FindingChart::main $w

    # Set name of image component (= SkyCatCtrl, derived from rtd::RtdImageCtrl)
    set ::FindingChart::imageCtrl [$w component image]

    # Set name of underlying rtdimage
    set ::FindingChart::image [$::FindingChart::imageCtrl get_image]

    # Set name of canvas component (= Tk canvas)
    set ::FindingChart::canvas [$::FindingChart::imageCtrl component canvas]

    # Add finding chart features to Skycat GUI
    ::FindingChart::AddFeatures

    # Ensure that deletion of the main window quits the application
    wm protocol $::FindingChart::main WM_DELETE_WINDOW {
	exit
    }

}

## Add features related to this plug-in:
#  - add menu entry
#  - disable some disturbing features (rotation+flipping)
#  - add some dedicated bindings
#
proc ::FindingChart::AddFeatures {} {

    variable main
    variable canvas
    variable imageCtrl

    # Handle to the 'File' menu
    set m [$main get_menu File]

    # Insert new menu item (including keyboard shortcut)
    $main insert_menuitem $m 5 command "Make finding chart..." \
	{Make a finding chart} \
	-command [code ::FindingChart::MakeDialog] \
	-accelerator {Control-F}

    # Disable flipping and rotating functionalities
    # (unmap corresponding widgets)
    # NOTE: see how one can re-enable this?
    set RtdImagePanel [$imageCtrl component panel]
    
    pack forget $RtdImagePanel.info.trans.rotate
    pack forget $RtdImagePanel.info.trans.flipx
    pack forget $RtdImagePanel.info.trans.flipy

    # Shift + Button-3 --> recenter image on mouse cursor position
    bind $canvas <Shift-Button-3> [code ::FindingChart::Recenter %x %y]

    # Alt+Shift+Z --> zoom in
    bind $canvas <Alt-Z> +[code ::FindingChart::ZoomIn]

    # Alt+z --> zoom out
    bind $canvas <Alt-z> +[code ::FindingChart::ZoomOut]

    set draw [$imageCtrl component draw]
    bind $canvas <BackSpace> +[code eval $draw delete_selected_objects]
    bind $canvas <Delete> +[code eval $draw delete_selected_objects]

    # Change behaviour of RTD zoom in/out buttons to reflect the <Alt-Z>
    # and <Alt-z> shortcuts
    $main.image.panel.info.trans.larger configure -command [code ::FindingChart::ZoomIn]
    $main.image.panel.info.trans.smaller configure -command [code ::FindingChart::ZoomOut]

}

## Build finding chart dialog
#
proc ::FindingChart::MakeDialog {} {

    # Finding chart indications
    variable runID
    variable piName
    variable obName
    variable imgBand
    variable userChosen

    # Parameters for drawing of boxes
    variable box_Xdim
    variable box_Ydim
    variable box_angle
    variable box_RA
    variable box_DEC

    # If dialog window already exists, just bring it
    # to front and exit
    if { [winfo exists .fcDialog] == 1 } {
        raise .fcDialog
        return
    }
    
    # Create toplevel window
    toplevel .fcDialog 
    wm title .fcDialog "Finding Chart information"

    # First line of dialog window: run ID
    set w1 [LabelEntry .fcDialog.runID -text "Run ID:" -orient horizontal \
		-valuewidth 20 -labelwidth 17 \
		-state normal -textvariable ::FindingChart::runID \
		-value $runID]

    # Second line: PI name
    set w2 [LabelEntry .fcDialog.piName -text "PI Name:" -orient horizontal \
		-valuewidth 20 -labelwidth 17 \
		-state normal -textvariable ::FindingChart::piName \
		-value $piName]

    # Third line: OB name(s)
    set w3 [LabelEntry .fcDialog.obName -text "List of OBs:" -orient horizontal \
		-valuewidth 20 -labelwidth 17 \
		-state normal -textvariable ::FindingChart::obName \
		-value $obName ]

    # Fourth line: Band/wavelength of the chart
    set w4 [LabelEntry .fcDialog.band -text "Band / Wavelength\n of this chart:" -orient horizontal \
		-valuewidth 20 -labelwidth 17 \
		-state normal -textvariable ::FindingChart::imgBand \
		-value $imgBand ]

    # Pack widgets together
    pack $w1 $w2 $w3 $w4 -side top -expand yes -fill both -padx 4 -ipady 3

    set pf [frame .fcDialog.pf]
    # Menu: choose place for info
    set w1 [LabelMenu $pf.infoLocation -text "Location of info:" \
		-labelwidth 13 -valuewidth 16 \
		-relief raised -orient horizontal -valuefont 8x13 ]

    # Menu: choose text color
    set w2 [LabelMenu $pf.infoColor -text "Text color:" \
		-labelwidth 8 -valuewidth 10 \
		-relief raised -orient horizontal -valuefont 8x13 ]

    # Pack menu widgets    
    pack $w1 $w2 -side left -expand yes -fill both -ipadx 8
    pack $pf -side top -pady 5

    # Populate menu for position of info
    $w1 add -label "top-left corner"
    $w1 add -label "top-right corner"
    $w1 add -label "bottom-left corner"
    $w1 add -label "bottom-right corner"
    $w1 add_separator
    $w1 add -label "user chosen"
    
    # Populate menu for text color
    $w2 add -label "black"
    $w2 add -label "white"
    $w2 add -label "red"
    $w2 add -label "blue"

    #
    # Dialog for box drawing
    #

    set bgcolor grey70

    set bf [frame .fcDialog.boxFrame -relief ridge -borderwidth 2 -bg $bgcolor]
    
    set bf1 [frame $bf.f1 -bg $bgcolor]
    set bf2 [frame $bf.f2 -bg $bgcolor]

    set w1 [LabelEntry $bf1.boxXdim -text "Draw a box of " -orient horizontal \
		-valuewidth 4 -labelwidth 11 -background $bgcolor \
		-state normal -textvariable ::FindingChart::box_Xdim \
		-value $box_Xdim ]

    set w2 [LabelEntry $bf1.boxYdim -text "arcsec wide   by" -orient horizontal \
		-valuewidth 4 -labelwidth 13 -background $bgcolor  \
		-state normal -textvariable ::FindingChart::box_Ydim \
		-value $box_Ydim ]
 
    set w3 [label $bf1.unit -text "arcsec long" -bg $bgcolor ]

    pack $w1 $w2 $w3 -side left -expand no -fill both -ipadx 4 -pady 7

    set w1 [LabelEntry $bf2.boxAngle -text "at a PA angle on sky of" -orient horizontal \
		-valuewidth 4 -labelwidth 19  -background $bgcolor \
		-state normal -textvariable ::FindingChart::box_angle \
		-value $box_angle ]
 
    set w2 [label $bf2.unit -text "degrees" -bg $bgcolor ]

    pack $w1 $w2 -side left -expand no -fill both -ipadx 4 -pady 7

    set w3 [button $bf.drawBox -text "Draw box" -width 10 \
		-command [code ::FindingChart::PlotBox]]

    grid $bf1 -in $bf -row 1 -column 1 -sticky nsew -ipadx 10
    grid $bf2 -in $bf -row 2 -column 1 -sticky nsew -ipadx 10
    grid $w3  -in $bf -row 1 -column 2 -rowspan 2 -ipadx 10 -padx 10

    set bf3 [frame $bf.f3 -bg $bgcolor]
 
    set w1 [LabelEntry $bf3.boxRA -text "(Optional box center coords)  RA:" -orient horizontal \
		-valuewidth 10 -labelwidth 27 -background $bgcolor \
		-state normal -textvariable ::FindingChart::box_RA \
		-value $box_RA ]

    set w2 [LabelEntry $bf3.boxDEC -text "  DEC:" -orient horizontal \
		-valuewidth 10 -labelwidth 4 -background $bgcolor \
		-state normal -textvariable ::FindingChart::box_DEC \
		-value $box_DEC ]

    pack $w1 $w2 -side left -expand no -fill both -ipadx 4 -pady 7
   
    grid $bf3 -in $bf -row 3 -column 1 -columnspan 2 -sticky ew -ipadx 10
    pack $bf -side top -expand yes -fill both -pady 5 -padx 4

    # Frame for button row
    frame .fcDialog.buttonFrame

    # Button: draw info on finding chart
    button .fcDialog.buttonFrame.draw -text "Draw" \
        -command [code ::FindingChart::DrawFindingChart]

    # Button: save finding chart to jpeg
    button .fcDialog.buttonFrame.save -text "Save to jpeg..." \
        -command [code ::FindingChart::SaveFindingChart]
    
    # Button: print finding chart
    button .fcDialog.buttonFrame.print -text "Print..." \
        -command [code ::FindingChart::PrintFindingChart]

    # Button: reset coordinates of elements
    button .fcDialog.buttonFrame.reset -text "Reset coords" \
        -command { set ::FindingChart::userChosen "false" }

    # Button: clear all finding chart elements
    button .fcDialog.buttonFrame.clear -text "Clear" \
        -command [code ::FindingChart::ClearFindingChart]

    # Button: help window
    button .fcDialog.buttonFrame.help -text "Help" \
	-command [code ::FindingChart::DisplayHelp]

    # Pack all buttons in frame
    pack .fcDialog.buttonFrame.draw .fcDialog.buttonFrame.save \
	    .fcDialog.buttonFrame.print \
	    .fcDialog.buttonFrame.reset .fcDialog.buttonFrame.clear \
	    .fcDialog.buttonFrame.help \
	    -side left -fill both -padx 5

    # Pack frame into dialog
    pack .fcDialog.buttonFrame -side top -fill both -expand yes -padx 5 -pady 8

    # Make the dialog window not resizable
    wm resizable .fcDialog 0 0

    return
}


# HideToplevels -- withdraws all top levels so the
# main window is not obscured
#
proc ::FindingChart::HideToplevels {hide} {
    variable tops

    # Hide toplevel windows
    if {$hide} {
        set tops {}
        foreach w [winfo child .] {
            if {[winfo class $w] != "Toplevel"} continue
            lappend tops $w [wm state $w]
            if {[wm state $w] == "normal"} {
                wm withdraw $w
            }
        }
        raise .
        update
    } else {
	# restore previous states of toplevel windows
        foreach {w wstate} $tops {
            if {$wstate == "normal"} { wm deiconify $w }
        }
    }
}

# Procedure to recenter the image displayed in the Skycat interface,
# based on the current mouse position. See binding in procedure
# "AddFeatures".
# 
# inputs:
#   x, y: cursor coordinates
#
proc ::FindingChart::Recenter {x y} {

    variable main
    variable image
    variable canvas

    ##
    ## Step 0: is there an image or not? If not, return.
    ##

    # If no image loaded, return
    if { [$image isclear] } { return }

    ##
    ## Step 1: is the cursor really inside the image? If not, return.
    ##

    # Get image dimensions (original size, _not_ taking zoom factor into account)
    set image_height [$image height]
    set image_width [$image width]

    # Lower left corner of image (image coords -> screen coords)
    $image convert coords 0 0 image x1 y2 screen

    # Upper right corner of image (image coords -> screen coords)
    $image convert coords $image_width $image_height image x2 y1 screen

    # Get canvas dimensions (visible size on screen)
    set ch [winfo height $canvas]
    set cw [winfo width $canvas]

    # If image is larger than visible canvas, image corners = canvas corners
    if { $x1 < 0 } { set x1 0 }
    if { $y1 < 0 } { set y1 0 }
    if { $x2 > $cw } { set x2 $cw }
    if { $y2 > $ch } { set y2 $ch }

    # If cursor is out of image, do nothing (and return...)
    if { $x < $x1 || $x > $x2 || $y < $y1 || $y > $y2 } { return }

    ##
    ## Step 2: now we figure out the new scroll area and we recenter the image
    ##

    # Get apparent image dimensions (_includes_ the zoom factor)
    set dw [$image dispwidth]
    set dh [$image dispheight]
    
    if { $cw > $dw } {
	set scrollx [expr $cw/2.]
    } else {
 	set scrollx [expr $dw - $cw/2.]
    }

    if { $ch > $dh } {
	set scrolly [expr $ch/2.]
    } else {
 	set scrolly [expr $dh - $ch/2.]
    }


    $canvas config -scrollregion "[expr -$scrollx] [expr -$scrolly]  [expr $cw+$scrollx] [expr $ch+$scrolly]"

    # Convert cursor position to canvas coordinates
    $image convert coords $x $y screen xc yc canvas

    # Convert canvas visible center to canvas coordinates
    $image convert coords [expr $cw/2.] [expr $ch/2.] screen xcc ycc canvas

    # Compute scroll fractions
    set xFraction [expr ($xc-$cw/2.+$scrollx)/($cw+2.*$scrollx)]
    set yFraction [expr ($yc-$ch/2.+$scrolly)/($ch+2.*$scrolly)]

    if {$cw != 1 && $dw && $dh} {

	$image warp [expr int($xcc - $xc)] [expr int($ycc - $yc)]
	$canvas xview moveto $xFraction
	$canvas yview moveto $yFraction
	
    }
    
}

## Zoom: zooms in/out of the image
#
proc ::FindingChart::Zoom { increment } {

    variable imageCtrl
    variable image
    variable canvas

    # Get canvas height and width
    set ch [winfo height $canvas]
    set cw [winfo width $canvas]

    # Convert center of canvas to image coordinates
    $image convert coords [expr $cw/2.] [expr $ch/2.] screen xi yi image

    # Retrieve current scale factor of image
    set scale_start [lindex [$image scale] 0]

    # Add/subtract increment to scale, limit to -10
    if { $scale_start > -10 } {
	set scale [expr $scale_start + $increment]
    }

    # If we go from a scale <= 50% to a scale >= 100%,
    # add 2 to the final scale (because of the jump from -2 --> +1)
    if { $scale_start <= -2 && $scale >= -1 } {
	incr scale 2
    }

    # If we go from a scale >= 100% to a scale <= 50%,
    # subtract 2 from the final scale (because of the jump from +1 --> -2)
    if { $scale_start >= 1 && $scale <= 0 } {
	incr scale -2
    }

    # Set new scale of image
    # (remark: done through imageCtrl rather than image, so that
    #  all widgets get updated at the same time, and not the
    #  image only)
    $imageCtrl scale $scale $scale

    # Scale text in the finding chart
    ::FindingChart::ScaleText \
	[expr ($scale_start < 0) ? (-1./$scale_start) : $scale_start] \
	[expr ($scale < 0) ? (-1./$scale) : $scale]

    # Convert center of image back to canvas coordinates
    $image convert coords $xi $yi image xc yc canvas

    # Get apparent image dimensions (includes the zoom factor)
    set dw [$image dispwidth]
    set dh [$image dispheight]

    # Compute scroll region (similar to procedure ::FindingChart::Recenter)
    if { $cw > $dw } {
	set scrollx [expr $cw/2.]
    } else {
 	set scrollx [expr $dw - $cw/2.]
    }

    if { $ch > $dh } {
	set scrolly [expr $ch/2.]
    } else {
 	set scrolly [expr $dh - $ch/2.]
    }

    $canvas config -scrollregion "[expr -$scrollx] [expr -$scrolly]  [expr $cw+$scrollx] [expr $ch+$scrolly]"

    # Compute scroll fractions
    set xFraction [expr ($xc-$cw/2.+$scrollx)/($cw+2.*$scrollx)]
    set yFraction [expr ($yc-$ch/2.+$scrolly)/($ch+2.*$scrolly)]

    # Set canvas to new scroll position to ensure that image remains
    # centered on same point after zooming in !
    if {$cw != 1 && $dw && $dh} {

	$canvas xview moveto $xFraction
	$canvas yview moveto $yFraction
	
    }
    
}

## ZoomIn: zooms into the image
#
proc ::FindingChart::ZoomIn {} {

    ::FindingChart::Zoom 1

}

## ZoomOut: zooms out of the image
#
proc ::FindingChart::ZoomOut {} {

    ::FindingChart::Zoom -1

}

## Scale text up/down in the finding chart
proc ::FindingChart::ScaleText { scale1 scale2 } {

    variable canvas

    foreach item [$canvas find withtag fctext] {

	set font [$canvas itemcget $item -font]
	regexp {[0-9]+} $font size

	set newsize [expr int($size * (double($scale2)/$scale1))]

	regsub $size $font $newsize font

	$canvas itemconfigure $item -font $font
    }

}

## Clear the canvas from all Finding Chart information
proc ::FindingChart::ClearFindingChart {} {

    variable plotParams
    variable canvas
    variable box_counter

    # Delete all canvas elements tagged with 'fcinfo'
    $canvas delete fcinfo fcBoxInfo
    update
    
    set plotParams(draw) "false"
    set plotParams(drawbox) "false"
    .fcDialog.buttonFrame.draw configure -text "Draw"

    set box_counter 1
}    

## Draw Finding Chart info in canvas
proc ::FindingChart::DrawFindingChart {} {


    variable main
    variable image
    variable canvas

    variable runID
    variable piName
    variable obName
    variable imgBand
    variable textColor
    variable userChosen

    variable plotParams

    # If WCS info missing in image, cannot draw FC information --> exit
    if { [$image wcsheight] == "" } {

	tk_dialog .dia "WCS missing" "This image does not support\nthe WCS coordinate system !" \
	    warning 0 OK
	return
    }
    
    # If no image present --> exit
    if { [$image isclear] } {
	tk_dialog .dia "No image" "No image is currently loaded !" \
	    warning 0 OK
	return
    }

    ## Checks for missing mandatory info
    if { [string trim $runID] == "" } {
	tk_dialog .dia "Missing info" "You must provide\nthe run ID !" \
	    warning 0 OK
	return
    }

    if { [string trim $piName] == "" } {
	tk_dialog .dia "Missing info" "You must provide\nthe PI name !" \
	    warning 0 OK
	return
    }

    if { [string trim $obName] == "" } {
	tk_dialog .dia "Missing info" "You must provide\nthe OB name !" \
	    warning 0 OK
	return
    }

    if { [string trim $imgBand] == "" } {
	tk_dialog .dia "Missing info" "You must provide the band \nor wavelength of the image !" \
	    warning 0 OK
	return
    }

    if { [catch {set textColor [.fcDialog.pf.infoColor get]} ] != 0 } {
	set textColor black
    }

    ## DISABLE flipping and rotating facilities
    $main.image.panel.info.trans.rotate configure -state disabled
    $main.image.panel.info.trans.flipx configure -state disabled
    $main.image.panel.info.trans.flipy configure -state disabled

    ## for more security, restore initial flipping/rotating status
    $image rotate 0
    $image flip x 0
    $image flip y 0

    # Set colormap to "black-on-white"
    $image cmap file ramp.lasc
    $image itt file neg.iasc

    # Get original image dimensions (does _not_ take zoom factor
    # into account)
    set image_height [$image height]
    set image_width [$image width]

    # Get WCS pixel scale (required to draw scale on chart)
    set plotParams(pixscale) [lindex [$image wcsset] 2]

    # Get WCS rotation angle (required to figure out orientation)
    set plotParams(rotAngle) [lindex [$image wcsset] 7]

    # Get visible size of canvas
    set canvas_visible_height [winfo height $canvas]
    set canvas_visible_width [winfo width $canvas]

    # Lower left corner of image (image coords -> screen coords)
    $image convert coords 0 0 image x1 y2 screen

    # Upper right corner of image (image coords -> screen coords)
    $image convert coords $image_width $image_height image x2 y1 screen

    # If image is larger than visible canvas, image corners = canvas corners
    if { [$image rotate] == 0 } {
	
	# Case: image not rotated
	if { $y1 < 0 } { set y1 0 }
	if { $x1 < 0 } { set x1 0 }
	if { $x2 > $canvas_visible_width } { set x2 $canvas_visible_width }
	if { $y2 > $canvas_visible_height } { set y2 $canvas_visible_height }

    } else {

	# Case: image rotated
	# NB: THIS IS NOT USED BECAUSE ROTATION IS DISABLED
	if { $x1 > $canvas_visible_width } { set x1 $canvas_visible_width }
	if { $y1 > $canvas_visible_height } { set y1 $canvas_visible_height }
	if { $x2 < 0 } { set x2 0 }
	if { $y2 < 0 } { set y2 0 }

    }

    # Clean canvas of all FC information
    # (delete all elements with tag 'fcinfo')
    $canvas delete fcinfo
    update

    # Store coordinates of corners of visible image, in canvas coordinates
    if { [$image rotate] == 0 } {
	
	# Case: image not rotated
	$image convert coords $x1 $y1 screen plotParams(x1_c) plotParams(y1_c) canvas
	$image convert coords $x2 $y2 screen plotParams(x2_c) plotParams(y2_c) canvas

    } else {

	# Case: image rotated
	# NB: THIS IS NOT USED BECAUSE ROTATION IS DISABLED
	$image convert coords $x2 $y2 screen plotParams(x1_c) plotParams(y1_c) canvas
	$image convert coords $x1 $y1 screen plotParams(x2_c) plotParams(y2_c) canvas

    }


    ## Determine image scale
    set plotParams(scale) [lindex [$image scale] 0]

    # If scale is negative, convert into positive number
    # (e.g. scale = -2 --> scaling factor = 1/2,
    #  scale = -3 --> scaling factor = 1/3, etc.)
    if { $plotParams(scale) < 0 } {
	set plotParams(scale) [expr -1./$plotParams(scale)]
    }

    # Compute dimensions of visible image _on sky_
    # in arcseconds
    set plotParams(visImageHeight) \
	[expr $plotParams(pixscale)*abs($y2-$y1)/$plotParams(scale)]

    set plotParams(visImageWidth) \
	[expr $plotParams(pixscale)*abs($x2-$x1)/$plotParams(scale)]


    ## Choice for location of FC info, retrieved from the
    #  corresponding menu in the dialog window
    set fcLocation [.fcDialog.pf.infoLocation get]

    ###
    # Draw finding chart info on canvas, depending on user's choice
    ###

    switch $fcLocation {

	"top-left corner" {

	    # Case: top-left corner:
	    #  - run info = top-left
	    #  - scale    = bottom-left
	    #  - compass  = bottom-right

	    # Run info: 10pix of top-left corner
	    set plotParams(xInfo_c) [expr $plotParams(x1_c) + 10]
	    set plotParams(yInfo_c) [expr $plotParams(y1_c) + 10]
	    set plotParams(anchorInfo) "nw"

	    # Compass: origin determined according to orientation
	    set plotParams(origCompassX_c) "undef"
	    set plotParams(origCompassY_c) "undef"

	    # Scale: 20pix of bottom-left corner 
	    set plotParams(origScaleX_c) [expr $plotParams(x1_c) + 20]
	    set plotParams(origScaleY_c) [expr $plotParams(y2_c) - 20]

	    set userChosen false

	    ::FindingChart::PlotRunInfo
	    ::FindingChart::PlotScale
	    ::FindingChart::PlotCompass br
	}

	"top-right corner" {

	    # Case: top-right corner:
	    #  - run info = top-right
	    #  - scale    = bottom-right
	    #  - compass  = bottom-left

	    # Run info: 10pix of top-right corner
	    set plotParams(xInfo_c) [expr $plotParams(x2_c) - 10]
	    set plotParams(yInfo_c) [expr $plotParams(y1_c) + 10]
	    set plotParams(anchorInfo) "ne"

	    # Compass: origin determined according to orientation
	    set plotParams(origCompassX_c) "undef"
	    set plotParams(origCompassY_c) "undef"

	    # Scale: 20pix of bottom-left corner
	    set plotParams(origScaleX_c) [expr $plotParams(x1_c) + 20]
	    set plotParams(origScaleY_c) [expr $plotParams(y2_c) - 20]

	    set userChosen false

	    ::FindingChart::PlotRunInfo
	    ::FindingChart::PlotScale
	    ::FindingChart::PlotCompass br
	}

	"bottom-left corner" {

	    # Case: bottom-left corner:
	    #  - run info = bottom-left
	    #  - scale    = bottom-right
	    #  - compass  = top-left

	    # Run info: 10pix of bottom-left corner
	    set plotParams(xInfo_c) [expr $plotParams(x1_c) + 10]
	    set plotParams(yInfo_c) [expr $plotParams(y2_c) - 10]
	    set plotParams(anchorInfo) "nw"

	    # Compass: origin determined according to orientation
	    set plotParams(origCompassX_c) "undef"
	    set plotParams(origCompassY_c) "undef"

	    # Scale: 20pix of top-left corner
	    set plotParams(origScaleX_c) [expr $plotParams(x1_c) + 20]
	    set plotParams(origScaleY_c) [expr $plotParams(y1_c) + 20]

	    set userChosen false

	    ::FindingChart::PlotRunInfo
	    ::FindingChart::PlotScale
	    ::FindingChart::PlotCompass br
	}

	"bottom-right corner" {

	    # Case: bottom-right corner:
	    #  - run info = bottom-right
	    #  - scale    = bottom-left
	    #  - compass  = top-left

	    # Run info: 10pix of bottom-right corner
	    set plotParams(xInfo_c) [expr $plotParams(x2_c) - 10]
	    set plotParams(yInfo_c) [expr $plotParams(y2_c) - 10]
	    set plotParams(anchorInfo) "ne"

	    # Compass: origin determined according to orientation
	    set plotParams(origCompassX_c) "undef"
	    set plotParams(origCompassY_c) "undef"

	    # Scale: 20pix of top-left corner
	    set plotParams(origScaleX_c) [expr $plotParams(x1_c) + 20]
	    set plotParams(origScaleY_c) [expr $plotParams(y1_c) + 20]

	    set userChosen false

	    ::FindingChart::PlotRunInfo
	    ::FindingChart::PlotScale
	    ::FindingChart::PlotCompass bl
	}

	"user chosen" {

	    # Case: user chosen positioning
	    set plotParams(anchorInfo) "nw"

	    if { $userChosen == "true" } {

		# If positions already chosen, plot the FC info
		::FindingChart::PlotRunInfo
		::FindingChart::PlotScale
		::FindingChart::PlotCompass
	    
	    } else {
		
		#
		# Interactive procedure to choose positions of
		# FC elements
		#
		
		# Info dialog
		tk_dialog .dia "Choose anchor point" \
		    "Click 3 times in canvas to choose where to put:\n- program-related info\n- compass arrows\n- image scale" \
		    info 0 OK

		# Change cursor to cross
		$canvas config -cursor cross

		# temporary binding to retrieve coordinates when user clicks
		bind $canvas <Button-1> {
		    set ::fcXCoord %x
		    set ::fcYCoord %y
		}
		
		# Wait until variable is modified
		# (happens when click occurs due to above binding)
		vwait ::fcXCoord
		
		# Change cursor back to arrow
		$canvas config -cursor left_ptr

		# Record coordinates for placement of run info
		# (canvas coordinates)
		$image convert coords $::fcXCoord $::fcYCoord screen \
		    plotParams(xInfo_c) plotParams(yInfo_c) canvas

		# Plot run information on canvas
		::FindingChart::PlotRunInfo

		# Change cursor to cross
		$canvas config -cursor cross

		# Wait until variable is modified (user click)
		vwait ::fcXCoord

		# Change cursor back to arrow
		$canvas config -cursor left_ptr
		
		# Record coordinates for placement of compass
		$image convert coords $::fcXCoord $::fcYCoord screen \
		    plotParams(origCompassX_c) plotParams(origCompassY_c) canvas

		# Plot compass on canvas
		::FindingChart::PlotCompass

		# Change cursor to cross
		$canvas config -cursor cross

		# Wait until variable is modified (user click)
		vwait ::fcXCoord

		# Change cursor back to arrow
		$canvas config -cursor left_ptr

		# Record coordinates for placement of scale
		$image convert coords $::fcXCoord $::fcYCoord screen \
		    plotParams(origScaleX_c) plotParams(origScaleY_c) canvas

		# Plot scale on canvas
		::FindingChart::PlotScale

		# Record that user has chosen the placement
		# for the FC elements
		# (can be reset by the 'Reset Coords' button)
		set userChosen true

		# Remove temporary binding
		bind $canvas <Button-1> {}
	    }
	}
    }

    # FC elements have been drawn on canvas
    set plotParams(draw) "true"
    .fcDialog.buttonFrame.draw configure -text "Redraw"

    # In case the Finding Chart is redrawn: update the color
    # of all boxes drawn with the PlotBox method
    $canvas itemconfigure fcBoxInfo -fill $textColor
}

# Save the finding chart to a JPEG file
proc ::FindingChart::SaveFindingChart {} {

    global env

    variable main
    variable image
    variable canvas

    variable startDir
    variable runID
    variable plotParams

    # If FC elements are not drawn, exit
    if { $plotParams(draw) == "false" } {
        tk_dialog .fcdia "Missing info" \
            "You have to draw the Finding Chart\nbefore saving it !" \
            error 0 OK
        return
    }

    # If one has set an environment variable to store finding charts
    # use it as default path
    if { [info exists env(FC_DIR)] == 1 } {
	set fcpath $env(FC_DIR)
    } else {

	# otherwise remember directory used previously (if any)
	set fcpath ""
	catch {set fcpath $startDir}
    }

    # Remove parentheses from runID
    regsub -all {[()]} $runID "" runId

    # List of finding charts: use path/<runID>.*.jpg as pattern
    set fcList [glob -nocomplain "$fcpath/$runId.*.jpg"]

    # Determine index for name of finding chart:
    if { [llength $fcList] == 0 } {
       
	# If no files with that name exist yet, index = 1
        set index 1

    } else {

	# Otherwise get index from last file and increment
        set lastFile [file rootname [file tail [lindex $fcList end]]]
        if { [regexp {[0-9][0-9][0-9]$} $lastFile index] != 1 } {
            set index 1
        } else {
	    set index [string trimleft $index "0"]
	    set index [expr $index + 1]
	}
    }

    # Default filename: <runId>.xxx.jpg
    set index [format "%03d" $index]
    set filename "$runId.$index.jpg"

    # Filter for file browser
    set types { { {JPEG files} {.jpg .jpeg} } }

    # Get name of file to save Finding Chart
    set file [tk_getSaveFile -title "Save Finding Chart" \
		  -filetypes $types \
		  -parent $main \
		  -initialdir $fcpath \
		  -initialfile $filename \
		  -defaultextension ".jpg" ]

    # If no file chosen, cancel saving and exit
    if { $file == "" } {
	return
    }

    # Remove file browser
    destroy $main.__tk_filedialog

    # Remember directory
    set startDir [file dirname $file]

    # Get image dimensions (original size, not taking zoom factor into account)
    set image_height [$image height]
    set image_width [$image width]

    # Get canvas dimensions (visible size on screen)
    set ch [winfo height $canvas]
    set cw [winfo width $canvas]

    # Get coordinates of corners of image in screen coordinates
    $image convert coords 1 1 image im_tl_x im_br_y screen
    $image convert coords $image_width $image_height image im_br_x im_tl_y screen


    # If image bigger than canvas, set corners of visible image accordingly
    if { $im_tl_x < 0 } { 
	set top_left_corner_x 0
    } else {
	set top_left_corner_x $im_tl_x
    }

    if { $im_br_x > $cw } {
	set bottom_right_corner_x $cw
    } else {
	set bottom_right_corner_x $im_br_x
    }

    if { $im_tl_y < 0 } {
	set top_left_corner_y 0
    } else {
	set top_left_corner_y $im_tl_y
    }

    if { $im_br_y > $ch } {
	set bottom_right_corner_y $ch
    } else {
	set bottom_right_corner_y $im_br_y
    }


    # Deduce dimensions of visible image
    set viewable_image_width \
	[expr int($bottom_right_corner_x - $top_left_corner_x)]
    set viewable_image_height \
	[expr int($bottom_right_corner_y - $top_left_corner_y)]

    # Get corners of viewable image (screen --> image)
    $image convert coords \
	$top_left_corner_x $bottom_right_corner_y screen llx lly image
    $image convert coords \
	$bottom_right_corner_x $top_left_corner_y screen urx ury image

    ## Determine image scale
    set plotParams(scale) [lindex [$image scale] 0]

    # If scale is negative, convert into positive number
    # (e.g. scale = -2 --> scaling factor = 1/2,
    #  scale = -3 --> scaling factor = 1/3, etc.)
    if { $plotParams(scale) < 0 } {
	set plotParams(scale) [expr -1./$plotParams(scale)]
    }


    if { $plotParams(scale) >= 1 || \
	 ($image_width <= $cw && $image_height <= $ch) } {
	set plotParams(jpegResolution) "normal"
    } else {
	set plotParams(jpegResolution) "high-res"
    }

    # Plotting in 'normal' mode
    if { $plotParams(jpegResolution) == "normal" } {

	# raise main window
	raise $main
	update

	# Hide other toplevel windows
	::FindingChart::HideToplevels 1
	
	after 500
	update

	# Take snapshot of canvas (only the visible part is grabbed)
	set retVal [catch {image create photo -format window -data $canvas} ph]
	
	if { $retVal != 0 } {
	    tk_dialog .fcdia "Cannot create image" \
		"ERROR: Cannot create image from canvas window" \
		error 0 OK
	    # Restore other toplevel windows
	    ::FindingChart::HideToplevels 0
	    return
	}
	
	# Restore other toplevel windows
	::FindingChart::HideToplevels 0

	# Create other image of the visible size of the image
	# This is necessary in case the image is actually smaller
	# than the visible canvas (it then fits entirely and we do
	# not want black borders around the image)
	image create photo fc_print -height $viewable_image_height -width $viewable_image_width

	# Copy part of the image grabbed to final image
	fc_print copy $ph -from \
	    [expr int($top_left_corner_x)] \
	    [expr int($top_left_corner_y)] \
	    [expr int($bottom_right_corner_x)] \
	    [expr int($bottom_right_corner_y)]

	# Try and save image to JPEG file
	set retVal [ catch {fc_print write -format jpeg $file} ]

	# Delete both images created
	image delete fc_print
	image delete $ph

	# Grab exception if saving was not possible
	if { $retVal != 0 } {
	    tk_dialog .fcdia "Cannot write JPEG file" \
		"ERROR: Cannot save Finding Chart to JPEG file" \
		error 0 OK
	    return
	}

    } else {
	
	####
	# Case of 'high-resolution' finding chart:
	# so as to provide a more detailed finding chart,
	# e.g. for FCs with MOS masks, we first zoom in, and
	# we record the bigger version. Due to Tk limitations,
	# this requires a "pan & scan" scheme to record each time
	# the visible part of the canvas.
	####

	# Hide other toplevel windows
	::FindingChart::HideToplevels 1

	# Required final size of finding chart
	# (dimensions must both reach the following minimums)
	set required_fc_height 1200
	set required_fc_width 1200

	# If finding chart is already large enough, set the flag
	# to OK.
	if { [expr $viewable_image_height >= $required_fc_height] && \
		 [expr $viewable_image_width >= $required_fc_width] } {

	    set mag_ok true
	    
	} else {

	    set mag_ok false

	}

	# Counter: how many times do we zoom in
	set mag 0

	# Loop until zoomed image is large enough
	while { $mag_ok == "false" } {

	    # Zoom in
	    ::FindingChart::ZoomIn

	    # Increment "zoom counter"
	    incr mag

	    # Get image corners in canvas coordinates
	    $image convert coords $llx $lly image x1 y2 canvas
	    $image convert coords $urx $ury image x2 y1 canvas

	    # Compute new image dimensions
	    set zoom_height [expr $y2 - $y1]
	    set zoom_width [expr $x2 - $x1]

	    # Test if image is large enough
	    if { $zoom_height >= $required_fc_height && \
		     $zoom_width >= $required_fc_width } {
		set mag_ok true
	    }
	}

	update
	
	# Final size of finding chart (round off image dimensions)
	set fc_height [expr int($zoom_height)]
	set fc_width [expr int($zoom_width)]
	
	# Number of "columns" and "rows" to scan,
	# in units of visible image size
	set loop_x [expr int(ceil(double($zoom_width)/$cw))]
	set loop_y [expr int(ceil(double($zoom_height)/$ch))]

	# Save scrollregion...
	set scrollregionSAVED [$canvas cget -scrollregion]

	# Set scrollregion to the size of the finding chart
	$canvas configure -scrollregion [list $x1 $y1 [expr $x1 + $fc_width] [expr $y1 + $fc_height]]

	# Get scrollregion size
	lassign [$canvas cget -scrollregion] c_left c_top c_right c_bottom

	# Canvas coordinates of the topleft corner of the
	# finding charts
	set x_topleft [expr ($x1-$c_left)]
	set y_topleft [expr ($y1-$c_top)]

	# Create empty photo image
	image create photo fc_image -height $fc_height -width $fc_width

	# Raise main window on top of others to avoid obscuration
	raise $main

	set snapshot_OK 1

	# Loop on "columns", i.e. x-axis
	for {set i 0} {$i < $loop_x} {incr i} {

	    if { $snapshot_OK == 0 } { break }
	    
	    # Compute "x-fraction" to move canvas to the chosen column
	    set x [expr $x_topleft + $i * $cw]
	    set xfraction [expr double($x)/($c_right-$c_left)]

	    if { $i == [expr $loop_x - 1] } { set xfraction 1 }

	    # Move canvas to column #i
	    $canvas xview moveto $xfraction

	    # Loop on "rows", i.e. y-axis
	    for {set j 0} {$j < $loop_y} {incr j} {

		# Compute "y-fraction" to move canvas to chosen row
		set y [expr $y_topleft + $j * $ch]
		set yfraction [expr double($y)/($c_bottom-$c_top)]

		if { $j == [expr $loop_y - 1] } { set yfraction 1 }

		# Move canvas to row #j
		$canvas yview moveto $yfraction

		update
		
		if { [catch {image create photo canvas_grab -format window -data $canvas}] } {
		    
		    tk_dialog .fcdia "Snapshot failed" \
			"Cannot create snapshot. Please check\nthat image window is not partially\nobscured." \
			error 0 OK
		    set snapshot_OK 0
		    break
		}

		if { $j == [expr $loop_y - 1] } {

		    set y1_from [expr $loop_y * $ch - $fc_height]
		    set y2_from $ch

		    set y1_to [expr int($j * $ch)]
		    set y2_to $fc_height

		} else {

		    set y1_from 0
		    set y2_from $ch

		    set y1_to [expr int($j * $ch)]
		    set y2_to [expr int(($j+1)*$ch)]
		}

		if { $i == [expr $loop_x - 1] } {

		    set x1_from [expr $loop_x * $cw - $fc_width]
		    set x2_from $cw

		    set x1_to [expr int($i * $cw)]
		    set x2_to $fc_width

		} else {

		    set x1_from 0
		    set x2_from $cw

		    set x1_to [expr int($i * $cw)]
		    set x2_to [expr int(($i+1)*$cw)]
		}
 
		fc_image copy canvas_grab \
		    -from $x1_from $y1_from $x2_from $y2_from \
		    -to $x1_to $y1_to $x2_to $y2_to

		image delete canvas_grab
	    }
	}

	# Restore scroll region
	$canvas configure -scrollregion $scrollregionSAVED

	if { $snapshot_OK } {
	    
	    # TODO: catch error if writing not possible
	    fc_image write -format "jpeg -quality 60" $file

	}

	# Restoring previous state...
	::FindingChart::Zoom [expr -1 * $mag]

	# Recenter image to original position
	$image convert coords $llx $ury image x1 y1 canvas

	lassign [$canvas cget -scrollregion] c_left c_top c_right c_bottom

	set xfraction [expr double($x1-$c_left)/($c_right-$c_left)]
	set yfraction [expr double($y1-$c_top)/($c_bottom-$c_top)]

	$canvas xview moveto $xfraction
	$canvas yview moveto $yfraction

	# Restore other toplevel windows
	::FindingChart::HideToplevels 0

	# Cleaning up...
	image delete fc_image

	return


    }
}


## Save the Finding Chart to a Postscript File
proc ::FindingChart::PrintFindingChart {} {

    variable main
    variable image
    variable canvas
    
    variable runID
    variable startDir
    variable plotParams

    global env
    
    # If finding chart has not been drawn yet, exit the procedure
    if { $plotParams(draw) == "false" } {
        tk_dialog .fcdia "Missing info" \
            "You have to draw the Finding Chart\nbefore printing it !" \
            error 0 OK
        return
    }
    
    # Returns the value of the keyword 'OBJECT'
    # from the FITS header of the image
    set object [$image object]

    # Name of the loaded file (does this work?)
    set file [file tail [$main cget -file]]

    # WCS coordinates of the center of the image
    set center [$image wcscenter]

    # Username
    set user [id user]

    # Name of the application (Skycat)
    set app [lindex [winfo name .] 0]

    # Timestamp
    set date [clock format [clock seconds] -format {%b %d, %Y at %H:%M:%S}]

    # If print dialog already exists, bring it to front
    # and configure its default values with the above info
    if { [winfo exists $main.fcPrint] == 1 } {

 	wm deiconify $main.fcPrint
	$main.fcPrint configure \
	    -image [$main component image] \
	    -show_footer 1 \
	    -transient 1 \
	    -whole_canvas 1 \
	    -top_left "ESO\n$object" \
	    -top_right "$file\n$center" \
	    -bot_left "$user/$app" \
	    -bot_right "$date"

	update

    } else {

	# Build print dialog: use RtdImagePrint widget
	RtdImagePrint $main.fcPrint \
	    -image [$main component image] \
	    -show_footer 1 \
	    -transient 1 \
	    -whole_canvas 1 \
	    -top_left "ESO\n$object" \
	    -top_right "$file\n$center" \
	    -bot_left "$user/$app" \
	    -bot_right "$date"

	update
    }

    # If FC_DIR environment variable is set, use it as default directory
    if { [info exists env(FC_DIR)] == 1 } {

	set fcpath $env(FC_DIR)

    } else {

	# Else, use previously stored value (startDir variable)
	set filename ""
 	catch {set filename [$main.fcPrint.print_to_file get]}

	set fcpath [file dirname $filename]
	if { $fcpath == "." } {
	    set fcpath $startDir
	}
    }

    # Remove parentheses from the run ID
    regsub -all {[()]} $runID "" runId
    
    # List of all previously produced PostScript file
    # in the chosen directory, fitting the pattern <runID>.ps
    set fcList [glob -nocomplain "$fcpath/$runId.*.ps"]

    # Determine next index to use
    if { [llength $fcList] == 0 } {
        set index 1
    } else {
        set lastFile [file rootname [file tail [lindex $fcList end]]]
        if { [regexp {[0-9][0-9][0-9]$} $lastFile index] != 1 } {
            set index 1
        } else {
	    set index [string trimleft $index "0"]
	    set index [expr $index + 1]
	}
    }
    
    set index [format "%03d" $index]

    # Default filename for the PostScript file
    set filename "$fcpath/$runId.$index.ps"

    # Show default filename in print dialog
    $main.fcPrint.print_to_file configure -value $filename

    # Trick: reset value to 0 to avoid that it is shown
    # when we use print dialog next time
    $main.fcPrint.print_to_file configure -value ""

}

## Plot the run information on the finding chart
proc ::FindingChart::PlotRunInfo {} {

    variable plotParams
    variable canvas
    variable imageCtrl

    variable runID
    variable piName
    variable obName
    variable imgBand
    variable textColor
    variable textFont

    # CanvasDraw object, used to manage graphics
    set draw [$imageCtrl component draw]

    # Put runId, PI name, OB name and band on canvas

    # Create text object for run ID
    $canvas create text $plotParams(xInfo_c) $plotParams(yInfo_c) \
	-text "$runID" \
	-fill $textColor \
	-font $textFont \
	-tags {fcinfo runID userInfo fctext} \
	-justify left -anchor $plotParams(anchorInfo)

    # get bounding box coordinates for the above text element
    lassign [$canvas bbox runID] tlx tly lrx lry
    
    # Create text object for PI name
    $canvas create text $plotParams(xInfo_c) $lry \
	-text "$piName" \
	-fill $textColor \
	-font $textFont \
	-tags {fcinfo piName userInfo fctext} \
	-justify left -anchor $plotParams(anchorInfo)

    # get bounding box coordinates for the above text element
    lassign [$canvas bbox piName] tlx tly lrx lry

    # Create text element for OB name(s)
    $canvas create text $plotParams(xInfo_c) $lry \
	-text "$obName" \
	-fill $textColor \
	-font $textFont \
	-tags {fcinfo userInfo fctext} \
	-justify left -anchor $plotParams(anchorInfo)

    # get bounding box coordinates for the above text elements
    lassign [$canvas bbox userInfo] tlx tly lrx lry

    # Create text element for image band
    $canvas create text $plotParams(xInfo_c) $lry \
	-text "band: $imgBand" \
	-fill $textColor \
	-font $textFont \
	-tags {fcinfo userInfo fctext} \
	-justify left -anchor $plotParams(anchorInfo)

    # get bounding box coordinates for the above text elements
    lassign [$canvas bbox userInfo] tlx tly lrx lry

    # Re-adjustment of position: 

    # If lower right corner outside of canvas (on x- or/and y-axis),
    # bring the elements back into the canvas
    if { $lrx > $plotParams(x2_c) } {
	$canvas move userInfo [expr $plotParams(xInfo_c) - $lrx] 0
    } 

    if { $lry > $plotParams(y2_c) } {
	$canvas move userInfo 0 [expr $plotParams(yInfo_c) - $lry] 
    }

    # Same thing with top-left corner
    if { $tly < $plotParams(y1_c) } {
	$canvas move userInfo 0 [expr $plotParams(yInfo_c) - $tly]
    }

    if { $tlx < $plotParams(x1_c) } {
	$canvas move userInfo [expr $plotParams(xInfo_c) - $tlx] 0
    }

    # Set up the canvas object bindings for all objects tagged with "userInfo"
    $draw add_object_bindings userInfo

    # Additional binding: when drawn objects are released
    # after moving them, deselect them automatically
    $canvas bind userInfo <ButtonRelease-1> \
 	"+ [code eval $draw deselect_objects]"

}

## Plot compass on canvas (arrows showing N-E directions)
proc ::FindingChart::PlotCompass { {anchor ""} } {

    variable plotParams
    variable textColor
    variable textFont
    variable textFont2

    variable canvas
    variable imageCtrl
    variable image

    set draw [$imageCtrl component draw]

    # Set size of compass to percentage of image size:
    # it should be 20% of the smallest side of the image

    # size in arcsec
    set compass_size [expr [::math::min $plotParams(visImageHeight) $plotParams(visImageWidth)] * .2]

    # size in degrees
    set compass_size_deg [expr $compass_size / 3600.]

    # Get equinox from image
    set equinox [$image wcsequinox]

    # String for coordinate conversion
    set deg_eq "deg $equinox"

    # By default, first plot arrow in the middle of the visible image (why?)
    if { $plotParams(origCompassX_c) == "undef" } {
	set xc [expr ($plotParams(x2_c) + $plotParams(x1_c))/2.]
	set yc [expr ($plotParams(y2_c) + $plotParams(y1_c))/2.]
    } else {
	
	# ... except if its position has been defined interactively
	# by the user.
	set xc $plotParams(origCompassX_c)
	set yc $plotParams(origCompassY_c)
    }

    # Get coords of compass origin in ra/dec system
    # (take into account equinox of image)
    $image convert coords $xc $yc canvas ra0 dec0 $deg_eq	

    # Compute end points of compass in ra/dec system
    # East arrow: RA = RA_origin + compass_size/cos(DEC_origin)
    set ra1 [expr $ra0 + $compass_size_deg/cos( ($dec0/180.) * 3.14159 )]
    if { $ra1 < 0 } {
	set ra1 [expr 360 + $ra1]
    }

    # North arrow: DEC = DEC_origin + compass_size
    set dec1 [expr $dec0 + $compass_size_deg]
    if { $dec1 >= 90 } {
	set dec1 [expr 180 - $dec1]
    }

    # Get _canvas_ coords of end point of East arrow
    set retVal [catch {$image convert coords $ra1 $dec0 $deg_eq arrowEastX_c arrowEastY_c canvas}]

    # If one tries to draw the arrows outside of the image, reset user chosen
    # coordinates and exit procedure
    # NB: DOES NOT SEEM TO WORK !!!!

    if { $retVal != 0 } {
	tk_dialog .fcdia "Cannot draw compass arrows" \
	    "ERROR: cannot draw compass arrows. You are trying to draw\nthe arrows outside of the image" \
	    error 0 OK
	set plotParams(userChosenFlag) "false"
	return
    }

    # Get _canvas_ coords of end point of North arrow
    $image convert coords $ra0 $dec1 $deg_eq arrowNorthX_c arrowNorthY_c canvas

    # Draw compass line on canvas
    $canvas create line $arrowEastX_c $arrowEastY_c \
	$xc $yc \
	$arrowNorthX_c $arrowNorthY_c \
	-fill $textColor -width 2 -arrow both \
	-tags {fcinfo compass}

    # Position of the 'N'
    set northX_c [expr $arrowNorthX_c + 15 * ($arrowEastX_c - $xc)/abs($arrowEastX_c - $xc)]
    set northY_c $arrowNorthY_c

    # Position of the 'E'
    set eastX_c $arrowEastX_c
    set eastY_c [expr $arrowEastY_c + 15 * ($arrowNorthY_c - $yc)/abs($arrowNorthY_c - $yc)]

    # Create text ('N') on the canvas
    $canvas create text $northX_c $northY_c \
	-text "N" -fill $textColor -font $textFont2 \
	-tags {fcinfo compass fctext}

    # Create text ('E') on the canvas
    $canvas create text $eastX_c $eastY_c \
	-text "E" -fill $textColor -font $textFont2 \
	-tags {fcinfo compass fctext}

    # Get bounding box for whole compass (arrows + letters)
    lassign [$canvas bbox compass] tlx tly lrx lry

    #
    # Depending on anchor direction, move compass to final position
    # (move canvas items with tag 'compass', leave a 15-pixel margin
    #  from the edges of the image)

    if { $anchor == "br" } {

	# Bottom-right anchoring
	$canvas move compass [expr $plotParams(x2_c) - 15 - $lrx] [expr $plotParams(y2_c) - 15 - $lry]

    } elseif { $anchor == "bl" } {

	# Bottom-left anchoring
	$canvas move compass [expr $plotParams(x1_c) + 15 - $tlx] [expr $plotParams(y2_c) - 15 - $lry]

    } elseif { $anchor == "tl" } {

	# Top-left anchoring
	$canvas move compass [expr $plotParams(x1_c) + 15 - $tlx] [expr $plotParams(y1_c) + 15 - $lry]

    } elseif { $anchor == "tr" } {

	# Top-right anchoring
	$canvas move compass [expr $plotParams(x2_c) - 15 - $tlx] [expr $plotParams(y1_c) + 15 - $lry]

    } else {

	# Move compass if it is not entirely inside visible canvas
	if { $lrx > $plotParams(x2_c) } {
	    $canvas move compass [expr $plotParams(x2_c) - $lrx - 15] 0
	} 
	
	if { $lry > $plotParams(y2_c) } {
	    $canvas move compass 0 [expr $plotParams(y2_c) - $lry - 15] 
	}
	
	if { $tly < $plotParams(y1_c) } {
	    $canvas move compass 0 [expr $plotParams(y1_c) - $tly + 15]
	}
	
	if { $tlx < $plotParams(x1_c) } {
	    $canvas move compass [expr $plotParams(x1_c) - $tlx + 15] 0
	}
	
    }

    # Set up the canvas object bindings for all objects tagged with "compass"
    $draw add_object_bindings compass

    # Additional binding: when drawn objects are released
    # after moving them, deselect them automatically
    $canvas bind compass <ButtonRelease-1> \
 	"+ [code eval $draw deselect_objects]"

}

## Plot scale on canvas
proc ::FindingChart::PlotScale {} {

    variable plotParams
    variable textColor
    variable textFont
    variable textFont2

    variable image
    variable imageCtrl
    variable canvas

    set draw [$imageCtrl component draw]

    ### Scale size in seconds: 1/4 of the largest side of the image
    set scaleSize_s [expr floor([::math::max $plotParams(visImageHeight) $plotParams(visImageWidth)]/4.)]

    # Predefined list of scales (in arcseconds) and corresponding labels	    
    set scaleList { {1 "1\""} {5 "5\""} {10 "10\""} \
			{20 "20\""} {30 "30\""} {40 "40\""} \
			{60 "1'"} {120 "2'"} }


    #
    # Find predefined scale which is _closest_ to the pre-computed
    # scale size
    #

    # Arbitrarily high value used for first comparison
    set dmin 1000
    
    # Loop over predefined scales to find closest
    foreach sc $scaleList {

	set scVal [lindex $sc 0]
	set d [expr abs($scVal-$scaleSize_s)]

	if { $d < $dmin } {
	    set dmin $d
	    set chosenScale $scVal
	    set chosenLabel [lindex $sc 1]
	}
    }

    # Compute pixel length of scale (use image pixscale + zoom factor)
    set lengthScale \
	[expr $chosenScale / $plotParams(pixscale) * $plotParams(scale)]
    
    # Coordinates of end of the scale
    set endScaleX_c $plotParams(origScaleX_c)
    set endScaleY_c [expr $plotParams(origScaleY_c) - $lengthScale]

    # Coordinates for scale label
    set origScaleLabelX_c [expr $plotParams(origScaleX_c) + 10]
    set origScaleLabelY_c [expr $plotParams(origScaleY_c) - $lengthScale/2]

    # Create main line for scale
    $canvas create line $plotParams(origScaleX_c) $plotParams(origScaleY_c) \
	$endScaleX_c $endScaleY_c \
	-width 3 -fill $textColor -tags {fcinfo scale}  

    # Ends of scale line
    $canvas create line [expr $plotParams(origScaleX_c) - 5] \
	$plotParams(origScaleY_c) \
	[expr $plotParams(origScaleX_c) + 5] \
	$plotParams(origScaleY_c) \
	-width 3 -fill $textColor -tags {fcinfo scale}  

    $canvas create line [expr $endScaleX_c - 5] \
	$endScaleY_c \
	[expr $endScaleX_c + 5] \
	$endScaleY_c \
	-width 3 -fill $textColor -tags {fcinfo scale}  

    # Scale label
    $canvas create text $origScaleLabelX_c $origScaleLabelY_c \
	-text $chosenLabel -fill $textColor -font $textFont2 \
	-tags {fcinfo scale fctext} -anchor w

    # Get bounding box for scale
    lassign [$canvas bbox scale] tlx tly lrx lry

    # Move scale if it is not entirely inside visible canvas
    if { $lrx > $plotParams(x2_c) } {
	$canvas move scale [expr $plotParams(origScaleX_c) - $lrx] 0
    } 

    if { $lry > $plotParams(y2_c) } {
	$canvas move scale 0 [expr $plotParams(origScaleY_c) - $lry] 
    }

    if { $tly < $plotParams(y1_c) } {
	$canvas move scale 0 [expr $plotParams(origScaleY_c) - $tly]
    }

    if { $tlx < $plotParams(x1_c) } {
	$canvas move scale [expr $plotParams(origScaleX_c) - $tlx] 0
    }

    # Set up the canvas object bindings for all objects tagged with "scale"
    $draw add_object_bindings scale

    # Additional binding: when drawn objects are released
    # after moving them, deselect them automatically
    $canvas bind scale <ButtonRelease-1> \
 	"+ [code eval $draw deselect_objects]"
}


## Plot box on canvas
proc ::FindingChart::PlotBox {} {

    variable plotParams
    variable textColor

    variable image
    variable imageCtrl
    variable canvas

    variable box_Xdim
    variable box_Ydim
    variable box_angle
    variable box_counter

    variable box_RA
    variable box_DEC

    # If finding chart has not been drawn yet, exit the procedure
    if { $plotParams(draw) == "false" } {
        tk_dialog .fcdia "Cannot draw box" \
            "You have to draw the Finding Chart\nbefore drawing any box !" \
            error 0 OK
        return
    }


    # Check that both dimensions of the box are non-zero
    if { ($box_Xdim == 0. || $box_Ydim == 0.) && \
	     $plotParams(drawbox) == "false" } {

	tk_dialog .dia "Zero dimension" "At least one of the sides of the box is null." \
	    warning 0 OK
	return

    }

    # If no angle is given, default to zero
    if { $box_angle == "" || ![regexp {^\-?[0-9]+(\.[0-9]*)?$} $box_angle] } {
	set box_angle 0
    }

    set draw [$imageCtrl component draw]

    # Get visible size of canvas
    set cvs_vis_height [winfo height $canvas]
    set cvs_vis_width [winfo width $canvas]

    # Get equinox from image
    set equinox [$image wcsequinox]

    # Strings for coordinate conversion
    set deg_eq "deg $equinox"
    set wcs_eq "wcs $equinox"

    # If coordinates are given for the center of the box,
    # check that *both* are given
    if { $box_RA == "" ^ $box_DEC == "" } {

	tk_dialog .dia "Missing coordinate" "You have to specify both RA and DEC" \
	    warning 0 OK
	return
	
    }

    # If coordinates are given, check that they are
    # in the right format
    if { $box_RA != "" && $box_DEC != "" } {
	
	if { ![regexp {^([0-1][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9](\.[0-9]*)?$} $box_RA] || \
		 ![regexp {^[\-+]?([0-8][0-9]|90):[0-5][0-9]:[0-5][0-9](\.[0-9]*)?$} $box_DEC] } {
	    
	    tk_dialog .dia "Wrong format" "Wrong format for RA or DEC" \
		warning 0 OK
	    return
	    
	}

	# Convert coordinates to decimal format
	$image convert coords \
	    $box_RA $box_DEC $wcs_eq ra0 dec0 $deg_eq


    } else {

	# If coordinates not given, use the center of the visible image
	$image convert coords \
	    [expr $cvs_vis_width / 2] [expr $cvs_vis_height / 2] screen \
	    ra0 dec0 $deg_eq
	
	# Convert back coordinates to DD:MM:SS for display in dialog box
	$image convert coords \
	    $ra0 $dec0 $deg_eq box_RA box_DEC $wcs_eq

    }

    # Determine center of box in canvas coordinates
    $image convert coords \
	$ra0 $dec0 $deg_eq x0 y0 canvas

    # Box size in degrees (dimensions are given in arcsec in the plug-in)
    set box_width_deg  [expr $box_Xdim / 3600.]
    set box_height_deg [expr $box_Ydim / 3600.]

    #
    # Determine angle between -Y-axis and N-axis
    #

    # Reminder: canvas coordinates
    #
    #   +-----> X
    #   |
    #   |
    #   V
    #   Y
    #
    # (origin = top-left corner of image)
    #
    # We use -Y below (instead of Y) as a reference to have a
    # orthonormal system of coordinates (X,-Y)

    # Use the longer side of the box for the sample vectors
    set length [::math::max $box_Xdim $box_Ydim]

    # Box center = point O(x0,y0)

    # Take a point along N-axis at 'length' arcseconds of
    # box center: point A(xa,ya)
    $image convert coords \
	$ra0 [expr $dec0 + $length / 3600.] $deg_eq \
	xa ya canvas

    # Take a point along E-axis at 'length' arcseconds of
    # box center: point D(xd,yd)
    $image convert coords \
	[expr $ra0 + $length / 3600.] $dec0 $deg_eq \
	xd yd canvas

    # Pixel scale in pixels/arcsec
    set pixScale [expr $plotParams(scale) / $plotParams(pixscale) ]

    # Take a point along -Y-axis at 'length' arcseconds of
    # box center: point B(xb,yb)
    set xb $x0 
    set yb [expr $y0 - $length * $pixScale]

    # Compute angle AOB between North and -Y vertical axis
    # (convert to degrees)
    set offset_angle [::math::angleABAC $x0 $y0 $xa $ya $xb $yb]
    set offset_angle [expr $offset_angle *180. / 3.14159]

    # We compare the position of the vector along N w/r to the
    # center of the box to determine the sign of the angle
    if { $xa < $x0 } { set offset_angle [expr -1. * $offset_angle] }

    # Store this angle for later use (see getBoxCoords() )
    set plotParams(offset_angle) $offset_angle 

    # Determine the N-E orientation on the image: we want to always
    # count positive angle from N --> E.
    # We compute the sign of the vector product OA ^ OD. If positive,
    # then N-->E goes clockwise on screen (same as X-->Y), if negative
    # then N-->E goes counterclockwise, i.e. opposite of X-->Y

    set NE_orientation [expr ($xa-$x0)*($yd-$y0) - ($xd-$x0)*($ya-$y0)]
    set NE_orientation [expr $NE_orientation/abs($NE_orientation)]

    # Store this angle for later use (see getBoxCoords() )
    set plotParams(NE_orientation) $NE_orientation

    # We use the NE orientation to apply a sign correction to the angle
    # given by the user
    if { $box_angle == "" } {
	set angle 0
    } else {
	set angle [expr $NE_orientation * $box_angle]
    }

    # Coordinates of the 4 corners of the box
    # 
    set x1 [expr $x0 - $pixScale * $box_Xdim/2.]
    set y1 [expr $y0 - $pixScale * $box_Ydim/2.]
    set x2 [expr $x0 + $pixScale * $box_Xdim/2.]
    set y2 [expr $y0 - $pixScale * $box_Ydim/2.]
    set x3 [expr $x0 + $pixScale * $box_Xdim/2.]
    set y3 [expr $y0 + $pixScale * $box_Ydim/2.]
    set x4 [expr $x0 - $pixScale * $box_Xdim/2.]
    set y4 [expr $y0 + $pixScale * $box_Ydim/2.]

    # Create a line representing the box (we do not use a
    # rectangle item because we could not rotate it)
    # Tags associated to the box: fcbox<i> objects fcBoxInfo
    set boxId [$canvas create line \
		   $x1 $y1 $x2 $y2 $x3 $y3 $x4 $y4 $x1 $y1 \
		   -width 2 -fill $textColor \
		   -tags [list fcbox$box_counter objects fcBoxInfo] ]

    # We then rotate the canvas item of the appropriate amount
    # around its center. We sum up the angle given by the user
    # and the offset angle of the North w/r to the vertical axis
    # of the canvas
    # NB: RotateItem rotates objects clockwise for a positive angle

    ::math::RotateItem $canvas $boxId $x0 $y0 [expr $angle + $offset_angle]

    # Store the coordinates of the rotated box
    set plotParams(coords_$boxId) [$canvas coords $boxId]

    # We check that the (rotated) box is at least partly visible on screen
    # (we count the number of corners which are outside of the visible
    # area)

    # Number of not visible corners
    set count 0

    # Loop on 4 corners
    for {set k 1} {$k <= 4} {incr k} {
	
	# Get coordinates of corner #k
	set x [lindex $plotParams(coords_$boxId) [expr 2 * ($k -1)]]
	set y [lindex $plotParams(coords_$boxId) [expr 2 * $k - 1]]

	# Convert to screen coordinates (visible canvas area)
	$image convert coords $x $y canvas x_s y_s screen

	# If the corner is outside of visible canvas area, count it
	# as invisible
	if { $x_s < 0 || $y_s < 0 || \
		 $x_s > $cvs_vis_width || $y_s > $cvs_vis_height } {
	    
	    incr count
	}
	
    }

    # If all 4 corners are outside of the visible image, cannot draw:
    # Issue a warning and delete canvas item.
    if { $count == 4 } {
	$canvas delete fcbox$box_counter
	tk_dialog .dia "Out of bounds" "Box cannot be drawn outside of visible image" \
	    warning 0 OK
	return
    }
    
    # Store coordinates, size and position angle of box
    # (will be used to update the dialog when selecting
    # one from multiple boxes)
    set plotParams(box$boxId,RA) $box_RA
    set plotParams(box$boxId,DEC) $box_DEC
    set plotParams(box$boxId,width) $box_Xdim
    set plotParams(box$boxId,height) $box_Ydim
    set plotParams(box$boxId,posang) $box_angle
    
    # Set up the canvas object bindings for all objects tagged with "fcboxXX"
    $draw add_object_bindings fcbox$box_counter

    # Add a command which will be called when the box is moved or rotated or resized
    $draw add_notify_cmd $boxId [code eval ::FindingChart::getBoxCoords $boxId] 1
    
    # Add a command to update box characteristics in dialog box
    # when selecting one of several boxes
    $::FindingChart::canvas bind $boxId <Button-1> \
	"+[code eval ::FindingChart::updateBoxData $boxId]"

    # Increment the box counter
    incr box_counter

    # A FC box has been drawn on canvas
    set plotParams(drawbox) "true"

}

# Procedure used as a callback when selecting one of the boxes drawn on canvas,
# so as to update the input fields of the dialog box.
proc ::FindingChart::updateBoxData { boxID } {

    variable plotParams
    variable box_RA
    variable box_DEC
    variable box_Xdim
    variable box_Ydim
    variable box_angle

    set box_RA $plotParams(box$boxID,RA)
    set box_DEC $plotParams(box$boxID,DEC)
    set box_Xdim $plotParams(box$boxID,width)
    set box_Ydim $plotParams(box$boxID,height)
    set box_angle $plotParams(box$boxID,posang)
    
    return

}

# Callback procedure, used when one moves, or resizes, or rotates
# a box drawn on canvas
proc ::FindingChart::getBoxCoords { boxID action } {

    variable plotParams
    variable image
    variable imageCtrl
    variable canvas

    variable box_RA
    variable box_DEC
    variable box_Xdim
    variable box_Ydim
    variable box_angle

    set draw [$imageCtrl component draw]

    # Case of a box moved: update coordinates of center in dialog box
    if { $action == "move" } {

	# Get coordinates of box (box corners)
	set coords [lrange [$canvas coords $boxID] 0 7]
	
	# Compute center of box (canvas coordinates)
	set xc [expr ([lindex $coords 0] + [lindex $coords 2] + \
			  [lindex $coords 4] + [lindex $coords 6]) / 4.]
	
	set yc [expr ([lindex $coords 1] + [lindex $coords 3] + \
			  [lindex $coords 5] + [lindex $coords 7]) / 4.]
	
	# Get equinox from image
	set equinox [$image wcsequinox]
	set wcs_eq "wcs $equinox"
	
	# Convert center to sky coordinates, update dialog (box_RA and box_DEC
	# are tied to the input fields)
	$image convert coords $xc $yc canvas box_RA box_DEC $wcs_eq

	# Store new coordinates of box, and coords of center on sky
	set plotParams(coords_$boxID) [$canvas coords $boxID]
	set plotParams(box$boxID,RA) $box_RA
	set plotParams(box$boxID,DEC) $box_DEC
    
    } elseif { $action == "resize" } {

	# Prevent any resizing (leads to deformations of the
	# rectangle, because it is drawn as a 'polygon' item
	# on the canvas to allow rotation).

	# Deselect the box
	$draw deselect_object $boxID

	# Change cursor back to arrow
	$canvas config -cursor left_ptr

	# Restore the box coordinates from the global array
	$canvas coords $boxID $plotParams(coords_$boxID)

    } elseif { $action == "rotate" } {

	# Pixel scale in pixels/arcsec
	set pixScale [expr $plotParams(scale) / $plotParams(pixscale) ]

	# Get coordinates of all box corners
	set coords [lrange [$canvas coords $boxID] 0 7]
	
	# Center of box : point O(x0,y0)
	set x0 [expr ([lindex $coords 0] + [lindex $coords 2] + \
			  [lindex $coords 4] + [lindex $coords 6]) / 4.]
	
	set y0 [expr ([lindex $coords 1] + [lindex $coords 3] + \
			  [lindex $coords 5] + [lindex $coords 7]) / 4.]

	# Point C(xc,yc): middle of side point1-point2 ("top" side)
	set xc [expr ([lindex $coords 0] + [lindex $coords 2]) / 2.]
	set yc [expr ([lindex $coords 1] + [lindex $coords 3]) / 2.]

	# Point B(xb,yb): point in the -Y-axis direction
	set xb $x0
	set yb [expr $y0 - [::math::max $box_Xdim $box_Ydim] * $pixScale]

	# Compute angle COB between top of box and -Y vertical axis
	# (convert to degrees)
	set angle [::math::angleABAC $x0 $y0 $xc $yc $xb $yb]
	set angle [expr $angle *180. / 3.14159]

	# We compare the position of the vector along top of the
	# box w/r to the center of the box to determine the sign of the angle
	if { $xc < $x0 } { set angle [expr -1. * $angle] }

	# We then take into account the N-E orientation previously determined
	# to always count positive PA angle from N towards E,
	# and we add the offset angle between N-axis and the top of the image.
	set total_angle [expr $plotParams(NE_orientation) * $angle + $plotParams(offset_angle)]

	# Finally, as a convention we count the angle from 0 to 360 (we don't
	# show negative angles)
	if { $total_angle < 0 } { set total_angle [expr 360. + $total_angle] }

	# Format the angle for display in the dialog box
	set box_angle [format "%.1f" $total_angle]

	set plotParams(coords_$boxID) [$canvas coords $boxID]
	set plotParams(box$boxID,posang) $box_angle

    } else {

	return

    }

}

# Displays a help window describing the functions of the FC plug-in
#
proc ::FindingChart::DisplayHelp { } {

    if {[winfo exists .fcHelpText] == 1} {
	.fcHelpText center_window
	return
    }

    option add *Dialog.msg.font {Helvetica 10}
    
    set title "Help for the Finding Chart plug-in"

    set contents "Run ID: must be specified in the form 081.A-0123(B), or 081.A-0123(A-C)
        if the chart applies to several runs.

PI Name: name of the Principal Investigator, as given in the original proposal.

List of OBs: give the name(s) of the OB(s) to which this chart corresponds.

Band/Wavelength: indicate the band in which the underlying image has been taken.

Location of info: choose the position of the above info on the chart. Other
                  information (scale, N-E arrows) is drawn in the other corners.
                  You can also specify individual positions for each element
                  by selecting 'user-chosen'.

Text color: change the color of the above information on the chart.
            Choose the color that allows best legibility.

Draw box: allows to draw one or several boxes on the chart, of specified
          dimensions (width x height). The optional position angle is counted
          positively from North towards East. If no specific coordinates are
          given, the box will be drawn at the centre of the visible image.
          You must draw the finding chart before being able to draw any box.
          Functionalities:
          - Boxes can be selected and moved after being drawn (this will update
            the coordinates of the center of the box).
          - A box can be deleted by selecting it and hitting the Delete key.
          - Once drawn, a box cannot be resized.
          - It is possible to rotate a box by holding the Ctrl key and the 
            right mouse button on one of the handles that appear when 
            selecting the box. When releasing the mouse button, the new position
            angle appears in the Finding Chart dialog window.

Draw: draw (or redraw) the provided information on the chart.

Save to jpeg: creates a Jpeg snapshot of the Finding Chart, which you can append
              to your OB inside the P2PP tool.

Print: send your finding chart to the printer.

Reset coords: if you have selected 'user-chosen' for the location of the info,
              this button allows you to re-specify the position of all elements
              when clicking on the 'Draw' button.

Clear: delete all finding chart information.

(Note: the Finding Chart dialog can be closed by simply getting rid of the
window -- generally by hitting the cross in the top bar.)"

    set w [TextDialog .fcHelpText \
	       -text $title \
	       -title "FC plug-in help" \
	       -contents $contents \
	       -modal 0 -buttons Dismiss -textheight 40 ]

    update
    $w center_window
    [.fcHelpText component button0] configure -command {wm withdraw .fcHelpText}
    wm resizable $w 0 0
    $w activate
    

    return

}