#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: SkyCat_plugin.tcl 237649 2013-01-24 03:16:13Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2013/01/23  PPRS-23355: FIMS product package
#                       - Test env. var. HOST before usage. For non VLT systems.
# pbaksai   2012/12/28  PPRS-23355: FIMS product package
#                       - More options for debug and trace flags:
#                         OFF, NO, 0 mean FALSE
# pbaksai   2011/05/31  PPRS-24729: Fixed bugs
#                       - Ensure plugin is executed just once, and
#                         that it doesn't add instrument menu to any
#                         new SkyCat instance created with Menu/New
#                         Window because FORS class can have only one
#                         instance.
# pbaksai   2010/12/02  PPRS-23355: Improve pre-image filename management
#                       - Do not delete mosaiced pre-image since its
#                         name is recorded in the .fims file.
#                       - Preserve the original pre-image filename as part of
#                         the fsmosaic output filename. No fsmosaic_*.fits
#                         randomly generated names anymore.
#                       - If image has .CHIP? as part of the filename, 
#                         then replace it with .MOSAIC in fsmosaic output.
#                       - Delete class tracer_class before re-creating it.
# pbaksai   2010/11/16  PPRS-23355: Prepare for automated testing 
#                       - Updated debug and trace logic
#                       - Added TkReplay socket support
# pbaksai   2009/08/17  PPRS-27204: Suppress ERROR messages from fsmosaic
# pbaksai   2009/06/02  PPRS-27204: Renamed fors_menu to fims_menu
# pbaksai   2008/06/23  PPRS-27204: All menu management moved to fors_menu.tcl
# pbaksai   2008/06/19  PPRS-27204: Adapted menu and buttons state changes with
#                                   item index names.
# pbaksai   2007/12/02  PPRS-24729: Fixed more bugs found
#                       - Initialization of mw_list variable
#                       - Message "quitting fims application ..." if stdout gone
#                       - Local class tracer improved
# pbaksai   2007/11/22  PPRS-24729: Destroy slit parameter editors when leaving
#                       the mode or deleting the mask
# pbaksai   2006/04/04  PPRS-23198: Deactivated Coll_HR for FORS1 MOS mode also
# pbaksai   2006/03/21  Added debug and trace support
# aaguayo     28/10/03  PPRS 10127 - fsmosaic considered for both FORSes
# mkiekebu    06/11/01  Added rtd plugin to overload File-Exit
# mkiekebu    05/19/01  Added rtd plugin to overload File-Open
# mkiekebu    31/03/01  Removed bug (last modification)
# aaguayo     14/03/01  fims 2.28 (use of SKYCAT_VER variable) 
# hummel      13/08/99  Created
#
#*******************************************************************************
#   NAME
#      SkyCat_plugin - SkyCat startup file for plug-ins
#
#   SYNOPSIS
#      SkyCat_plugin {this}
#
#   DESCRIPTION
#      This is a plugin file for skycat. It can be used to add FORS FIMS
#      features to skycat at startup. To use this file for skycat, copy it
#      to $HOME/.skycatrc or set the environment variable SKYCATRC to the
#      path name of the file.
#
#      This file must define at least one tcl proc: SkyCat_plugin, which
#      takes one argument: the name of the top level skycat widget. 
#      You can refer to the man pages and source code for the Skycat and
#      RTD Itcl widgets for details.
#
#      FORS particularities:
#      several SkyCat methods are extended:
#      RtdCtrl
#      RtdImageTrans
#
#   FILES
#
#   ENVIRONMENT
#      Linux, ESO SkyCat Tool
#
#   COMMANDS
#      SkyCat_plugin {this} 
#   
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#       fims_menu.tcl for the definition of instrument specific menus
#
#   BUGS
#
#-------------------------------------------------------------------------------
#
# do not forget to 
# 'auto_mkindex /home/hummel/fims_plugin *.tcl .*rc' 
#
puts stdout "Loading the FIMS plugin..."

#===========================
proc getErrorStr {errcode} {
#===========================
    tracer ENTERING
    tracer LEAVING $errcode
    switch $errcode {
	2  {return "This is not a file"}
	3  {return "File has no permissions"}
	4  {return "This is not a fits file"}
	5  {return "Saving error"}
	6  {return "This file is a mosaic"}
	7  {return "Same file"}
	8  {return "Fits Error"}
	9  {return "Missing keyword"}
	10 {return "Slave not found"}
	11 {return "Master not found"}
	12 {return "Unknown CCD ID"}
	13 {return "Wrong keyword value"}
	14 {return "Align error"}
	15 {return "No instrument keyword"}
	16 {return "Unknown instrument"}
	default {return "Unknown error"}
    }
}

#=======================
body rtd::Rtd::quit {} {
#=======================
    tracer ENTERING
    global imageName

    catch { puts stdout "quitting fims application ..." }
    if { [info exists imageName] } {
        # Do not delete mosaiced pre-image as this name is
        # recorded to the .fims file.
	# catch { exec rm -f $imageName }
    }
    tracer LEAVING
    delete object tracer
    delete object $this
    after idle exit
}

#=============================================================
body rtd::RtdImageCtrl::open {{dir "."} {pattern "*.*fit*"}} {
#=============================================================
    tracer ENTERING
    global env imageName

    set w [utilNamespaceTail $this]
    set prep_dir $env(PREP_DIR)
    set file [filename_dialog $prep_dir $pattern $w]

    if { [string length $file] > 0 } {
	if { [file isfile $file] } {
	    detach_camera
	    # AAG - PPRS 10127
	    # fsmosaic considered for both FORSes
	    # set sec [clock seconds]
	    # set name [format "fsmosaic_%d.fits" $sec]
            # PPRS-23355: Preserve original name of the pre-image
            # - The filename without directory path
            set fn1 [file tail $file]
            # - The filename without extension
            set fn2 [file rootname $fn1]
            # - The filename extension
            set ext [file extension $fn1]
            # - The index of the first character of '.CHIP' in the filename
            set in  [string last .CHIP $fn2]
            # - Remove .CHIP? from the filename if it is there
            #   and is not at the beginning of the name.
            if { $in > 0 } {
                set fn2 [string replace $fn2 $in [expr {$in+5}]]
            }
            # - The index of the first character of '.MOSAIC' in the filename
            set in  [string last MOSAIC $fn2]
            # - Add .MOSAIC as the last part of the filename if it is not laready there
            if { $in == -1 } {
	        set mosaic [format "%s.MOSAIC%s" $fn2 $ext]
            } else {
	        set mosaic [format "%s%s" $fn2 $ext]
            }
            puts "Merging image $fn1 using fsmosaic..."
            set errcode [system fsmosaic $file $prep_dir/$mosaic]
            if { $errcode == 0 } {
	        puts "Loading merged image $mosaic ..."
                config -file "$prep_dir/$mosaic"
	        set imageName $prep_dir/$mosaic
	    } elseif { ($errcode == 6) } {	           
	        puts "$fn1 contains already a mosaic image... it has probably been to FIMS before."
	        puts "No problem, it has been loaded anyway. Please just go ahead."
	        config -file $file 		   
	    } elseif { ($errcode == 12) || ($errcode == 15) || ($errcode == 16) } {
	        puts "$fn1 is not part of a mosaic image... it was probably not taken with FORS2 instrument."
	        puts "No problem, it has been loaded anyway. Please just go ahead."
	        config -file $file 		   
	    } else {
                puts "fsmosaic: $file ERROR $errcode. No image loaded."
	        error [getErrorStr $errcode]
	    }
	    #
        } else {
            error_dialog "There is no file named '$file'" $w_
        }
    }
    tracer LEAVING
}

# This proc is required. It will be called once for each skycat
# instance. The parameter is the name ("$this") of the SkyCat
# class object.

#==========================
proc SkyCat_plugin {this} {
#==========================
    # "PBG DEBUG INI> Debugging tools hook"
    global ::env
    global ::auto_path
    global trace_state
    global trace_log
    global debug_state
    global debug_host
    global debug_path
    global debug_port
    global hooked_
    global debug_
    global skycat_plugin_initialized_


    if { ![info exists hooked_] } {
        set debug_ 0
        set debug_host "unknown"
        set debug_state "OFF"
        if { [info exists env(FIMS_DEBUG)] } {
            set debug_state $env(FIMS_DEBUG)
            if { [string compare -nocase -length 3 $debug_state "off"] != 0 &&
                 [string compare -nocase -length 2 $debug_state  "no"] != 0 &&
                 [string compare -nocase -length 1 $debug_state   "0"] != 0 } {
                set debug_ 1
                set debug_state "ON"
                if { [info exists env(HOST)] } {
                    set debug_host $env(HOST)
                }
                if { [info exists env(DEBUG_HOST)] } {
                    set debug_host $env(DEBUG_HOST)
                }
                set debug_path "/vlt/JAN2006/tcltk/TclPro/linux-ix86/bin/prodebug.tcl"
                if { [info exists env(DEBUG_PATH)] } {
                    set debug_path $env(DEBUG_PATH)
                }
                set debug_port "2576"
                if { [info exists env(DEBUG_PORT)] } {
                    set debug_port $env(DEBUG_PORT)
                }
                puts "."
            }
        }
        set reply 0
        if { $debug_state == "ON" } {
            if { [file exists $debug_path] } {
                if { $debug_ } { puts "Connect to Tcl Debugger" }
                source $debug_path
                set reply [debugger_init $debug_host $debug_port]
                if { $debug_ } { puts "Reply from Connect to Tcl Debugger = $reply" }
            }
        }

        set trace_state "OFF"
        if { [info exists env(FIMS_TRACE)] } {
            set trace_state $env(FIMS_TRACE)
            if { [string compare -nocase -length 3 $trace_state "off"] != 0 &&
                 [string compare -nocase -length 2 $trace_state  "no"] != 0 &&
                 [string compare -nocase -length 1 $trace_state   "0"] != 0 } {
                set trace_state "ON"
            }
        }
        set trace_log $env(HOME)/fims_trace.log
        if { [info exists env(TRACE_LOG)] } {
            set trace_log $env(TRACE_LOG)
        }
        set tbx_path .
        if { [info exists env(TBX_PATH)] } {
            set tbx_path $env(TBX_PATH)
        }

        set timer_path "$tbx_path/timer_class.tcl"
        set logger_path "$tbx_path/logger_class.tcl"
        set tracer_path "$tbx_path/tracer_class.tcl"
        set toolbox_path "$tbx_path/toolbox_class.tcl"
        if { [file exists $toolbox_path] } { 
             source $timer_path
             source $logger_path
             source $tracer_path
             source $toolbox_path
             ::toolbox_namespace::toolbox_class tracer
             tracer PATH [expr {[string length $trace_log] == 0 ? "stdout" : $trace_log}]
             puts "."
             tracer ON $trace_state
        } else {
            catch { delete class tracer_class }
            class tracer_class {
                private variable state "OFF"
                private variable tfile stdout
                private variable tabs ""
                constructor { arg } {
                    global trace_log
                    set state $arg
                    if { [string compare -nocase -length 2 $state "on"] == 0 } {
                        set state "ON"
                        set tfile [expr {[string length $trace_log] == 0 ? "stdout" : $trace_log}]
                        if { $tfile != "stdout" && $tfile != "stderr" } {
                            if { [catch { open $tfile w } tfile] } {
                                puts stderr "tracer: Cannot open $tfile"
                                puts stderr "tracer: Using stdout instead..."
                                set tfile stdout
                            }
                        }
                    } else {
                        set state "OFF"
                    }
                }
                destructor {
                    END
                    MSG "Tracer destroyed"
                    if { $state == "ON" && $tfile != "stdout" && $tfile != "stderr" } {
                        catch { flush $tfile }
                        catch { close $tfile }
                    }
                    catch { unset tfile }
                }
                public method MSG { {msg "EMPTY"} } {
                    if { $state == "ON" } {
                        puts $tfile "TRACER> $msg"
                        catch { flush $tfile }
                    } else {
                        return
                    }
                }
                public method TAB { {tab "  "} } {
                    set current_tab $tabs
                    switch -exact -- $tab {
                        in {
                            set tabs [string map {"=" " "} $tabs]
                            set tabs [cconcat $tabs " |=="]
                            return $tabs
                        }
                        is {
                            return $tabs
                        }
                        out {
                            set tabs [crange $tabs 4 end]
                            return $current_tab
                        }
                        default {
                            # return "[replicate " " [expr {[string length $tabs] - 4}]]|"
                            return "[crange $tabs 0 len-3]..."
                        }
                    }
                }
                public method ENTERING { {flag ""} } {
                    if { $state == "OFF" } {
                        return
                    }
                    MSG      "[TAB in]>Entering [lindex [info level -1] 0]$flag"
                    if { [info level] > 2 } {
                        MSG  "[TAB   ]....from [lindex [info level -2] 0]"
                    } else {
                        MSG  "[TAB   ]....from Top Level"
                    }
                    MSG      "[TAB   ]...with: [lrange [info level -1] 1 end]"
                }
                public method LEAVING { {flag "end" } } {
                    if { $state == "OFF" } {
                        return
                    }
                    MSG  "[TAB out]>Leaving at $flag [lindex [info level -1] 0]"
                }
                public method END { } {
                    if { $state == "OFF" } {
                        return
                    }
                    MSG "Trace END"
                }
            }
            catch { delete object tracer }
            tracer_class tracer $trace_state
        }
        # "PBG DEBUG END> Debugging tools hook"

        # "PBG DEBUG INI> Testing tools hook"
        set app_port 33133
        set tkreplay_port 30203
        set replay_path .
        if { [info exists env(REPLAY_LIBRARY)] } {
            set replay_path $env(REPLAY_LIBRARY)
        }
        set replay_state "OFF"
        if { [info exists env(FIMS_REPLAY)] } {
            set replay_state $env(FIMS_REPLAY)
            if { [string compare -nocase -length 3 $replay_state "off"] != 0 } {
                set replay_state "ON"
            }
        }
        set reply 0
        if { $replay_state == "ON" } {
            if { [file exists $replay_path/socksend/socksend.tcl] } {
                if { $debug_ } { puts "Connect to Tk Replay" }
                source $replay_path/socksend/socksend.tcl
                set reply [sockappsetup $app_port tkreplay.tcl $tkreplay_port]
                if { $debug_ } { puts "Reply from Connect to TkReplay = $reply" }
            }
        }
        set hooked_ 1
    }
    # "PBG DEBUG END> Testing tools hook"

    tracer ENTERING

    if { $debug_ } { puts "SkyCat_plugin $this (FIMS)" }

    # This code should run only once
    if { ![info exists skycat_plugin_initialized_] } {
        tracer MSG "[tracer TAB  ]Initializing SkyCat plugin..."
        # Get the toplevel widget name
        set w [utilNamespaceTail $this]

        # Add a new "Instrument?" menu button to the skycat menu bar
        # PBG: Commented out:  add_instrument_menu $w
        # For the time being, only FORS2 uses FIMS, so no instrument selection
        add_instrument_menu $w FORS2+1

        # Add new menu buttons in the skycat Help menu
        add_help_menu $w

        # Put here the body commands
       
        #--------------
        # RtdImageTrans
        #--------------
        # override flip/Z/z buttons
        set t [$w component image component info component trans]

        # ZOOM BUTTONS [Z] [z]
        # --------------------
        set body [$t info body update_trans]
        body rtd::RtdImageTrans::update_trans {} \
            "$body\n \
             catch { fors::FORS::my_mask update_and_replot\n } "
        # catch { fors::FORS::my_mask update_scale }\n 

        # ROTATE BUTTON [@]
        # -----------------
        set body [$t info body rotate]
        body rtd::RtdImageTrans::rotate {} \
            "$body\n \
             catch { fors::FORS::my_mask init_mask_world2 }\n  \
             catch { fors::FORS::my_mask update_and_replot } "
 
        # FLIP BUTTON [><] [^v]
        # ---------------------
        set body [$t info body flip]
        body rtd::RtdImageTrans::flip {xy} \
            "$body\n \
             catch { fors::FORS::my_mask init_mask_world2 }\n  \
             catch { fors::FORS::my_mask update_and_replot } "
        # Disable some skycat buttons no longer required
        # $t.rotate configure -state disabled 
        # $t.flipx  configure -state disabled 
        # $t.flipy  configure -state disabled 

        #--------
        # RtdCtrl
        #--------
        # Override open/file 
        # set t [$w component image]
        # set body [$t info body open]
        # set prep_dir $env(PREP_DIR)
      
        # set file {}
        # set pattern "*.*fit*"
        
        # body rtd::RtdImageCtrl::open {{dir "."} {pattern "*.*fit*"}} \
        # "global .fs\n\
        # catch { destroy .fs }\n\
        # set file [filename_dialog $prep_dir $pattern $w]\n\
        # puts \"FILE: $file\"\n\
        # catch { \n\
        #     fors::FORS::my_mask reset_mask\n\
        #     $t.rotate configure -state disabled\n\
        #     $t.flipy  configure -state disabled \n\
        # }\n"
        tracer MSG "[tracer TAB  ]SkyCat plugin initialized"
        set  skycat_plugin_initialized_ 1
    } else {
        if { $debug_ } { puts "skycat_plugin_initialized_: already >$skycat_plugin_initialized_<" }
    }
    tracer LEAVING
}

#
#___FIMS___
