#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: conv_mc.tcl 194234 2010-01-28 05:02:47Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2010/01/24  Added tracing
# pbaksai   2009/07/28  PPRS-27204: Check for division by zero
# pbaksai   2009/06/02  PPRS-27204: Updated 
# W.Hummel  04.Mar.99   Created 
#
#*******************************************************************************
#   NAME             
#   CONV_MC - Tcl class           
# 
#   NAMESPACE
#       fors
#
#   SYNOPSIS         
#       CONV_MC image ?-eqx <equinox>?
#       
#   PARENT CLASS
#       none 
#
#   DESCRIPTION      
#       Handling of coordinate conversion 
#
#   OPTIONS
#
#   PUBLIC METHODS
#       coeff_mc constructs conversion matrix
#       c2m      canvas to mm foc coord conversion
#       m2c      mm_foc to canvas coord conversion
#
#   PUBLIC DATA MEMBERS
#       eqx         2000.0
#
#   PROTECTED METHODS
#
#   COMMON DATA MEMBERS
#
#   PRIVATE METHODS
#
#   PRIVATE DATA MEMBERS
#       p1,p2,p3,p4,
#       f1,f2,f3,f4,f5,f6,f7,f8
#
#   COMMANDS
#       CONV_MC <instance_name> {equinox} 
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#-------------------------------------------------------------------------------
#        1         2         3         4         5         6         7         8
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#

itk::usual FORS {}

# create a class for the application

namespace eval fors {

class CONV_MC {

#---------------------
constructor { args } {
#---------------------
    tracer ENTERING " of $this"
    eval configure $args
    tracer LEAVING "end of $this"
}

#-----------
destructor {
#-----------
    tracer ENTERING " of $this"
    tracer LEAVING "end of $this"
}

#===========================================
public method coeff_mc { mx1 my1 cx1 cy1 \
                         mx2 my2 cx2 cy2 \
                         mx3 my3 cx3 cy3 } {
#===========================================
# This method creates the matrix for conversion between 
# mm focal plane and canvas.
# This conversion will be used for MXU mode.
# Three points are given, each one in two coord systems:
# m for mm focal plane and c for canvas
# The coord conversion matrix elements are common 
# variables and accessible to all instances of mxu_slit

    tracer ENTERING
    # From canvas to mm focal plane

    # Now we have to extrapolate using some minor vector algebra
    # (mx)    (p1)   (f1 f2)   (cx)
    # (  ) =  (  ) + (     ) * (  )
    # (my)    (p2)   (f3 f4)   (cy)

    # puts stdout [format "sub_WC: cx1,cy1 mx1,my1 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $cx1 $cy1 $mx1 $my1]
    # puts stdout [format "sub_WC: cx2,cy2 mx2,my2 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $cx2 $cy2 $mx2 $my2]
    # puts stdout [format "sub_WC: cx3,cy3 mx3,my3 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $cx3 $cy3 $mx3 $my3]

    # Create the matrix elements f1-f4
    # f1 and f3 have the same nominator
    set nom1 [expr ($cx1-$cx2)*($cy2-$cy3)-($cy1-$cy2)*($cx2-$cx3)]
    set f1 [expr ($mx1-$mx2)*($cy2-$cy3)-($mx2-$mx3)*($cy1-$cy2)]
    set f3 [expr ($my1-$my2)*($cy2-$cy3)-($my2-$my3)*($cy1-$cy2)]
    set f1 [expr {$nom1?[expr $f1/$nom1]:$f1}]
    set f3 [expr {$nom1?[expr $f3/$nom1]:$f3}]
    # f2 and f4 have the same nominator
    set nom2 [expr ($cy1-$cy2)*($cx2-$cx3)-($cy2-$cy3)*($cx1-$cx2)]
    set f2 [expr ($mx1-$mx2)*($cx2-$cx3)-($mx2-$mx3)*($cx1-$cx2)]
    set f4 [expr ($my1-$my2)*($cx2-$cx3)-($my2-$my3)*($cx1-$cx2)]
    set f2 [expr {$nom2?[expr $f2/$nom2]:$f2}]
    set f4 [expr {$nom2?[expr $f4/$nom2]:$f4}]

    # puts stdout [format "sub_WC: f1 f2 nom = %10.3f %10.3f %8.8f" \
    #                      $f1 $f2 $nom1]
    # puts stdout [format "sub_WC: f3 f4 nom = %10.3f %10.3f %8.8f" \
    #                      $f3 $f4 $nom2]
    # puts stdout [format "sub_WC: scale = %8.3f ang=%8.3f" \
    #                      [expr sqrt($f1*$f1+$f3*$f3)]  \
    #                      [expr atan($f3/$f1)*57.29578]]
    # Now the offset
    set p1 [expr $mx1 - $cx1*$f1-$cy1*$f2]
    set p2 [expr $my1 - $cx1*$f3-$cy1*$f4]

    # puts stdout [format "sub_WC: p1 p2     = %10.3f %10.3f" $p1 $p2]

    # set p1 [expr $mx2 - $cx2*$f1-$cy2*$f2]
    # set p2 [expr $my2 - $cx2*$f3-$cy2*$f4]
    # puts stdout [format "sub_WC: p1 p2     = %10.3f %10.3f" $p1 $p2]

    # set p1 [expr $mx3 - $cx3*$f1-$cy3*$f2]
    # set p2 [expr $my3 - $cx3*$f3-$cy3*$f4]
    # puts stdout [format "sub_WC: p1 p2     = %10.3f %10.3f" $p1 $p2]

    # Verify
    # puts stdout [format "sub_WC: cx1,cy1 mx1,my1 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $cx1 $cy1 [expr $p1+$cx1*$f1+$cy1*$f2] [expr $p2+$cx1*$f3+$cy1*$f4]]
    # puts stdout [format "sub_WC: cx2,cy2 mx2,my2 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $cx2 $cy2 [expr $p1+$cx2*$f1+$cy2*$f2] [expr $p2+$cx2*$f3+$cy2*$f4]]
    # puts stdout [format "sub_WC: cx3,cy3 mx3,my3 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $cx3 $cy3 [expr $p1+$cx3*$f1+$cy3*$f2] [expr $p2+$cx3*$f3+$cy3*$f4]]
       
    #------------------
    # From mm to canvas 
    #------------------

    # Now we have to extrapolate using some minor vector algebra
    # (cx)    (p3)   (f5 f6)   (mx)
    # (  ) =  (  ) + (     ) * (  )
    # (cy)    (p4)   (f7 f8)   (my)

    # puts stdout "- - - - - - - - - - - - - - - - - - - -"
    # puts stdout [format "sub_CW: mx1,my1 cx1,cy1 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $mx1 $my1 $cx1 $cy1]
    # puts stdout [format "sub_CW: mx2,my2 cx2,cy2 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $mx2 $my2 $cx2 $cy2]
    # puts stdout [format "sub_CW: mx3,my3 cx3,cy3 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $mx3 $my3 $cx3 $cy3]

    # Create the matrix elements f5-f8
    # f5 and f7 have the same nominator
    set nom1 [expr ($mx1-$mx2)*($my2-$my3)-($my1-$my2)*($mx2-$mx3)]
    set f5 [expr ($cx1-$cx2)*($my2-$my3)-($cx2-$cx3)*($my1-$my2)]
    set f7 [expr ($cy1-$cy2)*($my2-$my3)-($cy2-$cy3)*($my1-$my2)]
    set f5 [expr {$nom1?[expr $f5/$nom1]:$f5}]
    set f7 [expr {$nom1?[expr $f7/$nom1]:$f7}]
    # f6 and f8 have the same nominator
    set nom2 [expr ($my1-$my2)*($mx2-$mx3)-($my2-$my3)*($mx1-$mx2)]
    set f6 [expr ($cx1-$cx2)*($mx2-$mx3)-($cx2-$cx3)*($mx1-$mx2)]
    set f8 [expr ($cy1-$cy2)*($mx2-$mx3)-($cy2-$cy3)*($mx1-$mx2)]
    set f6 [expr {$nom2?[expr $f6/$nom2]:$f6}]
    set f8 [expr {$nom2?[expr $f8/$nom2]:$f8}]

    # puts stdout [format "sub_CW: f5 f6 nom = %10.3e %10.3e %8.8f" \
    #                      $f5 $f6 $nom1]
    # puts stdout [format "sub_CW: f7 f8 nom = %10.3e %10.3e %8.8f" \
    #                      $f7 $f8 $nom2]
    # puts stdout [format "sub_CW: scale = %8.3f ang=%8.3f" \
    #                      [expr sqrt($f1*$f1+$f3*$f3)] \
    #                      [expr atan($f3/$f1)*57.29578]]
    # Now the offset
    set p3 [expr $cx1 - $mx1*$f5-$my1*$f6]
    set p4 [expr $cy1 - $mx1*$f7-$my1*$f8]

    # puts stdout [format "sub_CW: p3 p4     = %10.3f %10.3f" $p3 $p4]
    # set p3 [expr $cx2 - $mx2*$f5-$my2*$f6]
    # set p4 [expr $cy2 - $mx2*$f7-$my2*$f8]
    # puts stdout [format "sub_CW: p3 p4     = %10.3f %10.3f" $p3 $p4]
    # set p3 [expr $cx3 - $mx3*$f5-$my3*$f6]
    # set p4 [expr $cy3 - $mx3*$f7-$my3*$f8]
    # puts stdout [format "sub_CW: p3 p4     = %10.3f %10.3f" $p3 $p4]

    # Verify
    # puts stdout [format "sub_CW: mx1,my1 cx1,cy1 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $mx1 $my1 [expr $p3+$mx1*$f5+$my1*$f6] [expr $p4+$mx1*$f7+$my1*$f8]]
    # puts stdout [format "sub_CW: mx2,my2 cx2,cy2 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $mx2 $my2 [expr $p3+$mx2*$f5+$my2*$f6] [expr $p4+$mx2*$f7+$my2*$f8]]
    # puts stdout [format "sub_CW: mx3,my3 cx3,cy3 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $mx3 $my3 [expr $p3+$mx3*$f5+$my3*$f6] [expr $p4+$mx3*$f7+$my3*$f8]]
    tracer LEAVING
}

#============================
public method m2c { mx my } {
#============================
# Converts mm focplane coords to canvas coords 
    tracer ENTERING
    tracer LEAVING
    return [list [expr $p3+$mx*$f5+$my*$f6] [expr $p4+$mx*$f7+$my*$f8]]
}

#============================
public method c2m { cx cy } {
#============================
# Converts canvas coords to mm focplane coords 
    tracer ENTERING
    tracer LEAVING
    return [list [expr $p1+$f1*$cx+$f2*$cy] [expr $p2+$f3*$cx+$f4*$cy]]
}

    public variable eqx 2000.0

    private variable f1 0.0
    private variable f2 0.0
    private variable f3 0.0
    private variable f4 0.0
    private variable p1 0.0
    private variable p2 0.0
    private variable f5 0.0
    private variable f6 0.0
    private variable f7 0.0
    private variable f8 0.0
    private variable p3 0.0
    private variable p4 0.0

}
# End of the class CONV_MC

}
# End of namespace

# fors::CONV_MC           
#
#___oOo___
