#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: conv_wc.tcl 194234 2010-01-28 05:02:47Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2010/01/24  Added tracing
# pbaksai   2009/07/28  PPRS-27204: Check for division by zero
# pbaksai   2008/07/20  PPRS-27337: Updated
# W.Hummel  04.Mar.99   Created 
#
#*******************************************************************************
#   NAME             
#   CONV_WC - Tcl class           
# 
#   NAMESPACE
#       fors
#
#   SYNOPSIS         
#       CONV_WC image ?-eqx <equinox>?
#       
#   PARENT CLASS
#       none 
#
#   DESCRIPTION      
#       Handling of coordinate conversion 
#
#   OPTIONS
#
#   PUBLIC METHODS
#       coeff_wc constructs conversion matrix
#       w2c      world to canvas coord conversion 
#       c2w      canvas to world coord conversion
#
#   PUBLIC DATA MEMBERS
#       eqx         2000.0
#       m_image 
#
#   PROTECTED METHODS
#
#   PROTECTED DATA MEMBERS
#
#   PRIVATE METHODS
#
#   PRIVATE DATA MEMBERS
#       b1,b2,b3,b4,k1,k2,k3,k4,k5,k6,k7,k8
#
#   COMMANDS
#       CONV_WC <instance_name> {m_image equinox} 
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#
#   SEE ALSO
#
#-------------------------------------------------------------------------------
#        1         2         3         4         5         6         7         8
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#

itk::usual FORS {}

namespace eval fors {

class CONV_WC {

#---------------------
constructor { args } {
#---------------------
    tracer ENTERING " of $this"
    eval configure $args
    tracer LEAVING "end of $this"
}

#-----------
destructor {
#-----------
    tracer ENTERING " of $this"
    tracer LEAVING "end of $this"
}

#===============================
public method coeff_wc {a1 d1} {
#===============================
# The older fims versions used the bonsai_matrix 
# which is now done here 
# a1 and d1 are WCS-deg coords of the mask center 

    tracer ENTERING
    # From WCS to CANVAS 

    # Now we have to extrapolate using some minor vector algebra
    # (x)    (b1)   (k1 k2)   (alp)
    # ( ) =  (  ) + (     ) * (   )
    # (y)    (b2)   (k3 k4)   (dec)

    # First central pixel
    if { [catch { $m_image convert coords $a1 $d1 "deg $eqx" x1 y1 canvas } result] } {
        set x1 0
        set y1 0
    }

    # Two more test points inside the bonsai image
    # dist = distance from center in  arcsec
    set min_size_deg [expr $min_size_arcmin/60.0];  # radius in degrees 
    set a2 [expr $a1 + $min_size_deg]
    set d2 $d1
    set a3 $a1 
    set d3 [expr $d1 + $min_size_deg]
    # And convert them to canvas
    if { [catch { $m_image convert coords $a2 $d2 "deg $eqx" x2 y2 canvas } result] } {
        set x2 0
        set y2 0
    }
    if { [catch { $m_image convert coords $a3 $d3 "deg $eqx" x3 y3 canvas } result] } {
        set x3 0
        set y3 0
    }

    # puts stdout "sub_WC: min_size_arcmin =  $min_size_deg $min_size_deg"
    # puts stdout [format "sub_WC: a1,d1 x1,y1 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $a1 $d1 $x1 $y1]
    # puts stdout [format "sub_WC: a2,d2 x2,y2 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $a2 $d2 $x2 $y2]
    # puts stdout [format "sub_WC: a3,d3 x3,y3 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $a3 $d3 $x3 $y3]

    # Create the matrix elements k1-k4
    # k1 and k3 have the same nominator
    set nom1 [expr ($a1-$a2)*($d2-$d3)-($d1-$d2)*($a2-$a3)]
    set k1 [expr ($x1-$x2)*($d2-$d3)-($x2-$x3)*($d1-$d2)]
    set k3 [expr ($y1-$y2)*($d2-$d3)-($y2-$y3)*($d1-$d2)]
    set k1 [expr {$nom1?[expr $k1/$nom1]:$k1}]
    set k3 [expr {$nom1?[expr $k3/$nom1]:$k3}]
    # k2 and k4 have the same nominator
    set nom2 [expr ($d1-$d2)*($a2-$a3)-($d2-$d3)*($a1-$a2)]
    set k2 [expr ($x1-$x2)*($a2-$a3)-($x2-$x3)*($a1-$a2)]
    set k4 [expr ($y1-$y2)*($a2-$a3)-($y2-$y3)*($a1-$a2)]
    set k2 [expr {$nom2?[expr $k2/$nom2]:$k2}]
    set k4 [expr {$nom2?[expr $k4/$nom2]:$k4}]

    # puts stdout [format "sub_WC: k1 k2 nom = %10.3f %10.3f %8.8f" \
    #                      $k1 $k2 $nom1 ]
    # puts stdout [format "sub_WC: k3 k4 nom = %10.3f %10.3f %8.8f" \
    #                      $k3 $k4 $nom2 ]
    # puts stdout [format "bonsai_matrix: scale = %8.3f ang=%8.3f" \
    #                      [expr sqrt($k1*$k1+$k3*$k3)]  \
    #                      [expr atan($k3/$k1)*57.29578] ]
    # Now the offset
    set b1 [expr $x1 - $a1*$k1-$d1*$k2]
    set b2 [expr $y1 - $a1*$k3-$d1*$k4]

    # puts stdout [format "sub_WC: b1 b2     = %10.3f %10.3f" $b1 $b2]
    # set b1 [expr $x2 - $a2*$k1-$d2*$k2]
    # set b2 [expr $y2 - $a2*$k3-$d2*$k4]
    # puts stdout [format "sub_WC: b1 b2     = %10.3f %10.3f" $b1 $b2]
    # set b1 [expr $x3 - $a3*$k1-$d3*$k2]
    # set b2 [expr $y3 - $a3*$k3-$d3*$k4]
    # puts stdout [format "sub_WC: b1 b2     = %10.3f %10.3f" $b1 $b2]

    # Verify
    # puts stdout [format "sub_WC: a1,d1 x1,y1 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $a1 $d1 [expr $b1+$a1*$k1+$d1*$k2] [expr $b2+$a1*$k3+$d1*$k4]]
    # puts stdout [format "sub_WC: a2,d2 x2,y2 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $a2 $d2 [expr $b1+$a2*$k1+$d2*$k2] [expr $b2+$a2*$k3+$d2*$k4]]
    # puts stdout [format "sub_WC: a3,d3 x3,y3 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $a3 $d3 [expr $b1+$a3*$k1+$d3*$k2] [expr $b2+$a3*$k3+$d3*$k4]]


    # From CANVAS to WCS, the inverse coord conversion 

    # Now we have to extrapolate using some minor vector algebra
    # (alp)    (b3)   (k5 k6)   (x1)
    # (   ) =  (  ) + (     ) * (  )
    # (dec)    (b4)   (k7 k8)   (y1)
    #
    # First central pixel
    # $m_image convert coords $mos(0,x0) $mos(0,y0) canvas \
    #                         a1 d1  "deg $eqx"
    # set x1 $fr_x
    # set y1 $fr_y
    # set a1 $wx
    # set d1 $wy
    # set a1 0.0
    # set d1 0.0
    # set x1 10.0
    # set y1 10.0
    # Two more test points inside the bonsai image
    # dist = distance from center in  arcsec
    #---------------------------------------
    # If your image is too small for fims
    # decrase 'min_size_arcmin' on charge of mask pointing accuracy
    #---------------------------------------
    # set min_size_arcmin 0.8
    #---------------------------------------
    # $m_image convert dist [expr $min_size_arcmin/60.0 ] 0 "deg $eqx" \
    #                        msc_x  msc_y canvas
    # set min_size_canvas [expr sqrt( $msc_x*$msc_x+$msc_y*$msc_y)]
    # puts stdout "$min_size_arcmin $min_size_canvas"
    # set x2 [expr $x1 + $min_size_canvas]
    # set y2 $y1
    # set x3 $x1 
    # set y3 [expr $y1 + $min_size_canvas]
    # And convert them to canvas
    # puts stdout [format "%10.3f %10.3f %10.3f %10.3f %10.3f %10.3f" \
    #                      $x1 $y1 $x2 $y2 $x3 $y3]

    # Maybe this is not necessary
    catch { $m_image convert coords $x2 $y2 canvas a2 d2 "deg $eqx" }
    catch { $m_image convert coords $x3 $y3 canvas a3 d3 "deg $eqx" }

    # puts stdout [format "sub_CW: x1,y1 a1,d1 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $x1 $y1 $a1 $d1]
    # puts stdout [format "sub_CW: x2,y2 a2,d2 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $x2 $y2 $a2 $d2]
    # puts stdout [format "sub_CW: x3,y3 a3,d3 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $x3 $y3 $a3 $d3]

    # Create the matrix elements k5-k8
    # k5 and k7 have the same nominator
    set nom1 [expr ($x1-$x2)*($y2-$y3)-($y1-$y2)*($x2-$x3)]
    set k5 [expr ($a1-$a2)*($y2-$y3)-($a2-$a3)*($y1-$y2)]
    set k7 [expr ($d1-$d2)*($y2-$y3)-($d2-$d3)*($y1-$y2)]
    set k5 [expr {$nom1?[expr $k5/$nom1]:$k5}]
    set k7 [expr {$nom1?[expr $k7/$nom1]:$k7}]
    # k6 and k8 have the same nominator
    set nom2 [expr ($y1-$y2)*($x2-$x3)-($y2-$y3)*($x1-$x2)]
    set k6 [expr ($a1-$a2)*($x2-$x3)-($a2-$a3)*($x1-$x2)]
    set k8 [expr ($d1-$d2)*($x2-$x3)-($d2-$d3)*($x1-$x2)]
    set k6 [expr {$nom2?[expr $k6/$nom2]:$k6}]
    set k8 [expr {$nom2?[expr $k8/$nom2]:$k8}]

    # puts stdout [format "sub_CW: k5 k6 nom = %10.3e %10.3e %8.8f" \
    #                      $k5 $k6 $nom1]
    # puts stdout [format "sub_CW: k7 k8 nom = %10.3e %10.3e %8.8f" \
    #                      $k7 $k8 $nom2]
    # puts stdout [format "sub_CW: scale = %8.3f ang=%8.3f" \
    #                     [expr sqrt($k1*$k1+$k3*$k3)] \
    #                     [expr atan($k3/$k1)*57.29578]]
    # Now the offset
    set b3 [expr $a1 - $x1*$k5-$y1*$k6]
    set b4 [expr $d1 - $x1*$k7-$y1*$k8]

    # puts stdout [format "sub_CW: b3 b4     = %10.3f %10.3f" $b3 $b4]
    # set b3 [expr $a2 - $x2*$k5-$y2*$k6]
    # set b4 [expr $d2 - $x2*$k7-$y2*$k8]
    # puts stdout [format "sub_CW: b3 b4     = %10.3f %10.3f" $b3 $b4]
    # set b3 [expr $a3 - $x3*$k5-$y3*$k6]
    # set b4 [expr $d3 - $x3*$k7-$y3*$k8]
    # puts stdout [format "sub_CW: b3 b4     = %10.3f %10.3f" $b3 $b4]

    # Verify
    # puts stdout [format "sub_CW: x1,y1 a1,d1 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $x1 $y1 [expr $b3+$x1*$k5+$y1*$k6] [expr $b4+$x1*$k7+$y1*$k8]]
    # puts stdout [format "sub_CW: x2,y2 a2,d2 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $x2 $y2 [expr $b3+$x2*$k5+$y2*$k6] [expr $b4+$x2*$k7+$y2*$k8]]
    # puts stdout [format "sub_CW: x3,y3 a3,d3 = %8.3f %8.3f => %8.3f %8.3f" \
    #                      $x3 $y3 [expr $b3+$x3*$k5+$y3*$k6] [expr $b4+$x3*$k7+$y3*$k8]]
    tracer LEAVING
}

#============================
public method w2c { wx wy } {
#============================
# Converts radec world coords to canvas coords
    tracer ENTERING
    tracer LEAVING
    return [list [expr $b1+$k1*$wx+$k2*$wy] [expr $b2+$k3*$wx+$k4*$wy]]
}

#============================
public method c2w { cx cy } {
#============================
# Converts canvas coords to radec world coords
    tracer ENTERING
    tracer LEAVING
    return [list [expr $b3+$cx*$k5+$cy*$k6] [expr $b4+$cx*$k7+$cy*$k8]]
}

    public variable m_image 
    public variable eqx 2000.0

    private variable min_size_arcmin 0.4
    private variable k1
    private variable k2
    private variable k3
    private variable k4
    private variable b1
    private variable b2
    private variable k5
    private variable k6
    private variable k7
    private variable k8
    private variable b3
    private variable b4

} 
# End of the class CONV_WC

} 
# End of namespace

# fors::CONV_WC               
#
#___oOo___
