#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fims_menu.tcl 257795 2014-07-28 14:28:30Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2014/07/25  CCB-000579: Support for 64 bits and portability.
#                       - Adjusted to new product package structure.
# pbaksai   2012/11/01  PPRS-23355: Updated for new product package structure
#                       - Use global variable skycat_version
# pbaksai   2011/11/16  PPRS-23355: Updated for SVN versioning
#                                   Corrected grism info format
#                                   Relocated standard_mask_list to config
# pbaksai   2011/06/06  PPRS-27204: Re-arrange main menu options
# pbaksai   2011/05/31  PPRS-27204: Disable all grisms but XGRIS* for HIT-MS
# pbaksai   2010/12/03  PRPS-23355: Clarified comments
# pbaksai   2010/01/26  PPRS-24729: Added verbosity flag configuration
# pbaksai   2009/11/20  PPRS-33492: Removed Open_Blades, Close_Blades commands
# pbaksai   2009/08/13  PPRS-27204: Modified popups, fixed grism info
# pbaksai   2009/07/28  PPRS-27204: Modified color menu
# pbaksai   2009/06/30  PPRS-27204: Adjusted active menu options per modus
# pbaksai   2009/06/16  PPRS-27204: Added Open_Blades, Close_Blades commands
# pbaksai   2009/06/02  PPRS-27204: Renamed from fors_menu to instrument_menu
#                                   Improved menu indexation
# pbaksai   2008/11/25  PPRS-29438: P83 IP preparation
# pbaksai   2008/06/23  PPRS-27204: Menu reengineering. HIT-MS support
# pbaksai   2008/06/23  PPRS-27337: Added camera menu.
# pbaksai   2008/06/19  PPRS-27204: Created with Menu item index names.
#
#*******************************************************************************
#   NAME
# 
#   SYNOPSIS
# 
#   DESCRIPTION
#
#   FILES
#
#   ENVIRONMENT
#
#   COMMANDS
#       add_instrument_menu {w {instrument ""}} 
#              adds the 'FORS[f]' cascaded main menu
#              to the SkyCat Tool menu bar  
#       add_help_menu {w} 
#              adds 5 more buttons in the help menu
#       build_ins_menu {w instrument {del 0}}
#       build_fors_menu {w instrument {del 0}}
#       build_fors_menu_EnterMode {w instrument}
#       build_fors_menu_AutoSlit {w instrument modus}
#       build_fors_menu_RefTargets {w instrument modus}
#       build_fors_menu_StatusDemo {w}
#       build_fors_menu_Layout {w instrument modus}
#       build_fors_menu_SlitWidth {w instrument modus}
#       build_fors_menu_SlitLength {w instrument modus}
#       build_fors_menu_SlitShape {w instrument modus}
#       build_fors_menu_Collimator {w instrument modus col_range}
#       build_fors_menu_Filter {w instrument modus fil_range}
#       build_fors_menu_Grism {w instrument modus gris_range}
#       build_fors_menu_Rotation {w instrument modus}
#       build_fors_menu_Color {w instrument modus}
#
#   CALLBACKS
#       f_on  {w instrument mode m_v} 
#              calls the fors::FORS class proc to create one single 
#              instance of FORS
#              configures all necessary menu buttons 
#       f_off {w mode m_v} 
#              deletes the only instance of FORS
#              configures all necessary menu buttons 
#       fims_about {w} 
#              calls the fors info widget
#       mask_about {w}
#              calls the modified skycat info widget
#       grism_about {w} 
#              calls the grism info widget
#
#   RETURN VALUES
#
#   CAUTIONS
#       HITS mode in connection with highres collimator 
#       is allowed but not senseful
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS     
#
#-------------------------------------------------------------------------------
#
#----------------------------------------------
proc add_instrument_menu {w {instrument ""} } {
#----------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1), if empty then the menu is just
#             being created; else it is being redefined for another instrument.

    tracer ENTERING

    # m_b: Global menu button that stores all the instrument menus.
    # m_i: Global named array that stores the index of each item in the menu,
    #      so we can refer to each button by name, not by a hardcoded index or
    #      a hardcoded label.
    #      This way we can change the menu label or item order, or insert new
    #      items without worrying about the consistency of invoke commands in
    #      the rest of the code.
    global ::m_b
    global ::m_i

    # The mask instance that FORS class will create
    global my_mask

    set my_mask ::fors::FORS::my_mask

    if { ![winfo exists $w.menubar.instrument.m] } {
        set m_b [$w add_menubutton "instrument" "Instrument to build a mask for"]
    }
    $w configure_menubutton instrument -text "Instrument?"

    if { [string length $instrument] > 0 } {
        build_ins_menu $w $instrument
        tracer LEAVING INSTR
        return
    }

    # Menu items (buttons)
    # mi:  Local array of lists, each one describing a menu item.
    #      It forms a table with one menu item per row and one item
    #      attribute per column.
    # t:   Total number of defined menu items in the 'mi' table
    #      (mi(0) is dummy, mi(1..t) is the useful range)
    # m_n: Local named array that stores the index of each item in the 'mi'
    #      array, so we can refer to them by name while calling the respective
    #      item creator.
    #      The name for the item (or button) is given in the 'index' column of
    #      'mi' table.
    # The columns are:
    # - index: the name of the index of the menu; this name is used in the rest
    #          of the code and represents the value of i
    # - label: the actual label seen on the menu widget
    # - help: help text associated
    # - type: menu can be of type cascade: there is another menu inside it
    #                             command: it executes a command
    #                             radio:   it sets a selection a value
    #                             check:   it sets a boolean
    #                             separator: just an horizontal line
    # - command: the command line to execute, only valid for command type menu
    # - submenu: the submenu name, only valid for casacde type menu
    # - toff: option switch for the tear off feature, only valid for cascade type menu.
    #         If ON (1) the index starts at 1 not 0, since the tearoff line has the 0 index
    # - init: the initial state of the menu at creation time; normal
    #                                                         disabled
    # - var: the name of the variable holding the state of check or radio button
    #
    # To maintain this table, just add or remove rows, except the first and
    # last ones.
    # First row defines the column names
    # Last row must not increment 'i' but set 't' with its current value.
    #
    # If you need to remove an option from the menu, try not to delete the row
    # in the mi table. Instead, use menu item list (mil) to define a subset of
    # active menu items to be included.
    #
    set i 0
    set mi($i) {index   label   help               type    command submenu toff init   var}; incr i
    set mi($i) {FORS1   FORS1   {{Select FORS1}}   command FORS1   {}      {}   normal {} }; incr i
    set mi($i) {FORS2   FORS2   {{Select FORS2}}   command FORS2   {}      {}   normal {} }; incr i
    set mi($i) {FORS2+1 FORS2+1 {{Select FORS2+1}} command FORS2+1 {}      {}   normal {} }; set t $i

    # Active menu items list.
    set mil {}
    # Can include all the menu items of the 'mi' table,
    for { set i 1 } { $i <= $t } { incr i } {
        lappend mil $i
    }
    # or just a subset of buttons.
    # set mil {}
    # lappend mil 2 3

    # Local menu vector
    set mv $m_b

    # Menu item indexation, save the button position referenced with its name.
    set mii 0
    foreach i $mil {
        # Store the position of the item button in the menu (sequential 'mii')
        # and of the item name in the 'mi' table (subset 'mil' indexed by 'i')
        # Arrays are referenced by the item name located in the 'index' column.
        set item_name [lindex $mi($i) [lsearch -exact $mi(0) index]]
        set item_type [string tolower [lindex $mi($i) [lsearch -exact $mi(0) type]]]
        set m_i($item_name) $mii; # Position of item_name in the menu
        set m_n($item_name) $i;   # Position of item_name in the mi table
        incr mii
        switch -exact -- $item_type {
            cascade {
                menu_Cascade $w $mv $mi(0) $mi($i)
                switch -exact -- $item_name {
                    default {
                    }
                }
            }
            separator {
                $mv add separator
            }
            default {
                switch -exact -- $item_name {
                    default {
                        menu_Command $w $mv $mi(0) $mi($i) [code build_ins_menu $w [lindex $mi($i) [lsearch -exact $mi(0) command]]]
                    }
                }
            }
        }
    }

    tracer LEAVING
}

#-----------------------
proc add_help_menu {w} {
#-----------------------
# w: skycat instance 
# This proc adds five menus to the Help menu

    tracer ENTERING
    
    global ::m_i

    # Get the path name of the Help menu
    # see ToplevelWidget tclutil
    set m [$w component menubar]

    set i 0
    set mi($i) {index        label                     help                                                 type    command submenu toff init   var}; incr i
    set mi($i) {{FimsAbout}  {About FIMS...}           {FIMS version information}                           command {}      {}      {}   normal {} }; incr i
    set mi($i) {{ForsManual} {FORS User Manual}        {FoRS information if netscape is available}          command {}      {}      {}   normal {} }; incr i
    set mi($i) {{ForsETC}    {FORS Exposure Time Calc} {WWW connection to ESO ETC if netscape is available} command {}      {}      {}   normal {} }; incr i
    set mi($i) {{GrismAbout} {Grism Info}              {Information about FORS grism-OS filter combos}      command {}      {}      {}   normal {} }; incr i
    set mi($i) {{MaskAbout}  {Mask Info}               {Standard Masks information}                         command {}      {}      {}   normal {} }; set t $i
    # Active menu items list
    set mil {}
    for { set i 1 } { $i <= $t } { incr i } {
        lappend mil $i
    }
    # set mil {}
    # lappend mil 1 2 3

    # Local menu vector
    set mv $m.help.m

    # Clear menu items
    $mv delete 2 last

    # Menu item indexation
    set mii 0
    foreach i $mil {
        set item_name [lindex $mi($i) [lsearch -exact $mi(0) index]]
        set item_type [string tolower [lindex $mi($i) [lsearch -exact $mi(0) type]]]
        set m_i($item_name) $mii
        set m_n($item_name) $i
        incr mii
        switch -exact -- $item_type {
            cascade {
                menu_Cascade $w $mv $mi(0) $mi($i)
                switch -exact -- $item_name {
                    default {
                    }
                }
            }
            separator {
                $mv add separator
            }
            default {
                switch -exact -- $item_name {
                    default {
                    }
                }
            }
        }
    }

    $mv add separator
    menu_Command $w $mv $mi(0) $mi($m_n(FimsAbout))  [code fims_about $w]
    menu_Command $w $mv $mi(0) $mi($m_n(ForsManual)) [code $w.image send_to_netscape "http://www.eso.org/instruments/fors1/userman/userman.html"]
    menu_Command $w $mv $mi(0) $mi($m_n(ForsETC))    [code $w.image send_to_netscape "http://www.eso.org/observing/etc/"]
    menu_Command $w $mv $mi(0) $mi($m_n(GrismAbout)) [code grism_about $w]
    menu_Command $w $mv $mi(0) $mi($m_n(MaskAbout))  [code masks_about $w]

    tracer LEAVING
}

#--------------------------------------------
proc build_ins_menu {w instrument {del 0}} {
#--------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# del:        flag to clear menu button m_b

    tracer ENTERING

    switch -exact -- $instrument {
        FORS1 -
        FORS2 -
        FORS2+1 {
            build_fors_menu $w $instrument $del
        }
        default {
            set del 0
            add_instrument_menu $w
        }
    }

    if { $del } {
        add_instrument_menu $w
    }

    tracer LEAVING
}

#--------------------------------------------
proc build_fors_menu {w instrument {del 0}} {
#--------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# del:        flag to clear menu button m_b and return

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    # DEBUG INI
    # puts stdout "add_fors_menu w=$w instrument=$instrument del=$del"
    # puts stdout "[$w component]"
    # puts stdout "[$w component image]"
    # puts stdout "[$w component image component]"
    # puts stdout "[lindex [$w component image info variable canvas_] 4]"
    # puts stdout "[lindex [$w component image info variable image_ ] 4]"
    # DEBUG END

    set i 0
    set mi($i) {index        label               help                                      type      command submenu toff init      var}; incr i
    set mi($i) {EnterMode    {Enter Mode}        {{Create a FORS Instrumental Mask }}      cascade   {}      create  0    normal    {} }; incr i
    set mi($i) {ResetMode    {Reset Mode}        {{Reset the current FORS Mask}}           command   {}      {}      {}   disabled  {} }; incr i
    set mi($i) {Refresh      {Refresh Mask}      {{Redraws the current FORS mask}}         command   {}      {}      {}   disabled  {} }; incr i
    set mi($i) {QuitMode     {Quit Mode}         {{Delete the current FORS mask}}          command   {}      {}      {}   disabled  {} }; incr i
    set mi($i) {separator_1  {}                  {}                                        separator {}      {}      {}   {}        {} }; incr i
    set mi($i) {QuitInstr    {Quit Instrument}   {{Go back to Instrument menu}}            command   {}      {}      {}   normal    {} }; incr i
    set mi($i) {separator_2  {}                  {}                                        separator {}      {}      {}   {}        {} }; incr i
    set mi($i) {LoadSetup    {Load SetUp}        {{Load a Mask from TAF}}                  command   {}      {}      {}   disabled  {} }; incr i
    set mi($i) {AutoSlit     {Auto Slit}         {{Compare target list with MOS slitlets}} cascade   {}      autopos 0    disabled  {} }; incr i
    set mi($i) {RefTargets   {Reference Targets} {{Edit reference target list}}            cascade   {}      reftarg 0    disabled  {} }; incr i
    set mi($i) {SaveSetup    {Save SetUp}        {{Save the current FORS Mask}}            command   {}      {}      {}   disabled  {} }; incr i
    set mi($i) {separator_3  {}                  {}                                        separator {}      {}      {}   {}        {} }; incr i
    set mi($i) {StatusDemo   {Status / Demo}     {{Demo Test Info}}                        cascade   {}      misc    0    disabled  {} }; incr i
    set mi($i) {ConfigLayout {Config / Layout}   {{Graphical Layout of the FORS Mask}}     cascade   {}      layout  1    disabled  {} }; set t $i

    # set mil {}
    # for { set i 1 } { $i <= $t } { incr i } {
    #     lappend mil $i
    # }
    # Item 6 and separator 7 not used for the time being
    set mil {}
    lappend mil 1 2 3 4 5 8 9 10 11 12 13 14

    # Local menu vector
    set mv $m_b

    # Clear menu items
    # - Clear sub-menus first
    for { set i 1 } { $i <= $t } { incr i } {
        set submenu [lindex $mi($i) [lsearch -exact $mi(0) submenu]]
        if { [llength $submenu] > 0 } {
            if { [winfo exists $mv.$submenu] } {
                $mv.$submenu delete 0 last
                destroy $mv.$submenu 
            }
        }
    }
    # - Clear this menu last
    $mv delete 0 last
    # - Clear menu item index array
    if { [array exists m_i] } {
         array unset m_i
    }

    if { $del } {
        # It was invoked only to clear the m_b menus
        tracer LEAVING DEL
        return
    }

    # For the time being, only FORS2 uses FIMS with menu label FORS
    # $w configure_menubutton instrument -text $instrument
    $w configure_menubutton instrument -text "FORS"

    # Menu item indexation
    set mii 0
    foreach i $mil {
        # Store the position of the item button in the menu (sequential 'mii')
        # and of the item name in the 'mi' table (subset 'mil' indexed by 'i')
        # Arrays are referenced by the item name located in the 'index' column.
        set item_name [lindex $mi($i) [lsearch -exact $mi(0) index]]
        set item_type [string tolower [lindex $mi($i) [lsearch -exact $mi(0) type]]]
        set m_i($item_name) $mii
        set m_n($item_name) $i
        incr mii
        switch -exact -- $item_type {
            cascade {
                menu_Cascade $w $mv $mi(0) $mi($i)
                switch -exact -- $item_name {
                    EnterMode {
                        build_fors_menu_EnterMode $w $instrument
                    }
                    AutoSlit {
                        # Submenu is built by the constructor of the FORS class
                        # build_fors_menu_AutoSlit $w $instrument $mode
                    }
                    RefTargets {
                        # Submenu is built by the constructor of the FORS class
                        # build_fors_menu_RefTargets $w $instrument $mode
                    }
                    StatusDemo {
                        build_fors_menu_StatusDemo $w
                    }
                    ConfigLayout {
                        # Submenu is built by the constructor of the FORS class
                        # build_fors_menu_Layout $w $instrument $mode
                    }
                    default {
                    }
                }
            }
            separator {
                $mv add separator
            }
            default {
                switch -exact -- $item_name {
                    ResetMode {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask reset_mask}}
                    }
                    LoadSetup {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask load_TPX 0}}
                    }
                    QuitMode {
                        menu_Command $w $mv $mi(0) $mi($i) [code f_off $w all $m_b]
                    }
                    Refresh {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask update_and_replot}}
                    }
                    QuitInstr {
                        # For the time being, only FORS2 uses FIMS,
                        # "Quit Instrument" option is not used.
                        # Delete current instrument menu and add the instrument selection menu
                        # menu_Command $w $mv $mi(0) $mi($i) [code build_ins_menu $w $instrument 1]
                    }
                    SaveSetup {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask save_TPX}}
                    }
                    default {
                    }
                }
            }
        }
    }

    tracer LEAVING
}

#----------------------------------------------
proc build_fors_menu_EnterMode {w instrument} {
#----------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)

    tracer ENTERING

    global ::m_b
    global ::m_i

    set i 0
    set mi($i) {index           label  help                         type    command submenu toff init   var}; incr i
    set mi($i) {EnterMode_HITMS HIT-MS {{Create a new HIT-MS mask}} command hitms   {}      {}   normal {} }; incr i
    set mi($i) {EnterMode_HITS  HITS   {{Create a new HIS mask}}    command hits    {}      {}   normal {} }; incr i
    set mi($i) {EnterMode_IMG   IMG    {{Create a new IMG mask}}    command img     {}      {}   normal {} }; incr i
    set mi($i) {EnterMode_IPOL  IPOL   {{Create a new IPOL mask}}   command ipol    {}      {}   normal {} }; incr i
    set mi($i) {EnterMode_LSS   LSS    {{Create a new LSS mask}}    command lss     {}      {}   normal {} }; incr i
    set mi($i) {EnterMode_MOS   MOS    {{Create a new MOS mask}}    command mos     {}      {}   normal {} }; incr i
    set mi($i) {EnterMode_MXU   MXU    {{Create a new MXU mask}}    command mxu     {}      {}   normal {} }; incr i
    set mi($i) {EnterMode_PMOS  PMOS   {{Create a new PMOS mask}}   command pmos    {}      {}   normal {} }; set t $i
    # Active menu items list
    # set mil {}
    # for { set i 1 } { $i <= $t } { incr i } {
    #     lappend mil $i
    # }

    # Define the active menu items for this instrument
    set mil {}
    switch -exact -- $instrument {
        FORS1 {
            lappend mil 3 6 8
        }
        FORS2 {
            lappend mil 1 3 6 7
        }
        FORS2+1 {
            # lappend mil 1 2 3 4 5 6 7 8
            lappend mil 1 3 6 7 8
        }
        default {
            lappend mil 3
        }
    }
 
    # Local menu vector
    set mv $m_b.create

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0
    foreach i $mil {
        set item_name [lindex $mi($i) [lsearch -exact $mi(0) index]]
        set item_type [string tolower [lindex $mi($i) [lsearch -exact $mi(0) type]]]
        set m_i($item_name) $mii
        set m_n($item_name) $i
        incr mii
        switch -exact -- $item_type {
            cascade {
                menu_Cascade $w $mv $mi(0) $mi($i)
                switch -exact -- $item_name {
                    default {
                    }
                }
            }
            separator {
                $mv add separator
            }
            default {
                switch -exact -- $item_name {
                    default {
                        menu_Command $w $mv $mi(0) $mi($i) [code f_on $w $instrument [lindex $mi($i) 4] $m_b]
                    }
                }
            }
        }
    }

    tracer LEAVING
}

#---------------------------------------------------
proc build_fors_menu_AutoSlit {w instrument modus} {
#---------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index           label  help                                type    command submenu toff init   var}; incr i
    set mi($i) {AutoSlit_SIMPLE SIMPLE {{do not rotate or shift the mask}} command simple  {}      {}   normal {} }; incr i
    set mi($i) {AutoSlit_STATIC STATIC {{do not rotate or shift the mask}} command static  {}      {}   normal {} }; incr i
    set mi($i) {AutoSlit_SHIFT  SHIFT  {{shift the mask, do no rotate}}    command shift   {}      {}   normal {} }; incr i
    set mi($i) {AutoSlit_ROTAT  ROTATE {{rotate the mask, do not shift}}   command rotat   {}      {}   normal {} }; incr i
    set mi($i) {AutoSlit_ALL    ALL    {{shift and rotate the mask}}       command all     {}      {}   normal {} }; set t $i
    # Active menu items list
    # set mil {}
    # for { set i 1 } { $i <= $t } { incr i } {
    #      lappend mil $i
    # }

    # Define the active menu items for this instrument and mode
    set mil {}
    switch -exact -- $instrument {
        FORS1 {
            lappend mil 2
        }
        FORS2 {
            lappend mil 1 2
        }
        FORS2+1 {
            lappend mil 1 2
        }
        default {
        }
    }
 
    switch -exact -- $modus {
        hitms {
            set mil {1}
        }
        # This works as an exclusion
        mxu   {
            # Do nothing
        }
        default {
            lappend mil 3 4 5
        }
    }
 
    # Local menu vector
    set mv $m_b.autopos

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0
    foreach i $mil {
        set item_name [lindex $mi($i) [lsearch -exact $mi(0) index]]
        set item_type [string tolower [lindex $mi($i) [lsearch -exact $mi(0) type]]]
        set m_i($item_name) $mii
        set m_n($item_name) $i
        incr mii
        switch -exact -- $item_type {
            cascade {
                menu_Cascade $w $mv $mi(0) $mi($i)
                switch -exact -- $item_name {
                    default {
                    }
                }
            }
            separator {
                $mv add separator
            }
            default {
                switch -exact -- $item_name {
                    default {
                        menu_Command $w $mv $mi(0) $mi($i) "catch {$my_mask AutoPos [lindex $mi($i) 4]}"
                    }
                }
            }
        }
    }

    # Auto Slit is not used in IMG
    if { [string compare $modus img] == 0 } {
        $m_b entryconfigure $m_i(AutoSlit) -state disabled
    }

    tracer LEAVING
}

#-----------------------------------------------------
proc build_fors_menu_RefTargets {w instrument modus} {
#-----------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index             label  help                                type    command submenu toff init     var}; incr i
    set mi($i) {RefTarg_plot   Plot   {{Plot reference targets}}             command plot    {}      {}   normal   {} }; incr i
    set mi($i) {RefTarg_show   Show   {{Show reference targets}}             command show    {}      {}   normal   {} }; incr i
    set mi($i) {RefTarg_add    Add    {{Add reference targets}}              command add     {}      {}   normal   {} }; incr i
    set mi($i) {RefTarg_purge  Purge  {{Purge reference targets}}            command purge   {}      {}   normal   {} }; incr i
    set mi($i) {RefTarg_verify Verify {{Verify reference targets}}           command verify  {}      {}   normal   {} }; incr i
    set mi($i) {RefTarg_reset  Reset  {{Purge all reference targets}}        command reset   {}      {}   normal   {} }; incr i
    set mi($i) {RefTarg_MOS10  MOS10  {{Switch the MOS-10 alignment option}} command mtali   {}      {}   disabled {} }; set t $i
    # Active menu items list
    set mil {}
    for { set i 1 } { $i <= $t } { incr i } {
        lappend mil $i
    }

    # Local menu vector
    set mv $m_b.reftarg

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0
    foreach i $mil {
        set item_name [lindex $mi($i) [lsearch -exact $mi(0) index]]
        set item_type [string tolower [lindex $mi($i) [lsearch -exact $mi(0) type]]]
        set m_i($item_name) $mii
        set m_n($item_name) $i
        incr mii
        switch -exact -- $item_type {
            cascade {
                menu_Cascade $w $mv $mi(0) $mi($i)
                switch -exact -- $item_name {
                    default {
                    }
                }
            }
            separator {
                $mv add separator
            }
            default {
                switch -exact -- $item_name {
                    default {
                        menu_Command $w $mv $mi(0) $mi($i) "catch {$my_mask edit_ref_targ [lindex $mi($i) 4]}"
                    }
                }
            }
        }
    }

    # Define the active menu items for this instrument and mode
    switch -exact -- $modus {
        mos  -
        pmos {
            $mv entryconfigure $m_i(RefTarg_MOS10) -state normal
        }
        default {
        }
    }
 
    tracer LEAVING
}

#------------------------------------
proc build_fors_menu_StatusDemo {w} {
#------------------------------------
# w: skycat instance 

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index             label  help                       type    command     submenu toff init   var}; incr i
    set mi($i) {StatusDemo_Status Status {{Shows the Status Panel}} command show_status {}      {}   normal {} }; incr i
    set mi($i) {StatusDemo_Demo   Demo   {{Brief demonstration}}    command demo        {}      {}   normal {} }; set t $i
    # Active menu items list
    set mil {}
    for { set i 1 } { $i <= $t } { incr i } {
        lappend mil $i
    }

    # Local menu vector
    set mv $m_b.misc

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0
    foreach i $mil {
        set item_name [lindex $mi($i) [lsearch -exact $mi(0) index]]
        set item_type [string tolower [lindex $mi($i) [lsearch -exact $mi(0) type]]]
        set m_i($item_name) $mii
        set m_n($item_name) $i
        incr mii
        switch -exact -- $item_type {
            cascade {
                menu_Cascade $w $mv $mi(0) $mi($i)
                switch -exact -- $item_name {
                    default {
                    }
                }
            }
            separator {
                $mv add separator
            }
            default {
                switch -exact -- $item_name {
                    default {
                        menu_Command $w $mv $mi(0) $mi($i) "catch {$my_mask [lindex $mi($i) 4]}"
                    }
                }
            }
        }
    }
 
    tracer LEAVING
}

#-------------------------------------------------
proc build_fors_menu_Layout {w instrument modus} {
#-------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index                  label              help                                 type      command submenu toff init     var   }; incr i
    set mi($i) {Layout_SlitWidth       {Slit Width}       {{Configure the slit width}}         cascade   {}      sw      0    disabled {}    }; incr i
    set mi($i) {Layout_SlitLength      {Slit Length}      {{Configure the slit length}}        cascade   {}      sl      0    disabled {}    }; incr i
    set mi($i) {Layout_SlitShape       {Slit Shape}       {{Configure the slit aperture}}      cascade   {}      ss      0    disabled {}    }; incr i
    set mi($i) {Layout_Collimator      {Collimator}       {{Configure the collimator}}         cascade   {}      sh      0    disabled {}    }; incr i
    set mi($i) {Layout_OSFilter        {OS Filter}        {{Configure the OS filter}}          cascade   {}      fi      0    disabled {}    }; incr i
    set mi($i) {Layout_Grism           {Grism}            {{Configure the grism}}              cascade   {}      gr      0    disabled {}    }; incr i
    set mi($i) {Layout_Camera          {Camera}           {{Configure the camera}}             cascade   {}      cm      0    disabled {}    }; incr i
    set mi($i) {Layout_WaveLengthRange {WaveLength Range} {{Configure the wavelength range}}   command   {}      {}      {}   disabled {}    }; incr i
    set mi($i) {separator_4            {}                 {}                                   separator {}      {}      {}   {}       {}    }; incr i
    # set mi($i) {Layout_Open_Blades     {Open Blades}      {{Move all blades to open pos.}}     command   {}      {}      {}   disabled {}    }; incr i
    # set mi($i) {Layout_Close_Blades    {Close Blades}     {{Move all blades to closed pos.}}   command   {}      {}      {}   disabled {}    }; incr i
    set mi($i) {Layout_Reset           {Reset Layout}     {{Configure the default options}}    command   {}      {}      {}   normal   {}    }; incr i
    set mi($i) {Layout_RotationStep    {Rotation Step}    {{Configure the mask rotation step}} cascade   {}      rs      0    normal   {}    }; incr i
    set mi($i) {Layout_Color           {Color}            {{Configure the mask color}}         cascade   {}      cs      0    normal   {}    }; incr i
    set mi($i) {Layout_Transparency    {Transparency}     {{Show positioned slit blades}}      check     {}      {}      0    normal   transp}; incr i
    set mi($i) {Layout_BadPixelMap     {Bad Pixel Map}    {{Plot the bad pixels map}}          check     {}      {}      0    normal   bpm   }; incr i
    set mi($i) {Layout_Labels          {Labels}           {{Label the mask}}                   check     {}      {}      0    normal   lbl   }; incr i
    set mi($i) {Layout_CenterSlit      {Center Slit}      {{Use Pick Object to center slit}}   check     {}      {}      0    disabled pisc  }; incr i
    set mi($i) {Layout_ProjectName     {Project Name}     {{File name to be saved}}            command   {}      {}      {}   normal   {}    }; incr i
    set mi($i) {Layout_SetUpDir        {Setup Directory}  {{Directory to save project}}        command   {}      {}      {}   normal   {}    }; set t $i

    # Active menu items list
    set mil {}
    for { set i 1 } { $i <= $t } { incr i } {
        lappend mil $i
    }

    # Local menu vector
    set mv $m_b.layout

    # Clear menu items
    # - Clear sub-menus first
    for { set i 1 } { $i <= $t } { incr i } {
        set submenu [lindex $mi($i) [lsearch -exact $mi(0) submenu]]
        if { [llength $submenu] > 0 } {
            if { [winfo exists $mv.$submenu] } {
                $mv.$submenu delete 0 last
                destroy $mv.$submenu 
            }
        }
    }
    # - Clear this menu last
    $mv delete 0 last

    # Menu item indexation
    set mii 1; # 0 is for tearoff
    foreach i $mil {
        set item_name [lindex $mi($i) [lsearch -exact $mi(0) index]]
        set item_type [string tolower [lindex $mi($i) [lsearch -exact $mi(0) type]]]
        set m_i($item_name) $mii
        set m_n($item_name) $i
        incr mii
        # Add the menus
        switch -exact -- $item_type {
            cascade {
                menu_Cascade $w $mv $mi(0) $mi($i)
                switch -exact -- $item_name {
                    Layout_SlitWidth {
                        build_fors_menu_SlitWidth $w $instrument $modus
                    }
                    Layout_SlitLength {
                        build_fors_menu_SlitLength $w $instrument $modus
                    }
                    Layout_SlitShape {
                        build_fors_menu_SlitShape $w $instrument $modus
                    }
                    Layout_Collimator {
                        # Invoked from load_instrument method of FORS class
                        # build_fors_menu_Collimator $w $instrument $modus
                    }
                    Layout_OSFilter {
                        # Invoked from load_instrument method of FORS class
                        # build_fors_menu_Filter $w $instrument $modus
                    }
                    Layout_Grism {
                        # Invoked from load_instrument method of FORS class
                        # build_fors_menu_Grism $w $instrument $modus
                    }
                    Layout_Camera {
                        # Invoked from load_instrument method of FORS class
                        # build_fors_menu_Camera $w $instrument $modus
                    }
                    Layout_RotationStep {
                        build_fors_menu_Rotation $w $instrument $modus
                    }
                    Layout_Color {
                        build_fors_menu_Color $w $instrument $modus
                    }
                    default {
                    }
                }
            }
            separator {
                $mv add separator
            }
            default {
                switch -exact -- $item_name {
                    Layout_WaveLengthRange {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask wave_length}}
                    }
                    Layout_Reset {
                        menu_Command $w $mv $mi(0) $mi($i) {[code reset_layout $my_mask]}
                    }
                    Layout_Transparency {
                        menu_Check   $w $mv $mi(0) $mi($i) {catch {$my_mask configure -graytype $transp}}
                    }
                    Layout_BadPixelMap {
                        menu_Check   $w $mv $mi(0) $mi($i) {catch {$my_mask configure -badpixelmap $bpm}}
                    }
                    Layout_Labels {
                        menu_Check   $w $mv $mi(0) $mi($i) {catch {$my_mask configure -label $lbl}}
                    }
                    Layout_CenterSlit {
                        menu_Check   $w $mv $mi(0) $mi($i) {catch {$my_mask configure -pics $pisc}}
                    }
                    Layout_Open_Blades {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask open_blades}}
                    }
                    Layout_Close_Blades {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask close_blades}}
                    }
                    Layout_ProjectName {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask select_filename}}
                    }
                    Layout_SetUpDir {
                        menu_Command $w $mv $mi(0) $mi($i) {catch {$my_mask select_set_dir}}
                    }
                    default {
                    }
                }
            }
        }
    }

    # Activate tear off
    $mv entryconfigure 0 -state normal

    # Activate menu items for this instrument and mode
    switch -exact -- $instrument {
        FORS2+1 {
            $mv entryconfigure $m_i(Layout_Camera)          -state normal
        }
        default {
        }
    }
 
    switch -exact -- $modus {
        echs {
            $mv entryconfigure $m_i(Layout_SlitWidth)       -state normal
            $mv entryconfigure $m_i(Layout_Grism)           -state normal
            $mv entryconfigure $m_i(Layout_CenterSlit)      -state normal
        }
        img {
            $mv entryconfigure $m_i(Layout_Collimator)      -state normal
            # $mv entryconfigure $m_i(Layout_Open_Blades)     -state normal
            # $mv entryconfigure $m_i(Layout_Close_Blades)    -state normal
        }
        mxu  {
            $mv entryconfigure $m_i(Layout_SlitWidth)       -state normal
            $mv entryconfigure $m_i(Layout_SlitLength)      -state normal
            $mv entryconfigure $m_i(Layout_SlitShape)       -state normal
            $mv entryconfigure $m_i(Layout_OSFilter)        -state normal
            $mv entryconfigure $m_i(Layout_Grism)           -state normal
            $mv entryconfigure $m_i(Layout_WaveLengthRange) -state normal
            $mv entryconfigure $m_i(Layout_CenterSlit)      -state normal
        }
        hitms {
            $mv entryconfigure $m_i(Layout_SlitWidth)       -state normal
            $mv entryconfigure $m_i(Layout_SlitLength)      -state normal
            $mv entryconfigure $m_i(Layout_OSFilter)        -state normal
            $mv entryconfigure $m_i(Layout_Grism)           -state normal
            $mv entryconfigure $m_i(Layout_WaveLengthRange) -state normal
            $mv entryconfigure $m_i(Layout_CenterSlit)      -state normal
        }
        mos  -
        pmos {
            $mv entryconfigure $m_i(Layout_Collimator)      -state normal
            $mv entryconfigure $m_i(Layout_SlitWidth)       -state normal
            $mv entryconfigure $m_i(Layout_SlitLength)      -state normal
            $mv entryconfigure $m_i(Layout_OSFilter)        -state normal
            $mv entryconfigure $m_i(Layout_Grism)           -state normal
            $mv entryconfigure $m_i(Layout_WaveLengthRange) -state normal
            $mv entryconfigure $m_i(Layout_CenterSlit)      -state normal
            # $mv entryconfigure $m_i(Layout_Open_Blades)     -state normal
            # $mv entryconfigure $m_i(Layout_Close_Blades)    -state normal
        }
        hits  -
        lss   {
            $mv entryconfigure $m_i(Layout_Collimator)      -state normal
            $mv entryconfigure $m_i(Layout_SlitWidth)       -state normal
            $mv entryconfigure $m_i(Layout_OSFilter)        -state normal
            $mv entryconfigure $m_i(Layout_Grism)           -state normal
            $mv entryconfigure $m_i(Layout_WaveLengthRange) -state normal
            $mv entryconfigure $m_i(Layout_CenterSlit)      -state normal
        }
        default {
        }
    }

    tracer LEAVING
}

#----------------------------------------------------
proc build_fors_menu_SlitWidth {w instrument modus} {
#----------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index label help type command submenu toff init var}; incr i
    set mi($i) {{}    {}    {}   {}   {}      {}      {}   {}   {} }; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.sw

    # Clear menu items
    $mv delete 0 last

    # Define the active menu items for this instrument and mode
    switch -exact -- $modus {
        echs {
            set swl { 0.3 0.4 0.5 0.6 0.7 0.8 1.0 1.2 1.4 1.7 2.0 2.5 3.0 3.5 4.0 5.0 10.0 22.0 }
        }
        hitms {
            set swl { 0.3 0.4 0.5 0.6 0.7 0.8 1.0 1.2 1.4 1.7 2.0 2.5 3.0 3.5 4.0 5.0 10.0 22.0 30.0 }
        }
        hits {
            set swl { all PH 0.5 0.7 1.0 1.3 1.6 2.0 }
        }
        lss {
            set swl { all 0.3 0.4 0.5 0.7 1.0 1.3 1.6 2.0 2.5 }
        }
        mxu {
            set swl { 0.3 0.4 0.5 0.6 0.7 0.8 1.0 1.2 1.4 1.7 2.0 2.5 3.0 3.5 4.0 5.0 10.0 22.0 30.0 }
        } 
        mos  -
        pmos {
            set swl { 0.3 0.4 0.5 0.6 0.7 0.8 1.0 1.2 1.4 1.7 2.0 2.5 3.0 3.5 4.0 5.0 10.0 22.0 30.0 45.0 60.0 open }
        }
        default {
            set swl { none }
        }
    }
 
    switch -exact -- $modus {
        hitms -
        mos   -
        mxu   -
        pmos  {
            # Menu item indexation
            set mii 0
            foreach i $swl {
                set swid [string map {. _} $i]
                set m_i(Layout_SlitWidth_$swid) $mii
                incr mii
                # in particular MOS and PMOS
                # open means slitless spectroscopy
                $w add_menuitem $mv radio "  $i '' " \
                                          {Set Slit Width to $i} \
                                          -variable sw \
                                          -value $i \
                                          -command {catch {$my_mask configure -slit_width $sw}}
            }
        }
        hits -
        lss  {
            # Menu item indexation
            set mii 0
            foreach i $swl {
                set swid [string map {. _} $i]
                set m_i(Layout_SlitWidth_$swid) $mii
                incr mii
                # in particular LSS and HITS
                $w add_menuitem $mv radio "  $i '' " \
                                          {Set Long Slit Width to $i} \
                                          -variable sw \
                                          -value $i \
                                          -command {catch {$my_mask configure -long_width $sw}}
            }
        }
        default {
            # Menu item indexation
            set mii 0
            foreach i $swl {
                set swid [string map {. _} $i]
                set m_i(Layout_SlitWidth_$swid) $mii
                incr mii
                # in particular LSS and HITS
                $w add_menuitem $mv radio "  $i '' " \
                                          {Set Slit Width to $i} \
                                          -variable sw \
                                          -value $i \
                                          -command {catch {$my_mask configure -slit_width $sw}}
            }
        }
    }

    tracer LEAVING
}

#-----------------------------------------------------
proc build_fors_menu_SlitLength {w instrument modus} {
#-----------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index label help type command submenu toff init var}; incr i
    set mi($i) {{}    {}    {}   {}   {}      {}      {}   {}   {} }; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.sl

    # Clear menu items
    $mv delete 0 last

    # Define the active menu items for this instrument and mode
    switch -exact -- $modus {
        hitms {
            set sll { 5.0 10.0 }
        }
        default {
            set sll { real 18.0 16.0 14.0 12.0 10.0 8.0 6.0 4.0 2.0 1.0 0.5 0.2 }
        }
    }

    # Menu item indexation
    set mii 0
    foreach i $sll {
        set slen [string map {. _} $i]
        set m_i(Layout_SlitLength_$slen) $mii
        incr mii
        $w add_menuitem $mv radio "  $i ''" \
                                  {Set Slit Length to $i} \
                                  -variable sl \
                                  -value $i \
                                  -command {catch {$my_mask configure -slit_length $sl}}
    }

    tracer LEAVING
}

#----------------------------------------------------
proc build_fors_menu_SlitShape {w instrument modus} {
#----------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS2)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index label help type command submenu toff init var}; incr i
    set mi($i) {{}    {}    {}   {}   {}      {}      {}   {}   {} }; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.ss

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0 
    foreach i { STRAIGHT CIRCLE CURVED none } {
        set m_i(Layout_SlitShape_$i) $mii
        incr mii
        $w add_menuitem $mv radio " $i " \
                                  {Set Slit Shape to $i} \
                                  -variable mx \
                                  -value $i \
                                  -command {catch {$my_mask configure -slit_shape $mx}}
    }

    tracer LEAVING
}

#---------------------------------------------------------------
proc build_fors_menu_Collimator {w instrument modus col_range} {
#---------------------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)
# col_range:  list of collimators from ISF file

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index label help type command submenu toff init var}; incr i
    set mi($i) {{}    {}    {}   {}   {}      {}      {}   {}   {} }; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.sh

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0 
    # Generic labels for SR: Standard Resolution collimator
    #                    HR: High Resolution collimator
    set res_range {SR HR}
    foreach i $col_range {
        set m_i(Layout_Collimator_[lindex $res_range $mii]) $mii
        incr mii
        $w add_menuitem $mv radio " $i " \
                                  {Select [lindex $res_range $mii] Collimator} \
                                  -variable co \
                                  -value $i \
                                  -command {catch {$my_mask configure -collimat $co}}
    }

    # Define the active menu items for this instrument and mode
    switch -exact -- $modus {
        hitms -
        hits  -
        mxu   {
            # Disable HR collimator
            $mv entryconfigure $m_i(Layout_Collimator_HR) -state disabled
        }
        default {
        }
    }

    tracer LEAVING
}

#-----------------------------------------------------------
proc build_fors_menu_Filter {w instrument modus fil_range} {
#-----------------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)
# fil_range:  list of filters from ISF file

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index label help type command submenu toff init var}; incr i
    set mi($i) {{}    {}    {}   {}   {}      {}      {}   {}   {} }; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.fi

    # Filters are organized in decades. 
    # The first 10 are put in layout.fi menu (item index 0..9)
    # From next decade (item index 10..19) on, they are put in
    # cascade submenus layout.fi.m1,...,m4,... and so forth.

    # Clear menu items
    # - Clear sub-menus first
    for { set i 1 } { $i <= 9 } { incr i } {
        if { [winfo exists $mv.m$i] } {
            $mv.m$i delete 0 last
            destroy $mv.m$i
        }
    }
    # - Clear this menu last
    $mv delete 0 last

    # Menu item indexation
    set mii 0
    foreach i $fil_range {
        # Current decade 0,1...4,5,...
        set sm [expr {$mii / 10}]
        # Index from 0 to 9 for each menu item of the current decade
        set smi [expr {$mii % 10}]
        set m_i(Layout_Filter_m${sm}_$smi) $smi
        if { $sm == 0 } {
            # Add to layout.fi menu
            $w add_menuitem $mv radio "$i" \
                                {SetUp Instrument : Select OS Filter} \
                                -variable fi \
                                -value $i \
                                -command {catch {$my_mask configure -filter $fi}}
        } else {
            if { $smi == 0 } {
                # Create new layout.fi.m$sm menu
                $w add_menuitem $mv cascade "more Filters (${sm}1-[expr {$sm+1}]0)" \
                                            {Instrument SetUp : more Filters} \
                                            -state normal \
                                            -menu [menu $mv.m$sm]
            }
            # Add to layout.fi.m$sm menu
            $w add_menuitem $mv.m$sm radio "$i" \
                                           {SetUp Instrument : Select OS Filter} \
                                           -variable fi \
                                           -value $i  \
                                           -command {catch {$my_mask configure -filter $fi}}
        }
        incr mii
    }

    # Define the active menu items for this instrument and mode
    switch -exact -- $modus {
        default {
        }
    }

    tracer LEAVING
}

#----------------------------------------------------------------------
proc build_fors_menu_Grism {w instrument modus gris_range} {
#----------------------------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)
# gris_range: list of grisms from ISF file

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index label help type command submenu toff init var}; incr i
    set mi($i) {{}    {}    {}   {}   {}      {}      {}   {}   {} }; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.gr

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0 
    foreach i $gris_range {
        set m_i(Layout_Grism_$i) $mii
        incr mii
        $w add_menuitem $mv radio "$i" \
                                  {SetUp Instrument : Select Grism} \
                                  -variable gr \
                                  -value $i \
                                  -state disabled \
                                  -command {catch {$my_mask configure -grism $gr}}
    }

    # Define the active menu items for this instrument and mode
    switch -exact -- $instrument {
        default {
            switch -exact -- $modus {
                hitms {
                    foreach i $gris_range {
                        if { [string match XGRIS* $i] == 1 } {
                            $mv entryconfigure $m_i(Layout_Grism_$i) -state normal
                        }
                    }
                }
                default {
                    foreach i $gris_range {
                        $mv entryconfigure $m_i(Layout_Grism_$i) -state normal
                    }
                }
            }
        }
    }

    tracer LEAVING
}

#-----------------------------------------------------------
proc build_fors_menu_Camera {w instrument modus cam_range} {
#-----------------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index           label        help                    type  command submenu toff init   var}; incr i
    set mi($i) {Layout_Camera_B {E2V (Blue)} {{Use the Blue camera}} radio B       {}      {}   normal cam}; incr i
    set mi($i) {Layout_Camera_R {MIT (Red)}  {{Use the Red camera}}  radio R       {}      {}   normal cam}; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.cm

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0 
    for { set i 1 } { $i <= $t } { incr i } {
        set m_i([lindex $mi($i) [lsearch -exact $mi(0) index]]) $mii
        incr mii
        set camval [lindex $mi($i) [lsearch -exact $mi(0) command]]
        menu_Radio $w $mv $mi(0) $mi($i) $camval "[code camera_warning $w $my_mask $mv $camval]"
    }

    tracer LEAVING
}

#---------------------------------------------------
proc build_fors_menu_Rotation {w instrument modus} {
#---------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index label help type command submenu toff init var}; incr i
    set mi($i) {{}    {}    {}   {}   {}      {}      {}   {}   {} }; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.rs

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0 
    foreach i { 0.05 0.1 0.3 1 2 5 10 15 45 90 } {
        set rot_step [string map {. _} $i]
        set m_i(Layout_RotationStep_$rot_step) $mii
        incr mii
        $w add_menuitem $mv radio "  $i deg/click " \
                                  {Set Rotation Speed to $i} \
                                  -variable rs \
                                  -value $i \
                                  -command {catch {$my_mask configure -rotangle $rs}} 
                                            # {catch {$my_mask update_and_replot}}
    }

    tracer LEAVING
}

#------------------------------------------------
proc build_fors_menu_Color {w instrument modus} {
#------------------------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# modus:      instrumental mode (e.g. mos, lss)

    tracer ENTERING

    global ::m_b
    global ::m_i
    global my_mask

    set i 0
    set mi($i) {index label help type command submenu toff init var}; incr i
    set mi($i) {{}    {}    {}   {}   {}      {}      {}   {}   {} }; set t $i

    # Active menu items list
    set mil {}

    # Local menu vector
    set mv $m_b.layout.cs

    # Clear menu items
    $mv delete 0 last

    # Menu item indexation
    set mii 0 
    foreach i { \
                Black/White \
                Blue/White \
                Red/Yellow \
                Yellow/Blue \
                Yellow/White \
                Orange/Yellow \
                Purple/Orange \
                Bright \
                Dark \
                Gray \
              } {
        set color [string map {/ _} $i]
        set m_i(Layout_Color_$color) $mii
        incr mii
        $w add_menuitem $mv radio " $i " \
                                  {Set color of mask to $i}  \
                                  -variable cs \
                                  -value $i \
                                  -command {catch {$my_mask configure -colorset $cs}}
    }

    tracer LEAVING
}

#===============
# Menu Callbacks
#===============

#----------------------------------
proc f_on {w instrument mode m_v} {
#----------------------------------
# w:          skycat instance 
# instrument: instrument identifier (e.g. FORS1)
# mode:       instrumental mode (e.g. mos, lss)
# m_v:        external menu vector

    tracer ENTERING

    global ::env
    global ::m_i
    global my_mask
    global mw_list
    global ::skycat_version

    # puts stdout "f_on $w $instrument $mode $m_v"
    set mw_list ""
    # set s_v [eval skycat_version]
    set s_v $skycat_version

    if { [string compare $s_v $env(SKYCAT_VER)] } {
        set wid .skycat1.dm         
        catch {destroy $wid}
        set sv [DialogWidget $wid \
                             -title "Warning" \
                             -text "SkyCat Version Error:\n\
                                    Found : $s_v\n\
                                    Required: $env(SKYCAT_VER)" \
                             -bitmap warning \
                             -default -1 \
                             -buttons {OK Cancel} ]
        if { [$sv activate] } {
            # 'Cancel' was pressed
            tracer LEAVING "Cancel"
            return 
        }
    }

    # Debug flag to send to fims class
    set dbg 0
    if { [info exists env(FIMS_DEBUG)] } {
        if { [string compare -nocase -length 3 $env(FIMS_DEBUG) "off"] != 0 } {
            set dbg 1
        }
    }

    # Verbosity flag to send to fims class
    set vbs 0
    if { [info exists env(FIMS_VERBOSE)] } {
        if { [string compare -nocase -length 3 $env(FIMS_VERBOSE) "off"] != 0 } {
            set vbs 1
        }
    }

    # Disable the skycat pickobject button 
    set p [$w component view]
    $p.m  entryconfigure 3 -state disabled

    set canvas_ [lindex [$w component image info variable canvas_] 4]
    set image_  [lindex [$w component image info variable image_ ] 4]

    # Ask for the options we want to preserve from previous mode to this one.
    # - Remember that some menus are regenerated upon entering this new mode.
    # - To set the option we need to read the value of the checkbutton/radio
    #
    # ask for camera option
    if { [info exists m_i(Layout_Camera_B)] } {
        set loc_var_name [$m_v.layout.cm entrycget $m_i(Layout_Camera_B) -variable]
        global $loc_var_name
        set doit [concat set camera $$loc_var_name]
        eval $doit
    } else {
        # The first time get it from environment variable
        if { [info exists env(FORS_CAM)] } {
            set camera $env(FORS_CAM)
        } else {
            set camera R
        }
    }

    # ask for rotation step radio button state
    if { [info exists m_i(Layout_RotationStep_90)] } {
        set loc_var_name [$m_v.layout.rs entrycget $m_i(Layout_RotationStep_90) -variable]
        global $loc_var_name
        set doit [concat set l_rotation $$loc_var_name]
        eval $doit
    } else {
        set l_rotation 5
    }

    # ask for color radio button state
    if { [info exists m_i(Layout_Color_Dark)] } {
        set loc_var_name [$m_v.layout.cs entrycget $m_i(Layout_Color_Dark) -variable]
        global $loc_var_name
        set doit [concat set l_color $$loc_var_name]
        eval $doit
    } else {
        set l_color "Yellow/Blue"
    }

    # ask for transparency check button state 
    if { [info exists m_i(Layout_Transparency)] } {
        set loc_var_name [$m_v.layout entrycget $m_i(Layout_Transparency) -variable]
        global $loc_var_name
        set doit [concat set l_transp $$loc_var_name]
        eval $doit
    } else {
        set l_transp 0
    }

    # ask for bad pixel map check button state
    if { [info exists m_i(Layout_BadPixelMap)] } {
        set loc_var_name [$m_v.layout entrycget $m_i(Layout_BadPixelMap) -variable]
        global $loc_var_name
        set doit [concat set l_BPM $$loc_var_name]
        eval $doit
    } else {
        set l_BPM 0
    }

    # ask for labels check button state
    if { [info exists m_i(Layout_Labels)] } {
        set loc_var_name [$m_v.layout entrycget $m_i(Layout_Labels) -variable]
        global $loc_var_name
        set doit [concat set l_labels $$loc_var_name]
        eval $doit
    } else {
        set l_labels 0
    }

    # ask for center slit check button state
    if { [info exists m_i(Layout_CenterSlit)] } {
        set loc_var_name [$m_v.layout entrycget $m_i(Layout_CenterSlit) -variable]
        global $loc_var_name
        set doit [concat set l_center $$loc_var_name]
        eval $doit
    } else {
        set l_center 0
    }

    # enable/disable menu buttons 
    $m_v entryconfigure $m_i(EnterMode)    -state disabled
    $m_v entryconfigure $m_i(ResetMode)    -state normal
    $m_v entryconfigure $m_i(LoadSetup)    -state normal
    $m_v entryconfigure $m_i(QuitMode)     -state normal
    $m_v entryconfigure $m_i(Refresh)      -state normal
    # separator
    # Not used for the time being
    # $m_v entryconfigure $m_i(QuitInstr)    -state disabled
    # separator
    $m_v entryconfigure $m_i(AutoSlit)     -state normal
    $m_v entryconfigure $m_i(SaveSetup)    -state normal
    $m_v entryconfigure $m_i(RefTargets)   -state normal
    $m_v entryconfigure $m_i(StatusDemo)   -state normal
    # separator
    $m_v entryconfigure $m_i(ConfigLayout) -state normal

    # create one mask instance
    #......................................................................
    fors::FORS::FORSInit $w $canvas_ $image_ $m_v $instrument $camera $mode \
                         $l_rotation \
                         $l_color \
                         $l_transp \
                         $l_BPM \
                         $l_labels \
                         $l_center \
                         $dbg \
                         $vbs \
                         1
    #......................................................................

    set instrument [$my_mask cget -instrument]
    tracer LEAVING
}

#------------------------
proc f_off {w mode m_v} {
#------------------------
# w:    skycat instance 
# mode: instrumental mode (e.g. mos, lss)
# m_v:  menu vector

    tracer ENTERING

    global ::env
    global ::m_i
    global my_mask
    global mw_list

    set ln {none}
    catch { set ln [lindex [split [$my_mask info variable last_save_fn] " "] 4] }

    if { [string length $ln] < 3 && [$my_mask cget -interactive] } {
        # Warning, you want to delete mask but setup not yet saved
        set wid .skycat1.dem
        catch {destroy $wid}
        set dew [DialogWidget $wid \
                              -title "Warning" \
                              -text "No Setup Saved !\n\n\
                                     Really Delete Mask ?" \
                              -bitmap warning \
                              -default -1 \
                              -buttons {Yes No} \
                ]
        if { [$dew activate] } {
            tracer LEAVING 1
            return
        }
    }

    # Enable the skycat pickobject button 
    set p [$w component view]
    $p.m  entryconfigure 3 -state normal

    set canvas_ [lindex [$w component image info variable canvas_] 4]
    set image_  [lindex [$w component image info variable image_ ] 4]

    # Destroy slit parameter editor panels left around this mask
    if { [llength mw_list] > 0 } {
        foreach mw $mw_list {
            if { [info exists mw] } {
                catch { destroy $mw }
            }
        }
        set mw_list ""
    }

    set instrument [$my_mask cget -instrument]
    # Delete object FORS::my_mask
    #................................................................
    fors::FORS::FORSInit $w $canvas_ $image_ $m_v $instrument R $mode \
                         0 \
                         "" \
                         0 \
                         0 \
                         0 \
                         0 \
                         0 \
                         0 \
                         0
    #................................................................

    # Set menu items to default state
    $m_v entryconfigure $m_i(EnterMode)    -state normal
    $m_v entryconfigure $m_i(ResetMode)    -state disabled
    $m_v entryconfigure $m_i(LoadSetup)    -state disabled
    $m_v entryconfigure $m_i(QuitMode)     -state disabled
    $m_v entryconfigure $m_i(Refresh)      -state disabled
    # separator
    # Not used for the time being
    # $m_v entryconfigure $m_i(QuitInstr)    -state normal
    # separator
    $m_v entryconfigure $m_i(AutoSlit)     -state disabled
    $m_v entryconfigure $m_i(SaveSetup)    -state disabled
    $m_v entryconfigure $m_i(RefTargets)   -state disabled
    $m_v entryconfigure $m_i(StatusDemo)   -state disabled
    # separator
    $m_v entryconfigure $m_i(ConfigLayout) -state disabled

    tracer LEAVING
}

#----------------------
proc reset_layout {m} {
#----------------------
# m: mask instance

    tracer ENTERING

    # Each option being configured takes care of alignment with the
    # corresponding menu button status
    catch {$m configure -rotangle 5}
    catch {$m configure -colorset Yellow/Blue}
    catch {$m configure -graytype 0}
    catch {$m configure -badpixelmap 0}
    catch {$m configure -label 0}
    catch {$m configure -pics 0}

    tracer LEAVING
}

#---------------------------------
proc camera_warning {w m mv cam} {
#---------------------------------
# w:   skycat instance
# m:   mask instance
# mv:  menu vector
# cam: new camera to configure

    tracer ENTERING

    global ::m_i

    if { [string compare $cam [$m cget -camera]] != 0 } {
        set wid $w.dcam
        catch {destroy $wid}
        set dw [DialogWidget $wid \
                             -title "Warning" \
                             -text "This Will Reset Your Mask !\n\n\
                                    Really Change Camera?" \
                             -bitmap warning \
                             -default -1 \
                             -buttons {Yes No} ]
        if { [$dw activate] } {
            # If answer was No, reselect the current camera in the menu
            $mv invoke $m_i(Layout_Camera_[$m cget -camera])
            tracer LEAVING 1
            return
        }
        # answer was Yes, configure new camera
        tracer MSG "[tracer TAB  ]Configuring new camera option: $cam"
        catch { $m configure -camera $cam }
    } else {
        tracer MSG "[tracer TAB  ]Camera option was the same: $cam"
    }
    tracer LEAVING
}

#--------------------
proc fims_about {w} {
#--------------------
# w: skycat instance
# This is a copy of skycat_about with some modifications
    
    tracer ENTERING

    global ::env
    global ::about_fors
    global fors_logo

    # look for home directory
    set file $env(FIMSROOT)
    append file "/../bitmap/fors-logo.gif"
    # create the FoRS logo
    if { [catch {
              # create a dummy rtdimage to make sure the colormap is initialized before
              # the logo is created to avoid color problems...
              image delete [image create rtdimage]
              set fors_logo [image create photo \
                                   -format GIF \
                                   -file $file]
          } msg] } {
    }

    # set the text string for "About FIMS ..."
    set vf [eval fims_version]
    set version [lindex $vf 0]
    set last_day [lindex $vf 1]
    set last_time [lindex $vf 2]
    set about_fors [format "\n%s\n\n%s\n\n%s\n\n%s\n\n%s\n\n%s\n\n%s\n\n%s\n" \
                           "FORS Instrumental Mask Simulator (FIMS)" \
                           "SVN $version " \
                           "Copyright (C ) 1998 - 2014 ESO - European Southern Observatory" \
                           "usg@eso.org" \
                           "Author:" \
                           "Wolfgang Hummel " \
                           "USM - University Sternwarte Munich" \
                           "Last modification: $last_day  $last_time" \
                   ]

    # the rest is original      
    if { ![winfo exists $w.image.about] } {
        if { ![info exists fors_logo] || "fors_logo" == "" } {
            set bitmap info
            set logo ""
        } else {
            set bitmap ""
            set logo $fors_logo
        }
        catch {destroy $w.image.about}
        DialogWidget $w.image.about \
                     -image $logo \
                     -bitmap $bitmap \
                     -messagewidth 6i \
                     -justify center \
                     -text $about_fors \
                     -title "About FIMS"
    }
    $w.image.about activate

    tracer LEAVING
}

#---------------------
proc masks_about {w} {
#---------------------
# w: skycat instance
# This is a copy of skycat_about with some modifications

    tracer ENTERING

    global ::env 
    global fors_logo

    # look for home directory
    set file $env(FIMSROOT)
    append file "/../bitmap/fors-logo-small.gif"

    # create the FoRS logo
    if { [catch {
              # create a dummy rtdimage to make sure the colormap is initialized before
              # the logo is created to avoid color problems...
              image delete [image create rtdimage]
              set fors_logo [image create photo \
                                   -format GIF \
                                   -file $file]
          } msg] } {
    }
    # set the text string for "About Masks..."
    set about_masks "
    MASKID\t\t MASK NAME\t\t MODE\t description\n
--------------------------------------------------------------------------\n"
    set contents ""
    set myfile $env(FIMSROOT)
    append myfile "/../config/standard_mask_list"
    if { [file exists $myfile] & [file readable $myfile] } {
        set fileId [open $myfile r]
        # as long as there are lines in the file
        while { [gets $fileId line_org] >= 0 } {
            if { [string match #* $line_org] } {
                # ignore comment lines
            } else {
                append contents $line_org
                append contents "\n"
            }
        }
    } else {
        puts stdout "could not open $myfile"
    }

    append about_masks $contents 

    # the rest is original      
    if { ![winfo exists $w.image.about] } {
        if { ![info exists fors_logo] || "fors_logo" == "" } {
            set bitmap info
            set logo ""
        } else {
            set bitmap ""
            set logo $fors_logo
        }
        catch { destroy $w.image.about }
        TextDialog $w.image.about \
                   -image $logo \
                   -bitmap $bitmap \
                   -justify center  \
                   -textheight 20 \
                   -text "FORS2-MXU Technical Mask Info" \
                   -title "Mask Info" \
                   -contents $about_masks
    }
    $w.image.about activate

    tracer LEAVING
}

#---------------------
proc grism_about {w} {
#---------------------
# w: skycat instance
# This is a copy of skycat_about with some modifications
    
    tracer ENTERING

    global ::env 
    global my_mask
    global fors_logo

    # look for home directory
    set file $env(FIMSROOT)
    append file "/../bitmap/fors-logo-small.gif"

    # create the FoRS logo
    if { [catch {
              # create a dummy rtdimage to make sure the colormap is initialized before
              # the logo is created to avoid color problems...
              image delete [image create rtdimage]
              set fors_logo [image create photo \
                                          -format GIF \
                                          -file $file]
          } msg] } {
    }

    # set the text string for "About Grism..."
    set about_grism "
     GRISM              FILTER             dis        cen        min        max        ang\
\n---------------------------------------------------------------------------------------------\n"
    set contents ""

    catch { set contents [lindex [$my_mask info variable disp_string] 4] }

    set inst "No Instrument selected"
    catch { set inst [$my_mask cget -instrument] }

    append about_grism $contents 

    # the rest is original      
    if { ![winfo exists $w.image.about] } {
        if { ![info exists fors_logo] || "fors_logo" == "" } {
            set bitmap info
            set logo ""
        } else {
            set bitmap ""
            set logo $fors_logo
        }
        # DialogWidget $w.image.about 
        catch { destroy $w.image.about }
        TextDialog $w.image.about \
                   -image $logo \
                   -bitmap $bitmap \
                   -justify center  \
                   -textheight 20 \
                   -textwidth 100 \
                   -text $inst \
                   -title "Grism Info" \
                   -contents $about_grism
    }
    $w.image.about activate

    tracer LEAVING
}

#==============
# End Callbacks
#==============

#=============
# Menu Wrapers
#=============

#-------------------------------------
proc menu_Cascade {w menu ihdr item} {
#-------------------------------------
# w:    skycat instance
# menu: menu vector
# ihdr: list of column headers in the item definition list
# item: menu item definition list

    tracer ENTERING
    $w add_menuitem $menu cascade "[lindex $item [lsearch -exact $ihdr label]]" \
                                   [lindex $item [lsearch -exact $ihdr help]] \
                         -menu     [menu $menu.[lindex $item [lsearch -exact $ihdr submenu]] -tearoff [lindex $item [lsearch -exact $ihdr toff]]] \
                         -state    [lindex $item [lsearch -exact $ihdr init]]
    tracer LEAVING
}

#----------------------------------------------
proc menu_Command {w menu ihdr item {cmd {}}} {
#----------------------------------------------
# w:    skycat instance
# menu: menu vector
# ihdr: list of column headers in the item definition list
# item: menu item definition list

    tracer ENTERING
    $w add_menuitem $menu command "[lindex $item [lsearch -exact $ihdr label]]" \
                                   [lindex $item [lsearch -exact $ihdr help]]  \
                         -command  $cmd \
                         -state    [lindex $item [lsearch -exact $ihdr init]]
    tracer LEAVING
}

#--------------------------------------------
proc menu_Check {w menu ihdr item {cmd {}}} {
#--------------------------------------------
# w:    skycat instance
# menu: menu vector
# ihdr: list of column headers in the item definition list
# item: menu item definition list

    tracer ENTERING
    $w add_menuitem $menu checkbutton "[lindex $item [lsearch -exact $ihdr label]]" \
                                       [lindex $item [lsearch -exact $ihdr help]]  \
                             -variable [lindex $item [lsearch -exact $ihdr var]] \
                             -onvalue  1 \
                             -offvalue 0 \
                             -command  $cmd \
                             -state    [lindex $item [lsearch -exact $ihdr init]]
    tracer LEAVING
}

#-----------------------------------------------------
proc menu_Radio {w menu ihdr item {val {}} {cmd {}}} {
#-----------------------------------------------------
# w:    skycat instance
# menu: menu vector
# ihdr: list of column headers in the item definition list
# item: menu item definition list

    tracer ENTERING
    $w add_menuitem $menu radio   "[lindex $item [lsearch -exact $ihdr label]]" \
                                   [lindex $item [lsearch -exact $ihdr help]]  \
                         -variable [lindex $item [lsearch -exact $ihdr var]] \
                         -value    $val \
                         -command  $cmd \
                         -state    [lindex $item [lsearch -exact $ihdr init]]
    tracer LEAVING
}

#
#___oOo___
