#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: mxu_class.tcl 234845 2012-11-05 15:52:52Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2012/11/05  Fixed plot_slit method for straight and circle shapes
#                       that allow creating slits inside dispersion stripe of
#                       existing slits due to the use of line instead of poly.
# pbaksai   2010/11/08  PPRS-24729: Fixed slit update methods for add_mmu bug.
# pbaksai   2010/04/10  PPRS-35032: Catch all 'convert coords' errors
# pbaksai   2010/04/08  PPRS-34948: Fixed bug in M2W coordinate conversion
# pbaksai   2010/01/24  PPRS-24729: Fixed bugs in code for curved slits
# pbaksai   2009/07/28  PPRS-27204: HIT-MS implementation
# pbaksai   2009/07/19  PPRS-27204: HIT-MS implementation
# pbaksai   2009/06/30  PPRS-27204: HIT-MS implementation
# pbaksai   2009/06/16  PPRS-27204: HIT-MS implementation
# pbaksai   2009/06/12  PPRS-27204: HIT-MS implementation
# pbaksai   2009/06/02  PPRS-27204: HIT-MS implementation
# pbaksai   2007/12/03  Added tracing
# pbaksai   2007/11/22  PPRS-24729: Fixed so_mm initialization
# pbaksai   2006/03/21  Implemented slit offset in Y
# mkiekebu  2004/06/14  Fixed problem with dialog windows (RH9.0)
# tsz/pbg   2003/11/13  clarified warning in method M2W
# aaguayo   2003/10/28  PPRS 10127 - warning about ref-slit out of range
# aaguayo   2000/12/12  added ref-slit label for CURVED slit type 
#                       (to display warning later) 
# W.Hummel   02.MAR.99  adopted from ref-targ class 
#
#*******************************************************************************
#   NAME             
#   MXU_SLIT - Tcl class           
# 
#   NAMESPACE
#       fors
#
#   SYNOPSIS         
#       MXU_SLIT <instance_name> canvas image ?-eqx <equinox>?
#       
#   PARENT CLASS
#       none 
#
#   DESCRIPTION      
#       Handling of a single MXU slit
#
#   OPTIONS
#
#   PUBLIC METHODS
#       update_straight {}
#       update_circle {}
#       update_curve {}
#       update_curved_center {}
#       check_range {}
#          checks slit center within the world
#       plot_slit {}
#          plots current slit or circle instance to canvas
#       plot_label { oriA oriB }
#          plots labels of current slit instance to canvas 
#       plot_bezier_curve { p1x p1y p2x p2y p3x p3y color tag }
#       plot_bezier_dots { tagname }
#       put_RADEC { radec }
#       put_OC { a }
#          a is slit center as a list of world coords, 
#          corrected for optical distortion 
#          sets slit center in world coords and focplane coords 
#       put_b_oc { nr val }
#       get_xy_mmu {}
#          returns slit center as list in mm mmu working table
#       get_xy {}
#          returns slit center as list in mm foc field 
#       get_OC_RA {}
#          returns slit center, corrected for optical distortion 
#       get_OC_DEC {}
#          returns slit center, corrected for optical distortion 
#       get_RA {}
#          returns slit center, not corrected for optical distortion 
#       get_DEC {}
#          returns slit center, not corrected for optical distortion 
#       get_slit_mmu {}
#          returns slit position and dimension in mm mmu working table
#       get_slit {}
#          returns slit position and dimension in mm foc field 
#       get_oc_slit_mmu {}
#          returns slit position and dimension in mm mmu working table
#          corrected for optical distortion 
#       get_oc_slit {}
#          returns slit position and dimension in mm foc field 
#          corrected for optical distortion 
#       get_oc_slit_contour_mmu {}
#          returns the 4 corner points of rectangular slits 
#          in mm mmu working table 
#       get_oc_slit_contour {}
#          returns the 4 corner points of rectangular slits 
#          in mm foc field 
#       get_slit_contour_mmu {}
#          returns the 4 corner points of rectangular slits 
#          in mm mmu working table NOT corrected for OC
#       get_slit_contour {}
#       get_b_oc_mmu {}
#       get_b_oc {}
#       get_curve_mm {}
#       get_curve_ca {}
#       get_add_mmu {}
#       get_add {}
#       get_mc_off {}
#       set_mc_off { val }
#       mod_bezier { tagname shift_x shift_y }
#       fit_bezier_curve { p1x p1y p2x p2y p3x p3y res }
#       move_curve { curve }
#       M2C {}
#       M2W {}
#       erase {}
#          deletes current slit instance from canvas
#       status {}
#
#   PUBLIC DATA MEMBERS
#       slit_number 999
#       slit_length 10.0
#       slit_width  1.0
#       slit_offset 0.0
#       pos_ang     0.0
#       ref_slit    0
#       ON          1
#       target_name NN
#       debug       0
#       verbosity   0
#       ff
#       eqx         2000.0
#       child
#           to whom I give my gens 
#       shape       STRAIGHT
#           slit shape: STRAIGHT, CURVED, or CIRCLE
#
#   PUBLIC COMMON PROCEDURES
#       get_MASK_NAID {}
#       set_MASK_NAID { pm }
#       get_mmff {}
#       set_mmff {flag}
#       set_col { col }
#       disp { 5val {debug 0} }
#       fc { focscale }
#           sets the focal scale         
#       coeff { mx1 my1 cx1 cy1 mx2 my2 cx2 cy2 mx3 my3 cx3 cy3 {of_whom ""} }
#       label { flag }
#       nos {}
#           returns the number of created slits 
#           the deleted slits are included 
#       nor {}
#           returns the number of reference slits 
#       noas {}
#           returns the number of active slits actually shown in the canvas
#       noars {}
#           returns the number of active reference slits 
#
#   PRIVATE METHODS
#       create_slit {}
#           sets the s-matrix containing slit dimensions in mm focplane 
#       straight_contour {}
#       create_straight {}
#       circle_contour {}
#       create_circle {}
#           sets the s-matrix containing circle dimensions in mm focplane 
#       curve_contour {}
#       create_curve {}
#           sets the s-matrix containing curve dimensions in mm focplane
#       dump_array { a }
#           dumps array a for debugging purposes
#       dump_slit {}
#           dumps slit information for debugging purposes
#
#   PRIVATE DATA MEMBERS
#       alp
#           RA of target in slit, in wcs  
#       del
#           DEC of target in slit, in wcs 
#       oc_alp
#           RA of target optical image 
#       oc_del
#           DEC of target, with optical correction (OC)  
#       x
#           slit central position in mm_ff
#       y
#           slit central position in mm_ff 
#       oc_x
#           central position in mm_ff, OC applied 
#       oc_y
#           central position in mm_ff, OC applied 
#       x1
#           First gridpoint for bezier curved slit 
#       y1
#           First gridpoint for bezier curved slit
#       x2
#           Middle gridpoint for bezier curved slit 
#       y2
#           Middle gridpoint for bezier curved slit
#       x3
#           Last gridpoint for bezier curved slit
#       y3
#           Last gridpoint for bezier curved slit
#       s
#           slit dimension array in mm_ff units 
#       m
#           slit dimension array in canvas units 
#       m_canvas
#           to plot 
#       m_image
#           to convert 
#       slit_initialized 0
#           flag 
#       sw_mm
#           full slit width  in mm focal plane
#       sl_mm
#           full slit length in mm focal plane
#       so_mm            0
#           full slit offset in mm focal plane
#       pa
#           slit position angle in rad   
#       res              30
#           Bezier resolution 
#       add              ""
#           aperture definition 
#       add_mmu          ""
#           aperture definition 
#       mc_off
#           offset for convert distance 
#       bf1
#       bf2
#       bf3
#       bf4
#       bf5
#       bf6
#
#   PRIVATE COMMON DATA
#       debug_flag   0
#       verbose_flag 0
#       MASK_NAID    {M 999999}
#       slit_counter 0
#           number of slits 
#       ref_slit_cnt 0
#           number of ref_slits 
#       fosc
#           focal scale
#       disp
#           dispersion in mm/A
#       lcg
#           lambdacenter of grism                
#       lcf
#           lambdacenter of interference filter
#       fwhm
#           fwhm of interference filter
#       ang
#           angle of spectra dispersion to X axis
#       c1           red
#       c2           yellow
#       c3           red
#       c4           yellow
#       c5           orange
#       c6           white
#       trapa        gray25
#       mmff         0
#           input x,y = mmff or canvas 
#       label        0
#           graphical label            
#
#   COMMANDS
#       MXU_SLIT auto {m_canvas m_image equinox} 
#
#   RETURN VALUES
#
#   CAUTIONS
#       Use FIMS V1.3.1 only with skycat V2.2  
#
#   EXAMPLES
#
#   SEE ALSO
#
#-------------------------------------------------------------------------------
#        1         2         3         4         5         6         7         8
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#

itk::usual FORS {}

namespace eval fors {

class MXU_SLIT {
# Inherits from the Coordinate Conversion classes
inherit CONV_COORD

#-----------------------------------
constructor { m_c m_i xc yc args } {
#-----------------------------------
    tracer ENTERING " of $this"

    eval configure $args

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set m_canvas $m_c
    set m_image  $m_i

    incr slit_counter 1
    if { [$this cget -ref_slit] } {
        incr ref_slit_cnt 1
    }

    eval configure { -slit_number $slit_counter }

    # Export instance pointer with generic name 'slit<i>'
    global slit$slit_counter
    set slit$slit_counter $this

    if { $debug && $verbose } {
        set slit slit$slit_counter
        puts stdout "MXU constructor: slit$slit_counter = [set $slit]"
    }

    # Call parent object (the mask) to update the coefficients
    # of the coordinate conversion matrix of this slit
    [$this cget -child] update_xform_coeff $this

    if { $mmff } {
        # Input coords are read from FAD in mmff units
        # is a fixed mask, no oc corrections 
        set x $xc
        set y $yc 
        set dummy [m2c $x $y]
        set xc [lindex $dummy 0]
        set yc [lindex $dummy 1]
        set oc_x 0.0
        set oc_y 0.0
        set oc_alp 0.0
        set oc_del 0.0
        if { $debug && $verbose } {
            puts stdout "                 (x,y) in mm focalfield to (xc,yc) in canvas coords"
            puts stdout [format "                 Slit $slit_counter,  (%9.3f, %9.3f) --> (%9.3f, %9.3f)" $x $y $xc $yc]
        }
    } else {
        # Input coordinates are read from the user 'click' in canvas units,
        # convert them to mmff
        set dummy [c2m $xc $yc]  
        set x [lindex $dummy 0]
        set y [lindex $dummy 1]
        if { $debug && $verbose } {
            puts stdout "                 (xc,yc) in canvas coords to (x,y) in mm focalfield"
            puts stdout [format "                 Slit $slit_counter,  (%9.3f, %9.3f) --> (%9.3f, %9.3f)" $xc $yc $x $y]
        }
    }

    # Set the private variables alp del in wcs 
    # For all three slit types: x,y xs,ys alp,del are
    # the coords of the cursor click or center/gauss 
    # For curved slits this point is initially not on the slit 
    # and will not stay there if the curved slit is modified

    if { [catch { $m_image convert coords $xc $yc canvas alp del "wcs $eqx" } result] } {
        set alp 99:99:99.99
        set del 99:99:99.99
        puts stdout [format "                 Error converting (%9.3f, %9.3f) \[canvas\] to WCS coord." $xc $yc]
        puts stdout "                 $result"
    }

    if { $debug && $verbose } {
        set slit slit$slit_counter
        puts stdout "                 ALPHA = $alp"
        puts stdout "                 DELTA = $del"
    }

    create_slit

    set slit_initialized 1; # flag to avoid multiple init

    if { $debug && $verbose } {
        dump_slit
    }
    tracer LEAVING "end of $this"
}

#-----------
destructor {
#-----------
    tracer ENTERING " of $this"

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    set i [$this cget -slit_number]
    if { [info exists slit$i] } {
        unset slit$i
    }
    incr slit_counter -1 
    if { [$this cget -ref_slit] } {
        incr ref_slit_cnt -1
    }
    tracer LEAVING "end of $this"
}


# 1 ------------ Create 

#==============================
private method create_slit {} {
#==============================
# Creates a slit

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    switch -exact -- [$this cget -shape] {
        STRAIGHT {
            create_straight
        }
        CURVED {
            #-------------------
            # Default slit shape 
            #-------------------
            # In mm_ff plane units 
            set x1 [expr $x - $sl_mm]
            set y1 [expr $y - $sl_mm/2]
            set x2 $x
            set y2 $y
            set x3 [expr $x - $sl_mm]
            set y3 [expr $y + $sl_mm/2]

            create_curve
            set mc_off [c2m 0.0 0.0]

            set bf1 [list 0 0]
            set bf2 [list 0 0] 
            set bf3 [list 0 0]
            set bf4 [list 0 0] 
            set bf5 [list 0 0]
            set bf6 [list 0 0]
        }
        CIRCLE {
            set sl_mm $sw_mm
            set slit_length $slit_width
            create_circle
        }
        none { 
            puts stdout "shape 'none' not supported" 
            $this configure -ON 0
            tracer LEAVING "NONE"
            return 
        } 
        default { 
            puts stdout "Error: unknown slit shape [$this cget -shape]" 
            $this configure -ON 0
            tracer LEAVING "SHAPE"
            return
        }
    }
    if { [$this cget -ON] } {
        plot_slit
        plot_label e w
    }
    tracer LEAVING
}

#=============================
public method update_slit {} {
#=============================

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        set i [$this cget -slit_number]
        puts stdout " " 
        puts stdout "update_slit: $i" 
    }

    switch -exact -- [$this cget -shape] {
        STRAIGHT {
            $this update_straight
        }
        CIRCLE {
            $this update_circle
        }
        CURVED {
            $this update_curve
        }
    }
    if { [$this cget -ON] } {
        plot_slit
        plot_label e w
    }
    tracer LEAVING
}

#===================================
private method straight_contour {} {
#===================================
# >>>> All coords in mmff 
# Calculates contours of slit (a,b,c,...) again 
# from slit dimensions (width, length)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set xmin [lindex $ff 0];  # focal plane border, XSRMIN or XHRMIN 
    set xmax [lindex $ff 1];  # focal plane border, XSRMAX or XHRMAX
    set ymin [lindex $ff 2];  # focal plane border, YSRMIN or YHRMIN 
    set ymax [lindex $ff 3];  # focal plane border, YSRMAX or YHRMAX

    # focal field coordinates
    # using an order separation filter OG_.. GG_..
    # one slit per dispersion stripe 
    #
    # posang == 0 !!
    #
    #                NORTH
    #  |                             |
    #              b   c              
    # a+ - - - - - +---+ - - - - - - +d
    #              |   |              
    #  |           |   |             |      WEST
    #              |   |              
    # e+ - - - - - +---+ - - - - - - +h
    #              f   g             
    #  |                             |
    #               SOUTH 
    #
    # posang > 0 !!
    #                NORTH
    #  |                             |
    #              b   c              
    # a+ - - - - - +---+ - - - - - - +d
    #               \   \             
    #  |             \   \           |      WEST
    #                 \   \           
    # e+ - - - - - - - +---+ - - - - +h
    #                  f   g             
    #  |                             |
    #               SOUTH 
    #
    # using interference filter (e.g. H_ALPHA+35)
    # more than 1 slit per interference stripe 
    # posang == 0 !!
    #
    #                NORTH
    #  |                               |
    #          a     b   c     d        
    #  +   -   + - - +---+ - - +   -   + 
    #          |     |   |     |        
    #  |       |     |   |     |       |      WEST
    #          |     |   |     |        
    #  +   -   + - - +---+ - - +   -   + 
    #          e     f   g     h       
    #  |                               |
    #               SOUTH 
    #
    # posang > 0 !!
    #                NORTH
    #  |                             |
    #        a     b   c     d        
    #  + - - + - - +---+ - - + - - - +
    #         \     \   \     \       
    #  |       \     \   \     \     |      WEST
    #           \     \   \     \     
    #  + - - - - + - - +---+ - - + - +
    #            e     f   g     h       
    #  |                             |
    #               SOUTH 

    if { $debug && $verbose } {
        set i [$this cget -slit_number]
        puts stdout "straight_contour $i: FWHM: $fwhm"
        puts stdout "                     ANG:  $ang (rad)"
        puts stdout "                     PA:   $pa"
        puts stdout "                     XMIN: $xmin (X\[S|H\]RMIN)"
        puts stdout "                     XMAX: $xmax (X\[S|H\]RMAX)"
    }

    # Find out if the mask is missing grism and/or filter
    set opti 1
    if { [string compare [[$this cget -child] cget -grism] none] == 0 } {
        set opti 0
    }
    if { [string compare [[$this cget -child] cget -filter] none] == 0 } {
        set opti 0
    }
    if { $opti && $fwhm < 600.0 } {
        # Solution for interference filters 
        # - Northern strip line a - d
        if { $debug && $verbose } {
            puts "***** Ax = Ex = ($lcf - $lcg - $fwhm) * $disp *****"
            puts "***** Dx = Hx = ($lcf - $lcg + $fwhm) * $disp *****"
            puts " lcf = $lcf  lcg = $lcg  fwhm = $fwhm  disp = $disp"
        }
        set s(xa) [expr $x-$sl_mm*tan($pa)*0.5+($lcf-$lcg-$fwhm)*$disp]
        set s(ya) [expr $y - $so_mm - $sl_mm*0.5]
        set s(xd) [expr $x-$sl_mm*tan($pa)*0.5+($lcf-$lcg+$fwhm)*$disp]
        set s(yd) $s(ya)
        # - Northern slit rim b - c
        set s(xb) [expr $x+(-$sw_mm-$sl_mm*tan($pa))*0.5]
        set s(yb) $s(ya)
        set s(xc) [expr $x+($sw_mm-$sl_mm*tan($pa))*0.5]
        set s(yc) $s(ya)
        # - Southern strip line e - h
        set s(xe) [expr $x+$sl_mm*tan($pa)*0.5+($lcf-$lcg-$fwhm)*$disp]
        set s(ye) [expr $y - $so_mm + $sl_mm*0.5]
        set s(xh) [expr $x+$sl_mm*tan($pa)*0.5+($lcf-$lcg+$fwhm)*$disp]
        set s(yh) $s(ye)
        # - Southern slit rim f - g
        set s(xf) [expr $x+(-$sw_mm+$sl_mm*tan($pa))*0.5]
        set s(yf) $s(ye)
        set s(xg) [expr $x+($sw_mm+$sl_mm*tan($pa))*0.5]
        set s(yg) $s(ye)
    } else {
        # Solution for order separation or broad band filters 
        # - Northern strip line a - d
        set s(xa) $xmin
        set s(ya) [expr $y - $so_mm - $sl_mm*0.5] 
        set s(xd) $xmax
        set s(yd) $s(ya)
        # - Northern slit rim b - c
        set s(xb) [expr $x+(-$sw_mm-$sl_mm*tan($pa))*0.5]
        set s(yb) $s(ya) 
        set s(xc) [expr $x+($sw_mm-$sl_mm*tan($pa))*0.5]
        set s(yc) $s(ya) 
        # - Southern strip line e - h
        set s(xe) $xmin
        set s(ye) [expr $y - $so_mm + $sl_mm*0.5] 
        set s(xh) $xmax
        set s(yh) $s(ye)
        # - Southern slit rim f - g
        set s(xf) [expr $x+(-$sw_mm+$sl_mm*tan($pa))*0.5]
        set s(yf) $s(ye) 
        set s(xg) [expr $x+($sw_mm+$sl_mm*tan($pa))*0.5]
        set s(yg) $s(ye) 
    }

    set test [format "%6.2f" [expr {$ang*$rad2deg}]]
    if { $test != 0.00 && $test != 180.00 && $test != -180.00 } {
        # Solution for cross dispersion
        # 1. Rotate slit by 'ang' rad
        set cda [expr {cos($ang)}]
        set sda [expr {sin($ang)}]
        foreach val { a b c d e f g h } {
            set d1 [expr {$x + ($s(x$val) - $x)*$cda - ($s(y$val) - $y)*$sda}]
            set d2 [expr {$y + ($s(x$val) - $x)*$sda + ($s(y$val) - $y)*$cda}]
            set s(x$val) $d1
            set s(y$val) $d2
        }
        # 2. Adjust strip
        set s(ya) $ymax 
        set s(yd) $ymin
        set s(ye) $ymax
        set s(yh) $ymin
    }

    if { $debug && $verbose } {
        dump_array s
    }
    tracer LEAVING
}

#==================================
private method create_straight {} {
#==================================
# Creates a straight slit

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # From mm focal plane (s-array) to 
    # canvas coordinates (m-array)
    update_straight
    tracer LEAVING
}

#=================================
public method update_straight {} {
#=================================
# From mm focal plane (s-array) to 
# canvas coordinates (m-array)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # For offset slits or dispersion angle not zero, the
    # slit cannot be passed as an ADD, but as a contour.
    # In order to properly cut the slit in the MMU, it must
    # consider the coordinates of the corners, not only the
    # slit length and width.
    if { $pa == 0 && $so_mm == 0 && $ang == 0 } {
        set i [$this cget -slit_number]

        # Define aperture definition string for postscript 
        set add "%"
        append add "ADD"
        append add [ format "%1d" [expr $i+10] ]
        append add "R,"
        append add [format "%4.3f" $sl_mm]
        append add "X"
        append add [format "%4.3f" $sw_mm]
        append add "*%"

        # Define aperture definition string for MMU   
        set add_mmu "%"
        append add_mmu "ADD"
        append add_mmu [ format "%1d" [expr $i+10] ]
        append add_mmu "R,"
        append add_mmu [format "%4.3f" $sl_mm]
        append add_mmu "X"
        append add_mmu [format "%4.3f" $sw_mm]
        append add_mmu "*%"
    } else {
        set add ""
        set add_mmu ""
    }

    # Call parent object (the mask) to update the coefficients
    # of the coordinate conversion matrix of this slit
    [$this cget -child] update_xform_coeff $this

    # Calculate slit center and target coordinates
    M2W
    
    # Calculate the contour coordinates
    straight_contour

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "update_straight: [$this cget -slit_number]"
        puts stdout [format "                 X = %9.3f \[mmff\]    RA  = %14s \[wcs\]" $x $alp]
        puts stdout [format "                 Y = %9.3f \[mmff\]    DEC = %14s \[wcs\]" $y $del]
    }

    # Convert slit points from focfield mm to canvas
    foreach i { a b c d e f g h } {
        set dummy [m2c $s(x$i) $s(y$i)]
        set m(x$i) [lindex $dummy 0]
        set m(y$i) [lindex $dummy 1]
    }
    if { $debug && $verbose } {
        dump_array m
    }
    tracer LEAVING
}

#=================================
private method circle_contour {} {
#=================================
# >>>> All coords in mmff 
# Calculates contours of slit (a,b,c,...) again 
# from slit dimensions (width, length)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set xmin [lindex $ff 0];  # focal plane border, XSRMIN or XHRMIN 
    set xmax [lindex $ff 1];  # focal plane border, XSRMAX or XHRMAX

    # Focal field coordinates
    # using an order separation filter OG_.. GG_..
    # One slit per dispersion stripe 
    #
    # posang == 0 !!
    #
    #                NORTH
    #  |                             |
    #              b   c              
    # a+ - - - - - +-----+ - - - - - - +d
    #              |/   \|              
    #  |           |     |             |      WEST
    #              |\   /|              
    # e+ - - - - - +-----+ - - - - - - +h
    #              f     g             
    #  |                               |
    #               SOUTH 
    #
    #
    # Using interference filter (e.g. H_ALPHA+35)
    # more than 1 slit per interference stripe 
    # posang == 0 !!
    #
    #                NORTH
    #  |                               |
    #          a     b   c     d        
    #  +   -   + - - +-----+ - - +   -   + 
    #          |     |/   \|     |        
    #  |       |     |     |     |       |      WEST
    #          |     |\   /|     |        
    #  +   -   + - - +-----+ - - +   -   + 
    #          e     f     g     h       
    #  |                                 |
    #               SOUTH 
    #

    if { $debug && $verbose } {
        set i [$this cget -slit_number]
        puts stdout "circle_contour $i: FWHM: $fwhm"
        puts stdout "                   ANG:  $ang (rad)"
        puts stdout "                   PA:   $pa"
        puts stdout "                   XMIN: $xmin (X\[S|H\]RMIN)"
        puts stdout "                   XMAX: $xmax (X\[S|H\]RMAX)"
    }

    # Find out if the mask is missing grism and/or filter
    set opti 1
    if { [string compare [[$this cget -child] cget -grism] none] == 0 } {
        set opti 0
    }
    if { [string compare [[$this cget -child] cget -filter] none] == 0 } {
        set opti 0
    }
    # We use the slit width as circle diameter
    #                                 ========
    if { $opti && $fwhm < 600.0 } {
        # Solution for interference filters and low dispersion grism
        set s(xa) [expr $x+($lcf-$lcg-$fwhm)*$disp]
        set s(ya) [expr $y- $sw_mm*0.5]
        set s(xd) [expr $x+($lcf-$lcg+$fwhm)*$disp]
        set s(yd) $s(ya)
        set s(xb) [expr $x-$sw_mm*0.5]
        set s(yb) $s(ya)
        set s(xc) [expr $x+$sw_mm*0.5]
        set s(yc) $s(ya)

        set s(xe) [expr $x+($lcf-$lcg-$fwhm)*$disp]
        set s(ye) [expr $y+ $sw_mm*0.5]
        set s(xh) [expr $x+($lcf-$lcg+$fwhm)*$disp]
        set s(yh) $s(ye)
        set s(xf) [expr $x-$sw_mm*0.5]
        set s(yf) $s(ye)
        set s(xg) [expr $x+$sw_mm*0.5]
        set s(yg) $s(ye)
    } else {
        # Solution for order separation or broad band filters 
        set s(xa) $xmin
        set s(ya) [expr $y - $sw_mm*0.5] 
        set s(xd) $xmax
        set s(yd) $s(ya)
        set s(xb) [expr $x-$sw_mm*0.5]
        set s(yb) $s(ya) 
        set s(xc) [expr $x+$sw_mm*0.5]
        set s(yc) $s(ya) 

        set s(xe) $xmin
        set s(ye) [expr $y + $sw_mm*0.5] 
        set s(xh) $xmax
        set s(yh) $s(ye)
        set s(xf) [expr $x-$sw_mm*0.5]
        set s(yf) $s(ye) 
        set s(xg) [expr $x+$sw_mm*0.5]
        set s(yg) $s(ye) 
    }
    tracer LEAVING
}

#================================
private method create_circle {} {
#================================
# Creates a circular slit

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # From mm focal plane (s-array) to 
    # canvas coordinates (m-array)
    update_circle
    tracer LEAVING
}

#===============================
public method update_circle {} {
#===============================
# From mm focal plane (s-array) to 
# canvas coordinates (m-array)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set i [$this cget -slit_number]

    # Define aperture definition string for gerber 
    set add "%"
    append add "ADD"
    append add [ format "%1d" [expr $i+10] ]
    append add "C,"
    append add [format "%4.3f" $sw_mm]
    append add "*%"

    # Define aperture definition string for postscript file
    set add_mmu "%"
    append add_mmu "ADD"
    append add_mmu [ format "%1d" [expr $i+10] ]
    append add_mmu "C,"
    append add_mmu [format "%4.3f" $sw_mm]
    append add_mmu "*%"

    # Call parent object (the mask) to update the coefficients
    # of the coordinate conversion matrix of this slit
    [$this cget -child] update_xform_coeff $this

    # Calculate slit center and target coordinates
    M2W
    
    # Calculate the contour coordinates
    circle_contour

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "update_circle: [$this cget -slit_number]"
        puts stdout [format "               X = %9.3f \[mmff\]    RA  = %14s \[wcs\]" $x $alp]
        puts stdout [format "               Y = %9.3f \[mmff\]    DEC = %14s \[wcs\]" $y $del]
    }

    # Convert slit points from focfield mm to canvas
    foreach i { a b c d e f g h } {
        set dummy [m2c $s(x$i) $s(y$i)]
        set m(x$i) [lindex $dummy 0]
        set m(y$i) [lindex $dummy 1]
    }
    if { $debug && $verbose } {
        dump_array m
    }
    tracer LEAVING
}

#================================
private method curve_contour {} {
#================================
# >>>> All coords in mmff 
# Calculates contours of curved slit (a,b,c,...) again 
# This is called either by 'config slit_width' 
# or by config bez-shape  

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    # From slit dimensions (width, length)
    set xmin [lindex $ff 0];  # focal plane border, XSRMIN or XHRMIN 
    set xmax [lindex $ff 1];  # focal plane border, XSRMAX or XHRMAX

    # The three Bezier points are given by 
    # (b+c)/2, (p+q)/2, (f+g)/2 
    #
    # Focal field coordinates
    # using an order separation filter OG_.. GG_..
    # One slit per dispersion stripe 
    #
    #                NORTH
    #
    #                (x1,y1)
    #  |             |               |
    #              b | c              
    # a+ - - - - - +-+-+ - - - - - - +d
    #               \   \              
    #                \   \              
    #               p|   |q  +(x2,y2)            
    #                /   /              
    #               /   /              
    # e+ - - - - - +-+-+ - - - - - - +h
    #              f | g             
    #  |             |               |
    #                (x3,y3)         
    #               SOUTH 
    #
    # Bezier defined by three points (x1,y1), (x,y) and (x3,y3)
    # Central curve defined by bezier points
    # Blue slit rim defined by b p f
    # Red slit rim defined by c q g
    #
    # Using interference filter (e.g. H_ALPHA+35)
    # more than 1 slit per interference stripe 
    # posang == 0 !!
    #
    #                NORTH
    #  |                               |
    #          a     b   c     d        
    #  +   -   + - - +---+ - - +   -   + 
    #          |      \   \    |        
    #  |       |      |   |    |       |      WEST
    #          |      /   /    |        
    #  +   -   + - - +---+ - - +   -   + 
    #          e     f   g     h       
    #  |                               |
    #               SOUTH 
    #

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "curve_contour: [$this cget -slit_number]"
        puts stdout "               FWHM: $fwhm"
        puts stdout "               ANG:  $ang (rad)"
        puts stdout "               PA:   $pa"
        puts stdout "               XMIN: $xmin (X\[S|H\]RMIN)"
        puts stdout "               XMAX: $xmax (X\[S|H\]RMAX)"
    }

    # Find out if the mask is missing grism and/or filter
    set opti 1
    if { [string compare [[$this cget -child] cget -grism] none] == 0 } {
        set opti 0
    }
    if { [string compare [[$this cget -child] cget -filter] none] == 0 } {
        set opti 0
    }
    if { $opti && $fwhm < 600.0 } {
        # Solution for interference filters  and low dispersion grism
        set s(x1) $x1
        set s(y1) $y1
        set s(x2) $x2
        set s(y2) $y2
        set s(x3) $x3
        set s(y3) $y3

        set s(xa) [expr $s(x1)+($lcf-$lcg-$fwhm)*$disp]; 
        set s(ya) $s(y1) 
        set s(xd) [expr $s(x1)+($lcf-$lcg+$fwhm)*$disp]; 
        set s(yd) $s(y1)
        set s(xb) [expr $s(x1)-$sw_mm*0.5]
        set s(yb) $s(y1) 
        set s(xc) [expr $s(x1)+$sw_mm*0.5]
        set s(yc) $s(y1) 

        set s(xe) [expr $s(x3)+($lcf-$lcg-$fwhm)*$disp]; 
        set s(ye) $s(y3) 
        set s(xh) [expr $s(x3)+($lcf-$lcg+$fwhm)*$disp]; 
        set s(yh) $s(y3)
        set s(xf) [expr $s(x3)-$sw_mm*0.5]
        set s(yf) $s(y3)
        set s(xg) [expr $s(x3)+$sw_mm*0.5]
        set s(yg) $s(y3)

        set s(xp) [expr $s(x2)-$sw_mm*0.5]
        set s(yp) $s(y2)
        set s(xq) [expr $s(x2)+$sw_mm*0.5]
        set s(yq) $s(y2)
    } else {
        # Solution for order separation or broad band filters 
        # a,d,e,h at the border
        set s(x1) $x1
        set s(y1) $y1
        set s(x2) $x2
        set s(y2) $y2
        set s(x3) $x3
        set s(y3) $y3

        set s(xa) $xmin
        set s(ya) $s(y1) 
        set s(xd) $xmax
        set s(yd) $s(y1)
        set s(xb) [expr $s(x1)-$sw_mm*0.5]
        set s(yb) $s(y1) 
        set s(xc) [expr $s(x1)+$sw_mm*0.5]
        set s(yc) $s(y1) 

        set s(xe) $xmin
        set s(ye) $s(y3) 
        set s(xh) $xmax
        set s(yh) $s(y3)
        set s(xf) [expr $s(x3)-$sw_mm*0.5]
        set s(yf) $s(y3) 
        set s(xg) [expr $s(x3)+$sw_mm*0.5]
        set s(yg) $s(y3) 
   
        set s(xp) [expr $s(x2)-$sw_mm*0.5]
        set s(yp) $s(y2)
        set s(xq) [expr $s(x2)+$sw_mm*0.5]
        set s(yq) $s(y2)
    }
    tracer LEAVING
}

#===============================
private method create_curve {} {
#===============================
# >>>> All coords in mmff 
# Calculates contours of curved slit (a,b,c,...) again 
# This is called either by 'config slit_width' 
# or by config bez-shape  

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    # From mm focal plane (s-array) to 
    # canvas coordinates (m-array)
    update_curve
    tracer LEAVING
}

#==============================
public method update_curve {} {
#==============================
# From mm focal plane (s-array) to 
# canvas coordinates (m-array)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set add ""
    set add_mmu ""

    # Call parent object (the mask) to update the coefficients
    # of the coordinate conversion matrix of this slit
    [$this cget -child] update_xform_coeff $this

    # Calculate slit center and target coordinates
    M2W
    
    # Calculate the contour coordinates
    curve_contour

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "update_curve: [$this cget -slit_number]"
        puts stdout [format "              X = %9.3f \[mmff\]    RA  = %14s \[wcs\]" $x $alp]
        puts stdout [format "              Y = %9.3f \[mmff\]    DEC = %14s \[wcs\]" $y $del]
    }

    # Convert slit points from focfield mm to canvas
    foreach i { a b c d e f g h 1 2 3 p q } {
        set dummy [m2c $s(x$i) $s(y$i)]
        set m(x$i) [lindex $dummy 0]
        set m(y$i) [lindex $dummy 1]
        if { $debug && $verbose } {
            puts stdout "           $i: m(x$i) = $m(x$i)"
            puts stdout "              m(y$i) = $m(y$i)"
        }
    }

    # Recalculate bezier center
    update_curved_center

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "              After update_curved_center"
        puts stdout [format "              X = %9.3f \[mmff\]    RA  = %14s \[wcs\]" $x $alp]
        puts stdout [format "              Y = %9.3f \[mmff\]    DEC = %14s \[wcs\]" $y $del]
    }

    tracer LEAVING
}

#======================================
public method update_curved_center {} {
#======================================

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    if { $debug && $verbose } {
        puts stdout "update_curved_center: "
    }

    if { [string match *CURVED* [$this cget -shape]] } {
        # Define the central point on the slit 
        # for saving sequence 
        # central slit point
        # ------------------
        set xc [expr 0.25*$m(x1) + 0.5*$m(x2) + 0.25*$m(x3) ] 
        set yc [expr 0.25*$m(y1) + 0.5*$m(y2) + 0.25*$m(y3) ] 
        if { [catch { $m_image convert coords $xc $yc canvas alp del "wcs $eqx" } result] } {
            set alp 99:99:99.99
            set del 99:99:99.99
            if { $debug && $verbose } {
                puts stdout         "                      Curved slit [$this cget -slit_number] center (xc, yc)"
                puts stdout [format "                      Error converting (%9.3f, %9.3f) \[canvas\] to WCS coord." $xc $yc]
                puts stdout "                      $result"
            }
        }

        set x [expr 0.25*$s(x1) + 0.5*$s(x2) + 0.25*$s(x3) ] 
        set y [expr 0.25*$s(y1) + 0.5*$s(y2) + 0.25*$s(y3) ] 

        tracer LEAVING
        return [list $xc $yc] 
    }
    tracer LEAVING NOT_CURVED
}

# 2 ------------ Check 

#=============================
public method check_range {} {
#=============================
# Checks that slit center is within the world
# Returns 0: slit center is out of range
#         1: slit center is OK

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set a 0
    set d 0
    set mx $x
    set my $y
    set c [m2c $x $y]
    set cx [lindex $c 0]
    set cy [lindex $c 1]
    if { [catch {$m_image convert coords $cx $cy canvas a d "wcs $eqx" } result]} {
        if { $debug && $verbose } {
            puts stdout "check_range: MX: $x MY: $y --> CX: $cx CY: $cy --> RA: $a DEC: $d are out of range. $result"
        }
        tracer LEAVING DIALOG
	return 0
    } else {
        if { $debug && $verbose } {
            puts stdout "check_range: MX: $x MY: $y --> CX: $cx CY: $cy --> RA: $a DEC: $d are OK. $result"
        }
    }
    tracer LEAVING
    return 1
}


# 3 ------------ Plot 

#===========================
public method plot_slit {} {
#===========================
# Plot current MXU slit instance to canvas 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set i [$this cget -slit_number]

    # Find out if the parent mask is missing grism and/or filter
    set opti 1
    if { [string compare [[$this cget -child] cget -grism] none] == 0 } {
        set opti 0
    }
    if { [string compare [[$this cget -child] cget -filter] none] == 0 } {
        set opti 0
    }

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "plot_slit: $i  [$this cget -shape]"
        puts stdout "           FWHM = $fwhm"
        puts stdout "           Opti = $opti"
    }

    # Delete from canvas 
    $m_canvas delete mxu$i

    # Redraw 
    switch -exact -- [$this cget -shape] {
        STRAIGHT {
            # The dispersion stripe
            if { $opti && $fwhm < 600.0 } {
                # Solution for interference filters 
                # Use poligons
                $m_canvas create poly $m(xa) $m(ya) $m(xd) $m(yd) \
                                      $m(xh) $m(yh) $m(xe) $m(ye) \
                                      -outline $c6 \
                                      -fill $c1 \
                                      -stipple $trapa \
                                      -tag " A mm mxu$i ma$i "
            } else {
                # Solution for order separation filters 
                # Use poligons
                $m_canvas create poly $m(xa) $m(ya) $m(xb) $m(yb) \
                                      $m(xf) $m(yf) $m(xe) $m(ye) \
                                      -outline $c3 \
                                      -fill $c1 \
                                      -stipple $trapa \
                                      -tag " A mm mxu$i ma$i "
                $m_canvas create poly $m(xd) $m(yd) $m(xc) $m(yc) \
                                      $m(xg) $m(yg) $m(xh) $m(yh) \
                                      -outline $c3 \
                                      -fill $c1 \
                                      -stipple $trapa \
                                      -tag " A mm mxu$i mb$i "
            }
            if { [$this cget -ref_slit] } {
                set dummy $c3
                set c3 $c7
            }
            # The slit rim
            $m_canvas create poly $m(xb) $m(yb) $m(xc) $m(yc) \
                                  $m(xg) $m(yg) $m(xf) $m(yf) \
                                  -outline $c3 \
                                  -fill "" \
                                  -stipple "" \
                                  -tag " A mm mxu$i ms$i "
            if { [$this cget -ref_slit] } {
                set c3 $dummy
            }
        }
        CIRCLE {
            if { $opti && $fwhm < 600.0 } {
                # Solution for interference filters 
                # Use poligons
                $m_canvas create poly $m(xa) $m(ya) $m(xd) $m(yd) \
                                      $m(xh) $m(yh) $m(xe) $m(ye) \
                                      -outline $c6 \
                                      -fill $c1 \
                                      -stipple $trapa \
                                      -tag " A mm mxu$i ma$i "
            } else {
                # Solution for order separation filters 
                # Use poligons
                $m_canvas create poly $m(xa) $m(ya) $m(xb) $m(yb) \
                                      $m(xf) $m(yf) $m(xe) $m(ye) \
                                      -outline $c3 \
                                      -fill $c1 \
                                      -stipple $trapa \
                                      -tag " A mm mxu$i ma$i "
                $m_canvas create poly $m(xd) $m(yd) $m(xc) $m(yc) \
                                      $m(xg) $m(yg) $m(xh) $m(yh) \
                                      -outline $c3 \
                                      -fill $c1 \
                                      -stipple $trapa \
                                      -tag " A mm mxu$i mb$i "
            }
            $m_canvas create poly $m(xb) $m(yb) $m(xc) $m(yc) \
                                  $m(xg) $m(yg) $m(xf) $m(yf) \
                                  -outline $c3 \
                                  -fill "" \
                                  -stipple "" \
                                  -tag " A mm mxu$i ms$i "
            # tcltk too stupid, cannot draw oval with angle
            set lx [expr $m(xb) - $m(xg)]
            set ly [expr $m(yb) - $m(yg)]
            set lr [expr sqrt($lx*$lx + $ly*$ly)/2.0/sqrt(2.0)]
            set lx [expr ($m(xb) + $m(xg))/2.0]
            set ly [expr ($m(yb) + $m(yg))/2.0]
            $m_canvas create oval [expr $lx-$lr] \
                                  [expr $ly-$lr] \
                                  [expr $lx+$lr] \
                                  [expr $ly+$lr] \
                                  -outline $c3 \
                                  -fill "" \
                                  -tag " A mm mxu$i ms$i "
        }
        CURVED {
            if { $label } {
                # If label is ON use high resolution for bezier fits
                set res 10
            } else {
                # Use the low resolution
                set res 5
            }
            if { $opti && $fwhm < 600.0 } {
                # Solution for interference filters 
                if { $transparency } {
                    # Use poligons
                    if { $debug && $verbose } {
                        puts stdout "           Transparency = $transparency, create poly"
                    }
                    $m_canvas create poly $m(xa) $m(ya) $m(xd) $m(yd) \
                                          $m(xh) $m(yh) $m(xe) $m(ye) \
                                          -outline $c6 \
                                          -fill $c1 \
                                          -stipple $trapa \
                                          -tag " A mm mxu$i ma$i "
                } else {
                    # Use lines to have access to graphic objects inside the stripes
                    if { $debug && $verbose } {
                        puts stdout "           Transparency = $transparency, create line"
                    }
                    $m_canvas create line $m(xa) $m(ya) $m(xd) $m(yd) \
                                          $m(xh) $m(yh) $m(xe) $m(ye) \
                                          $m(xa) $m(ya) \
                                          -fill $c6 \
                                          -stipple "" \
                                          -tag " A mm mxu$i ma$i "
                }
            } else {
                # Solution for order separation filters 
                if { $transparency } {
                    # Use poligons
                    if { $debug && $verbose } {
                        puts stdout "           Transparency = $transparency, create poly x 2"
                    }
                    # Use a bezier fit to close the stripe poligons
                    set points [$this fit_bezier_curve $m(xf) $m(yf) $m(xp) $m(yp) $m(xb) $m(yb) $res 1]
                    # Drop the first and last point coordinates to avoid closing the polygon (Tk error)
                    lvarpop points
                    lvarpop points
                    lvarpop points end
                    lvarpop points end
                    # Complete list of point coordinates for the polygon
                    set points "$m(xb) $m(yb) $m(xa) $m(ya) \
                                $m(xe) $m(ye) $m(xf) $m(yf) \
                                $points"
                    if { $debug && $verbose } {
                        puts stdout "           m(xb) = $m(xb)"
                        puts stdout "           m(yb) = $m(yb)"
                        puts stdout "           m(xa) = $m(xa)"
                        puts stdout "           m(ya) = $m(ya)"
                        puts stdout "           m(xe) = $m(xe)"
                        puts stdout "           m(ye) = $m(ye)"
                        puts stdout "           m(xf) = $m(xf)"
                        puts stdout "           m(yf) = $m(yf)"
                        puts stdout "           points = $points"
                    }
                    $m_canvas create poly $points \
                                          -outline $c3 \
                                          -fill $c1 \
                                          -stipple $trapa \
                                          -tag " A mm mxu$i ma$i "
                    set points [$this fit_bezier_curve $m(xg) $m(yg) $m(xq) $m(yq) $m(xc) $m(yc) $res 1]
                    # Drop the first and last point coordinates to avoid closing the polygon (Tk error)
                    lvarpop points
                    lvarpop points
                    lvarpop points end
                    lvarpop points end
                    # Complete list of point coordinates for the polygon
                    set points "$m(xc) $m(yc) $m(xd) $m(yd) \
                                $m(xh) $m(yh) $m(xg) $m(yg) \
                                $points"
                    if { $debug && $verbose } {
                        puts stdout "           m(xc) = $m(xc)"
                        puts stdout "           m(yc) = $m(yc)"
                        puts stdout "           m(xd) = $m(xd)"
                        puts stdout "           m(yd) = $m(yd)"
                        puts stdout "           m(xh) = $m(xh)"
                        puts stdout "           m(yh) = $m(yh)"
                        puts stdout "           m(xg) = $m(xg)"
                        puts stdout "           m(yg) = $m(yg)"
                        puts stdout "           points = $points"
                    }
                    $m_canvas create poly $points \
                                          -outline $c3 \
                                          -fill $c1 \
                                          -stipple $trapa \
                                          -tag " A mm mxu$i mb$i "
                } else {
                    # Use lines to have access to graphic objects inside the stripes
                    if { $debug && $verbose } {
                        puts stdout "           Transparency = $transparency, create line x 2"
                    }
                    $m_canvas create line $m(xb) $m(yb) $m(xa) $m(ya) \
                                          $m(xe) $m(ye) $m(xf) $m(yf) \
                                          -fill $c3 \
                                          -stipple "" \
                                          -tag " A mm mxu$i ma$i "
                    $m_canvas create line $m(xc) $m(yc) $m(xd) $m(yd) \
                                          $m(xh) $m(yh) $m(xg) $m(yg) \
                                          -fill $c3 \
                                          -stipple "" \
                                          -tag " A mm mxu$i mb$i "
                }
            }

            # If label is ON the beziers will be drawn in plot_label with high
            # resolution.
            # Central Bezier
            plot_bezier_curve $m(x1) $m(y1) $m(x2) $m(y2) $m(x3) $m(y3) $res $c7 bc

            # Blue Bezier
            plot_bezier_curve $m(xb) $m(yb) $m(xp) $m(yp) $m(xf) $m(yf) $res $c3 bb

            # Red Bezier
            plot_bezier_curve $m(xc) $m(yc) $m(xq) $m(yq) $m(xg) $m(yg) $res $c3 br
        }
    }
    # puts stdout "plot mxu $i"
    tracer LEAVING
}

#=======================================
public method plot_label { oriA oriB } {
#=======================================
# Plots labels of current slit instance to canvas 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set i [$this cget -slit_number]

    if { $debug && $verbose } {
        puts stdout "plot_label: $i ($oriA$oriB)"
    }

    if { !$label } {
        if { $debug && $verbose } {
            puts stdout "            Label option is not set. Exit."
        }
        tracer LEAVING NO_LABEL
        return
    }

    # Highlight rim of the slit  
    switch -exact -- [$this cget -shape] {
        STRAIGHT {  
            if { [$this cget -ref_slit] } {
                set dummy $c4
                set c4 $c7
            }
            $m_canvas create poly $m(xb) $m(yb) $m(xc) $m(yc) \
                                  $m(xg) $m(yg) $m(xf) $m(yf) \
                                  -outline $c4 \
                                  -fill "" \
                                  -tag "A mm mxu$i ms$i "
            # Slit number label (for reference slit)
            $m_canvas create text [expr ($m(xb)+$m(xf))/2] \
                                  [expr ($m(yb)+$m(yf))/2] \
                                  -anchor $oriA \
                                  -fill $c4 \
                                  -text " #$i " \
                                  -tag "A rs mxu$i" 
            if { [$this cget -ref_slit] } {
                set c4 $dummy
            }
        }
        CIRCLE {
            # tcltk too stupid, cannot draw oval with angle
            set lx [expr $m(xb) - $m(xg)]
            set ly [expr $m(yb) - $m(yg)]
            set lr [expr sqrt($lx*$lx+$ly*$ly)/2.0/sqrt(2.0)]
            set lx [expr ($m(xb) + $m(xg))/2.0]
            set ly [expr ($m(yb) + $m(yg))/2.0]
            # if { [$this cget -ref_slit] } {
            #     set dummy $c4
            #     set c4 $c7
            # }
            $m_canvas create oval [expr $lx-$lr] [expr $ly-$lr] \
                                  [expr $lx+$lr] [expr $ly+$lr] \
                                  -outline $c4 \
                                  -fill "" \
                                  -tag " A mm mxu$i ms$i "
            # Slit number label (for reference slit)
            $m_canvas create text [expr ($m(xb)+$m(xf))/2] \
                                  [expr ($m(yb)+$m(yf))/2] \
                                  -anchor $oriA \
                                  -fill $c4 \
                                  -text " #$i " \
                                  -tag "A rs mxu$i" 
            # if { [$this cget -ref_slit] } {
            #     set c4 $dummy
            # }
        }
        CURVED {
            # The high resolution  
            set res 200
            
            # Delete the low resolution curves 
            $m_canvas delete bc
            $m_canvas delete br
            $m_canvas delete bb
            # Central Bezier
            plot_bezier_curve $m(x1) $m(y1) $m(x2) $m(y2) $m(x3) $m(y3) $res $c7 mm

            # Blue Bezier
            plot_bezier_curve $m(xb) $m(yb) $m(xp) $m(yp) $m(xf) $m(yf) $res $c4 mm

            # Red Bezier
            plot_bezier_curve $m(xc) $m(yc) $m(xq) $m(yq) $m(xg) $m(yg) $res $c4 mm

            # Top and bottom of curved slit 
            $m_canvas create line $m(xb) $m(yb) $m(xc) $m(yc) \
                                 -fill $c4 \
                                 -width 1 \
                                 -tag "A mm mxu$i"
            $m_canvas create line $m(xf) $m(yf) $m(xg) $m(yg) \
                                 -fill $c4 \
                                 -width 1 \
                                 -tag "A mm mxu$i"
            # Slit number label (for reference slit)
            $m_canvas create text [expr ($m(xb) + $m(xf))/2] \
                                  [expr ($m(yb) + $m(yf))/2] \
                                  -anchor $oriA \
                                  -fill $c4 \
                                  -text " #$i " \
                                  -tag "A rs mxu$i"
            plot_bezier_dots B00

        }
        none {
        }
    }

    # Slit width label (to configure slit)
    $m_canvas create text [expr ($m(xc)+$m(xg))/2] \
                          [expr ($m(yc)+$m(yg))/2] \
                          -anchor $oriB \
                          -fill $c4 \
                          -text [format "%5.2f''" [$this cget -slit_width]] \
                          -tag "A sw mxu$i"

    # Do not put useless labels one ontop of the other for reference 'pinholes'
    if { [string compare [string tolower [$this cget -target_name]] refslit] != 0 } {
        # East drive A label (used to de-activate the slit)
        $m_canvas create text [expr ($m(xa)+$m(xe))/2] \
                              [expr ($m(ya)+$m(ye))/2] \
                              -anchor $oriA \
                              -fill $c4 \
                              -text "#$i" \
                              -tag "A kill mxu$i"

        # West drive B label (used to de-activate the slit)
        $m_canvas create text [expr ($m(xd)+$m(xh))/2] \
                              [expr ($m(yd)+$m(yh))/2] \
                              -anchor $oriB \
                              -fill $c4 \
                              -text "#$i" \
                              -tag "A kill mxu$i"
    }

    if { $mmff } {
        # Standard mask mode 
        # You are not allowed to modify a slit
        # You are not allowed to delete a slit
        $m_canvas dtag kill
        $m_canvas dtag sw 
    }
    tracer LEAVING
}

#==========================================================================
public method plot_bezier_curve { p1x p1y p2x p2y p3x p3y res color tag } {
#==========================================================================
# Plot current curved slit instance to canvas 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set i [$this cget -slit_number]

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "plot_bezier_curve: $i"
    }

    set points [$this fit_bezier_curve $p1x $p1y $p2x $p2y $p3x $p3y $res]
    $m_canvas create line $points \
                          -fill $color \
                          -width 1 \
                          -tag " A $tag mxu$i "
    tracer LEAVING
}

#===========================================
public method plot_bezier_dots { tagname } {
#===========================================
# Do not plot tagname,
# plot only the two other bezier points 
# The graphical elements (one of the 
# three cyan bezier points) is moved with 
# the tk internal move command
# but the shape of the curve will be updated
# in mxu_class

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set i [$this cget -slit_number]
    set a [string index $tagname 1] 
    if { $debug && $verbose } {
        puts stdout "plot_bezier_dots: $i"
        puts stdout "                  tagname = $tagname"
        puts stdout "                        a = >$a<"
    }

    if { ![string match B* $tagname] } {
        if { $debug && $verbose } {
            puts stdout "                  Invalid tag $tagname. Not a bezier point."
        }
        tracer LEAVING BAD_TAG
        return
    }

    # Convex hull 
    $m_canvas delete hull$i
    $m_canvas create line $m(x1) $m(y1) $m(x2) $m(y2) $m(x3) $m(y3) \
                          -fill black \
                          -width 1 \
                          -tag "A mm mxu$i hull$i"
    set r [expr sqrt($sl_mm*$sw_mm)*3]  

    # The three Bezier points 
    if { $a != 1 } {
        if { $debug && $verbose } {
            puts stdout "          redrawn oval B13$i"
        }
        $m_canvas delete B13$i
        $m_canvas create oval [expr $m(x1)-$r] \
                              [expr $m(y1)-$r] \
                              [expr $m(x1)+$r] \
                              [expr $m(y1)+$r] \
                              -fill $c7 \
                              -outline black \
                              -width 2 \
                              -tag "A door mxc$i B13$i" 
    }
    if { $a != 2 } {
        if { $debug && $verbose } {
            puts stdout "          redrawn oval B23$i"
        }
        $m_canvas delete B23$i
        $m_canvas create oval [expr $m(x2)-$r] \
                              [expr $m(y2)-$r] \
                              [expr $m(x2)+$r] \
                              [expr $m(y2)+$r] \
                              -fill $c7 \
                              -outline black \
                              -width 2 \
                              -tag "A door mxc$i B23$i"
    }
    if { $a != 3 } {
        if { $debug && $verbose } {
            puts stdout "          redrawn oval B33$i"
        }
        $m_canvas delete B33$i
        $m_canvas create oval [expr $m(x3)-$r] \
                              [expr $m(y3)-$r] \
                              [expr $m(x3)+$r] \
                              [expr $m(y3)+$r] \
                              -fill $c7 \
                              -outline black \
                              -width 2 \
                              -tag "A door mxc$i B33$i"
    }

    # The three labels for three Bezier points 
    # $m_canvas create text $m(x1) $m(y1) \
    #                       -anchor c \
    #                       -fill black 
    #                       -text "1" \
    #                       -tag "A mm mxu$i"
    # $m_canvas create text $m(x2) $m(y2) \
    #                       -anchor c \
    #                       -fill black 
    #                       -text "2" \
    #                       -tag "A mm mxu$i"
    # $m_canvas create text $m(x3) $m(y3) \
    #                       -anchor c \
    #                       -fill black 
    #                       -text "3" \
    #                       -tag "A mm mxu$i"
    if { $debug && $verbose } {
        puts stdout "          END"
    }
    tracer LEAVING
}


# 4 ------------ PUT 

#==================================
public method put_RADEC { radec } {
#==================================
# Sets RA DEC world coordinates of the target in the slit

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    if { "$radec" != "" } {
        set alp [lindex $radec 0]
        set del [lindex $radec 1]
    }
    tracer LEAVING
}

#===========================
public method put_OC { a } {
#===========================
# a = list with wcs coords 
# corrected for optical image distortion
# Usually called after calc_zemax to store 
# the values again 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    set oc_alp [lindex $a 0] 
    set oc_del [lindex $a 1] 
    # I know its a blame
    $m_image convert coords $oc_alp $oc_del "wcs $eqx" lx ly canvas
    set a [c2m $lx $ly] 
    # mmff OC corrected
    set oc_x [lindex $a 0]
    set oc_y [lindex $a 1]
    tracer LEAVING
}

#==================================
public method put_b_oc { nr val } {
#==================================
# val is list of pixel coords coords 
# just to store the two times (three bezier points) 
# corrected for optical distortion

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    $m_image convert coords [lindex $val 0] [lindex $val 1] image \
                            ca1 ca2 canvas 

    # The first three coord pairs for red  bezier curve
    # The last  three coord pairs for blue bezier curve

    switch -exact -- $nr {
        1 { set bf1 [c2m $ca1 $ca2] }
        2 { set bf2 [c2m $ca1 $ca2] }
        3 { set bf3 [c2m $ca1 $ca2] }
        4 { set bf4 [c2m $ca1 $ca2] }
        5 { set bf5 [c2m $ca1 $ca2] }
        6 { set bf6 [c2m $ca1 $ca2] }
    }
    tracer LEAVING
}


# 5 ------------ GET 

#   mmu coords of the MMU working table --> gerber file
#   OC  coords for the focal field      --> p_focf file 
#   normal                              -->   fims file 

# STRAIGHT and CIRCLE 

#============================
public method get_xy_mmu {} {
#============================
# Returns slit center as list in mm of the MMU working table 
# We use conversion X_mmu =  Y_ff, 
#                   Y_mmu = -X_ff

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list [format "%5.3f" [expr {$y - $so_mm}]] \
                 [format "%5.3f" [expr -1.0 * $x]]]
}

#========================
public method get_xy {} {
#========================
# Returns slit center as list in mm focplane units 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list [format "%5.3f" $x] [format "%5.3f" [expr {$y - $so_mm}]]]
}

#===========================
public method get_OC_RA {} {
#===========================
# Corrected for optical image distortion
        
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return $oc_alp
}
      
#============================
public method get_OC_DEC {} {
#============================
# Corrected for optical image distortion

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return $oc_del
}

#========================
public method get_RA {} {
#========================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return $alp
}
      
#=========================
public method get_DEC {} {
#=========================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return $del 
}

#==============================
public method get_slit_mmu {} {
#==============================
# Slit position and full slit width/length in mm MMU working table  

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list [expr {$y - $so_mm}] [expr -1.0 * $x] $sl_mm $sw_mm]
}

#==========================
public method get_slit {} {
#==========================
# Slit position and full slit width in mm focal plane 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list $x [expr {$y - $so_mm}] $sw_mm $sl_mm]
}

#=================================
public method get_oc_slit_mmu {} {
#=================================
# Slit position and full slit width in mm MMU working table  
# Corrected for image distortion 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list [expr {$oc_y - $so_mm}] [expr -1.0 * $oc_x]  $sl_mm $sw_mm]
}

#=============================
public method get_oc_slit {} {
#=============================
# Slit position and full slit width in mm focal plane at position 
# Corrected for image distortion 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list $oc_x [expr {$oc_y - $so_mm}] $sw_mm $sl_mm]
}


#  STRAIGHT POSANG != 0

#=========================================
public method get_oc_slit_contour_mmu {} {
#=========================================
# Returns the 4 corner points of rectangular slits 
# at position corrected for optical image distortion 
# ++++++++++++++++++++++++++++++
# in mm of the MMU working table 
# ++++++++++++++++++++++++++++++

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list [expr  $s(yb) + $oc_y - $y - $so_mm] \
                 [expr -$s(xb) - $oc_x + $x] \
                 [expr  $s(yc) + $oc_y - $y - $so_mm] \
                 [expr -$s(xc) - $oc_x + $x] \
                 [expr  $s(yg) + $oc_y - $y - $so_mm] \
                 [expr -$s(xg) - $oc_x + $x] \
                 [expr  $s(yf) + $oc_y - $y - $so_mm] \
                 [expr -$s(xf) - $oc_x + $x]]
}

#=====================================
public method get_oc_slit_contour {} {
#=====================================
# Returns the 4 corner points of rectangular slits 
# at position corrected for optical image distortion 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list [expr $s(xb) + $oc_x - $x] \
                 [expr $s(yb) + $oc_y - $y - $so_mm] \
                 [expr $s(xc) + $oc_x - $x] \
                 [expr $s(yc) + $oc_y - $y - $so_mm] \
                 [expr $s(xg) + $oc_x - $x] \
                 [expr $s(yg) + $oc_y - $y - $so_mm] \
                 [expr $s(xf) + $oc_x - $x] \
                 [expr $s(yf) + $oc_y - $y - $so_mm]]
}

#======================================
public method get_slit_contour_mmu {} {
#======================================
# Returns the 4 corner points of rectangular slits 
# at position NOT corrected for optical image distortion 
# when creating technical masks 
# ++++++++++++++++++++++++++++++
# in mm of the MMU working table 
# ++++++++++++++++++++++++++++++

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list  [expr $s(yb) - $so_mm] \
                  [expr -1.0 * $s(xb)] \
                  [expr $s(yc) - $so_mm] \
                  [expr -1.0 * $s(xc)] \
                  [expr $s(yg) - $so_mm] \
                  [expr -1.0 * $s(xg)] \
                  [expr $s(yf) - $so_mm] \
                  [expr -1.0 * $s(xf)]]
}

#==================================
public method get_slit_contour {} {
#==================================
# Returns the 4 corner points of rectangular slits 
# at position corrected for optical image distortion 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [list [expr $s(xb) - $x] \
                 [expr $s(yb) - $y - $so_mm] \
                 [expr $s(xc) - $x] \
                 [expr $s(yc) - $y - $so_mm] \
                 [expr $s(xg) - $x] \
                 [expr $s(yg) - $y - $so_mm] \
                 [expr $s(xf) - $x] \
                 [expr $s(yf) - $y - $so_mm]]
}


# CURVED 

#==============================
public method get_b_oc_mmu {} {
#==============================
# Returns the six bezier points in ff focal plane, 
# corrected for optical distortion
# in units of the MMU working table 
# y_mmu = x_ff
# x_mmu = -y_ff

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    lappend new [lindex $bf1 1] [expr -1.0 * [lindex $bf1 0]] 
    lappend new [lindex $bf2 1] [expr -1.0 * [lindex $bf2 0]] 
    lappend new [lindex $bf3 1] [expr -1.0 * [lindex $bf3 0]] 
    lappend new [lindex $bf4 1] [expr -1.0 * [lindex $bf4 0]] 
    lappend new [lindex $bf5 1] [expr -1.0 * [lindex $bf5 0]] 
    lappend new [lindex $bf6 1] [expr -1.0 * [lindex $bf6 0]] 
    tracer LEAVING
    return $new 
}

#==========================
public method get_b_oc {} {
#==========================
# Returns the six bezier points in ff focal plane, 
# corrected for optical distortion
# No longer in use 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return [concat $bf1 $bf2 $bf3 $bf4 $bf5 $bf6]
}

#==============================
public method get_curve_mm {} {
#==============================
# Returns the six bezier points in focal field units

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        puts stdout "get_curve_mm: [$this cget -slit_number]"
        puts stdout "               s(xb) = $s(xb)"
        puts stdout "               s(yb) = $s(yb)"
        puts stdout "               s(xp) = $s(xp)"
        puts stdout "               s(yp) = $s(yp)"
        puts stdout "               s(xf) = $s(xf)"
        puts stdout "               s(yf) = $s(yf)"
        puts stdout "               s(xc) = $s(xc)"
        puts stdout "               s(yc) = $s(yc)"
        puts stdout "               s(xq) = $s(xq)"
        puts stdout "               s(yq) = $s(yq)"
        puts stdout "               s(xg) = $s(xg)"
        puts stdout "               s(yg) = $s(yg)"
    }
    lappend new $s(xb) $s(yb) $s(xp) $s(yp) $s(xf) $s(yf) 
    lappend new $s(xc) $s(yc) $s(xq) $s(yq) $s(xg) $s(yg)
    tracer LEAVING
    return $new
}

#==============================
public method get_curve_ca {} {
#==============================
# Returns the six bezier points in canvas coords

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        puts stdout "get_curve_ca: [$this cget -slit_number]"
        puts stdout "              m(xb) = $m(xb)"
        puts stdout "              m(yb) = $m(yb)"
        puts stdout "              m(xc) = $m(xc)"
        puts stdout "              m(yc) = $m(yc)"
        puts stdout "              m(xf) = $m(xf)"
        puts stdout "              m(yf) = $m(yf)"
        puts stdout "              m(xg) = $m(xg)"
        puts stdout "              m(yg) = $m(yg)"
        puts stdout "              m(xp) = $m(xp)"
        puts stdout "              m(yp) = $m(yp)"
        puts stdout "              m(xq) = $m(xq)"
        puts stdout "              m(yq) = $m(yq)"
    }
    lappend new $m(xb) $m(yb) $m(xp) $m(yp) $m(xf) $m(yf) 
    lappend new $m(xc) $m(yc) $m(xq) $m(yq) $m(xg) $m(yg)
    tracer LEAVING
    return $new
}

#=============================
public method get_add_mmu {} {
#=============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    # puts stdout "$slit_counter [$this cget -ON] $add_mmu"
    tracer LEAVING
    return $add_mmu  
}

#=========================
public method get_add {} {
#=========================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    # puts stdout "$slit_counter [$this cget -ON] $add"
    tracer LEAVING
    return $add  
}

#============================
public method get_mc_off {} {
#============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    tracer LEAVING
    return $mc_off  
}

# 6 ------------ SET

#=================================
public method set_mc_off { val } {
#=================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    set mc_off $val
    tracer LEAVING
}

# 7 ------------ Special Bezier

#=====================================================
public method mod_bezier { tagname shift_x shift_y } {
#=====================================================
# Modifyes shape of CURVED
# Used via interactive cursor drags 
# tagname = tag of bezier point,  must be B13 B23 or B33
# shift_x shift_y = cursor shift in canvas
     
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set i [$this cget -slit_number]
    if { $debug && $verbose } {
        puts stdout "mod_bezier: $i"
        puts stdout "            tagname = $tagname"
        puts stdout "            shift_x = $shift_x"
        puts stdout "            shift_y = $shift_y"
    }

    if { ![string match B* $tagname] } {
        if { $debug && $verbose } {
            puts stdout "            Invalid tag $tagname. Not a bezier point."
        }
        tracer LEAVING BAD_TAG
        return
    }

    # Modify the curved slit online when cursor drag

    # Convert a distance from canvas to mm focf
    set dummy  [ c2m $shift_x $shift_y ]  
    set shaft_x [expr [lindex $dummy 0] - [lindex $mc_off 0]]
    set shaft_y [expr [lindex $dummy 1] - [lindex $mc_off 1]]
    # set shaft_x [expr [lindex $dummy 0] - -99.69]
    # set shaft_y [expr [lindex $dummy 1] - -152.02]
    if { $debug && $verbose } {
        puts stdout "            dummy   = $dummy"
        puts stdout "            mc_off  = $mc_off"
        puts stdout "            shaft_x = $shaft_x"
        puts stdout "            shaft_y = $shaft_y"
    }

    switch -glob -- $tagname { 
        B13* {
            set x1 [expr $s(x1) + $shaft_x]
            set y1 [expr $s(y1) + $shaft_y]
            if { $debug && $verbose } {
                puts stdout "            changed x1 = $x1"
                puts stdout "            changed y1 = $y1"
            }
        }
        B23* {
            set x2 [expr $s(x2) + $shaft_x]
            set y2 [expr $s(y2) + $shaft_y]
            if { $debug && $verbose } {
                puts stdout "            changed x2 = $x2"
                puts stdout "            changed y2 = $y2"
            }
        }
        B33* {
            set x3 [expr $s(x3) + $shaft_x]
            set y3 [expr $s(y3) + $shaft_y]
            if { $debug && $verbose } {
                puts stdout "            changed x3 = $x3"
                puts stdout "            changed y3 = $y3"
            }
        }
    }
    update_curve
    tracer LEAVING
}

#=======================================================================
public method fit_bezier_curve { p1x p1y p2x p2y p3x p3y res {pol 0} } {
#=======================================================================
# Plot current curved slit instance to canvas 
# pol: Flag 1|0. If 1 then the coordinates of a poligon are returned.
#                If 0 then the coordinates of a series of connected line
#                segments are returned.

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    set i [$this cget -slit_number]

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "fit_bezier_curve: $i"
        puts stdout "                  p1x = $p1x"
        puts stdout "                  p1y = $p1y"
        puts stdout "                  p2x = $p2x"
        puts stdout "                  p2y = $p2y"
        puts stdout "                  p3x = $p3x"
        puts stdout "                  p3y = $p3y"
        puts stdout "                  res = $res"
        puts stdout "                  pol = $pol"
    }

    set x0 $p1x
    set y0 $p1y
    for { set j 0 } { $j <= $res } { incr j 1 } {
        set t [expr (1.0*$j) / (1.0*$res)]
        set xt [expr (1-$t)*(1-$t)*$p1x + 2*(1-$t)*$t*$p2x + $t*$t*$p3x] 
        set yt [expr (1-$t)*(1-$t)*$p1y + 2*(1-$t)*$t*$p2y + $t*$t*$p3y] 
        # Coordinate list for a poligon matching the bezier line
        lappend coords $xt
        lappend coords $yt
        # Coordinate list for a series of connected segments for the bezier line
        lappend points $x0
        lappend points $y0
        lappend points $xt
        lappend points $yt
        if { $debug && $verbose } {
            puts stdout "                   j = $j"
            puts stdout "                   t = $t"
            puts stdout "                  xt = $xt"
            puts stdout "                  yt = $yt"
        }
        set x0 $xt
        set y0 $yt
    }
    if { $debug && $verbose } {
        puts stdout "                  coords = $coords"
        puts stdout "                  points = $points"
    }
    if { $pol } {
        tracer LEAVING POL
        return $coords
    }
    tracer LEAVING
    return $points
}

#===================================
public method move_curve { curve } {
#===================================
# Modifyes shape of CURVED via new Bezier point 
# positions. Used after CURVED has been created
# used in load_mxu_TPX 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "move_curve: [$this cget -slit_number]"
    }
    
    if { [string match *CURVED* [$this cget -shape]] } {
        # configure a curved slit by points 
        set x1 [lindex $curve 0]
        set y1 [lindex $curve 1]
        set x2 [lindex $curve 2]
        set y2 [lindex $curve 3]
        set x3 [lindex $curve 4]
        set y3 [lindex $curve 5]
        update_curve
    }
    tracer LEAVING
}


# 8 ------------ Convert Coordinates 

#=====================
public method M2C {} {
#=====================
# Converts the mmff coords (x,y) to canvas (xc,yc)

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    set c [m2c $x $y]
    tracer LEAVING
    return $c
}

#=====================
public method M2W {} {
#=====================
# Converts the mmff coords (x,y) to canvas (xc,yc) to WCS (ra,dec)
# and updates target WCS coordinates

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    if { $debug && $verbose } {
        puts stdout "M2W: "
    }

    set c [M2C]
    set xc [lindex $c 0]
    set yc [lindex $c 1]
    # c2w doesn't work! set w [c2w $xc $yc]
    if { [catch { $m_image convert coords $xc $yc canvas alp del "wcs $eqx" } result] } {
        set alp 99:99:99.99
        set del 99:99:99.99
        if { $debug && $verbose } {
            puts stdout [format "     Error converting (%9.3f, %9.3f) \[canvas\] to WCS coord." $xc $yc]
            puts stdout "     $result"
        }
    }

    set w [list $alp $del]
    tracer LEAVING
    return $w
}


# 9 ------------ Miscellaneous 

#=======================
public method erase {} {
#=======================
# Erases individual slit, circle or curve from canvas 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    set i [$this cget -slit_number]
    # Delete from canvas 
    $m_canvas delete mxu$i
    if { [string match *CURVED* [$this cget -shape]] } {
        $m_canvas delete mxc$i
    }
    $this configure -ON 0
    tracer LEAVING
}

#================================
public method status {} {
#================================
# Maintenance 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    set i [$this cget -slit_number]

    puts stdout [format "%2d/%2d %s" $i $slit_counter [$this cget -shape]]
     
    switch -exact -- [$this cget -shape] {
        STRAIGHT {
            # puts stdout [format "f=%7.3f,%7.3f co=%7.3f,%7.3f" \
            # $x $y $oc_x $oc_y ]
            puts stdout [format "f=%7.3f,%7.3f" $x [expr {$y - $so_mm}]]
            puts stdout [format "W=%7.3f'' =%7.3fmm \
                                 L=%7.3f'' =%7.3fmm \
                                 O=%7.3f'' =%7.3fmm \
                                 P=%7.3fdeg" \
                                [$this cget -slit_width] $sw_mm \
                                [$this cget -slit_length] $sl_mm \
                                [$this cget -slit_offset] $so_mm \
                                [$this cget -pos_ang]]
        }
        CIRCLE {
            # puts stdout [format "f=%7.3f,%7.3f co=%7.3f,%7.3f" \
            # $x $y $oc_x $oc_y ]
            puts stdout [format "f=%7.3f,%7.3f" $x $y]
            puts stdout [format "W=%7.3f''=%7.3fmm"  \
                                [$this cget -slit_width] $sw_mm]
        }
        CURVED {
            puts stdout [format "b:  f=%7.3f,%7.3f co=%7.3f,%7.3f" \
                                 $s(xb) $s(yb) [lindex $bf1 0] [lindex $bf1 1]]
            puts stdout [format "p:  f=%7.3f,%7.3f co=%7.3f,%7.3f" \
                                 $s(xp) $s(yp) [lindex $bf2 0] [lindex $bf2 1]]
            puts stdout [format "f:  f=%7.3f,%7.3f co=%7.3f,%7.3f" \
                                 $s(xf) $s(yf) [lindex $bf3 0] [lindex $bf3 1]]
            puts stdout [format "c:  f=%7.3f,%7.3f co=%7.3f,%7.3f" \
                                 $s(xc) $s(yc) [lindex $bf4 0] [lindex $bf4 1]]
            puts stdout [format "q:  f=%7.3f,%7.3f co=%7.3f,%7.3f" \
                                 $s(xq) $s(yq) [lindex $bf5 0] [lindex $bf5 1]]
            puts stdout [format "g:  f=%7.3f,%7.3f co=%7.3f,%7.3f" \
                                 $s(xg) $s(yg) [lindex $bf6 0] [lindex $bf6 1]]
            puts stdout [format "W=%7.3f'' =%7.3fmm" \
                                [$this cget -slit_width] $sw_mm]
        }
    }
    puts stdout "------------------------------------------------------------------------------------"
    puts stdout " "
    tracer LEAVING
}

#================================
private method dump_array { a } {
#================================
# a: s, dumps mm focal plane coordinates (s-array)
#    m, dumps canvas coordinates (m-array)
# For debugging

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    if { "$a" == "m" } {
        foreach i { a b c d e f g h } {
            puts stdout [format "$i: m(%10.2f, %10.2f)" $m(x$i) $m(y$i)]
        }
        foreach i { p q 1 2 3 } {
            catch { puts stdout [format "$i: m(%10.2f, %10.2f)" $m(x$i) $m(y$i)] }
        }
    } elseif { "$a" == "s" } {
        foreach i { a b c d e f g h } {
            puts stdout [format "$i: s(%10.2f, %10.2f)" $s(x$i) $s(y$i)]
        }
        foreach i { p q 1 2 3 } {
            catch { puts stdout [format "$i: s(%10.2f, %10.2f)" $s(x$i) $s(y$i)] }
        }
    } else {
        # Don't waste my time!
        foreach i { a b c d e f g h } {
            puts stdout [format "$i: s(%10.2f, %10.2f)  m(%10.2f, %10.2f)" $s(x$i) $s(y$i) $m(x$i) $m(y$i)]
        }
        foreach i { p q 1 2 3 } {
            catch { puts stdout [format "$i: s(%10.2f, %10.2f)  m(%10.2f, %10.2f)" $s(x$i) $s(y$i) $m(x$i) $m(y$i)] }
        }
    }
    puts stdout "-------------------------------------------------------"
    puts stdout " "
    tracer LEAVING
}

#============================
private method dump_slit {} {
#============================
# Dumps all the slit information
# For debugging

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]
    switch -exact -- [$this cget -shape] {
        STRAIGHT {
            puts stdout "                                    "
            puts stdout "               NORTH                       "
            puts stdout "  |                             |          "
            puts stdout "  |           b   c             |          "
            puts stdout " a+:::::::::::+---+:::::::::::::+d         "
            puts stdout "  |:::::::::::|   |:::::::::::::|          "
            puts stdout "  |:::::::::::|   |:::::::::::::|      WEST"
            puts stdout "  |:::::::::::|   |:::::::::::::|          "
            puts stdout " e+:::::::::::+---+:::::::::::::+h         "
            puts stdout "  |           f   g             |          "
            puts stdout "  |                             |          "
            puts stdout "               SOUTH                       "
            puts stdout "                                    "
        }
        CIRCLE {
            puts stdout "                                    "
            puts stdout "               NORTH                         "
            puts stdout "  |                               |          "
            puts stdout "  |           b     c             |          "
            puts stdout " a+:::::::::::+--_--+:::::::::::::+d         "
            puts stdout "  |:::::::::::|/   \\|:::::::::::::|          "
            puts stdout "  |:::::::::::|     |:::::::::::::|      WEST"
            puts stdout "  |:::::::::::|\\ _ /|:::::::::::::|          "
            puts stdout " e+:::::::::::+-----+:::::::::::::+h         "
            puts stdout "  |           f     g             |          "
            puts stdout "  |                               |          "
            puts stdout "               SOUTH                         "
            puts stdout "                                    "
        }
        CURVED {
            puts stdout "                                    "
            puts stdout "                NORTH               "
            puts stdout "                                    "
            puts stdout "                (x1,y1)             "
            puts stdout "  |              /                | "
            puts stdout "  |           b / c               | "
            puts stdout " a+ - - - - - +-+-+ - - - - - - - +d"
            puts stdout "  |            \\   \\              | "
            puts stdout "  |             \\   \\             | "
            puts stdout "  |            p |   |q  +(x2,y2) | "
            puts stdout "  |             /   /             | "
            puts stdout "  |            /   /              | "
            puts stdout " e+ - - - - - +-+-+ - - - - - - - +h"
            puts stdout "  |           f \\ g               | "
            puts stdout "  |              \\                | "
            puts stdout "                (x3,y3)             "
            puts stdout "                                    "
            puts stdout "               SOUTH                "
            puts stdout "                                    "
 
        }
    }
    status
    dump_array all
    tracer LEAVING
}

# 10 ----------- Public Class Procedures 

#=============================
public proc get_MASK_NAID {} {
#=============================
# Returns the ESO program ID and the MASK ID

    tracer ENTERING
    tracer LEAVING
    return $MASK_NAID
}

#=================================
public proc set_MASK_NAID { pm } {
#=================================
# Sets the MASK_NA the MASK_ID
        
    tracer ENTERING
    set MASK_NAID $pm
    # puts stdout ">$MASK_NAID<"
    tracer LEAVING
}

#========================
public proc get_mmff {} {
#========================
# Returns change status of mxu 

    tracer ENTERING
    tracer LEAVING
    return $mmff
}

#============================
public proc set_mmff {flag} {
#============================
# Sets change status of mxu 

    tracer ENTERING
    set mmff $flag
    tracer LEAVING
}

#============================
public proc set_col { col } {
#============================
# Sets color for all mxu slits

    tracer ENTERING
    if { $debug_flag && $verbose_flag } {
        puts stdout "set_col: $col"
    }
    set c1 [lindex $col 0]
    set c2 [lindex $col 1]
    set c3 [lindex $col 2]
    set c4 [lindex $col 3]
    set trapa [lindex $col 4]
    tracer LEAVING
}

#====================================
public proc disp { 5val {debug 0} } {
#====================================
# Reads the grism dispersion [mm/nm],
# the grism lam0
# the lam0 of the interference filter and 
# the FWHM of the i-filter; all in nm 
# the angle of the dispersion to X axis, in rad

    tracer ENTERING
    set disp [lindex $5val 0]
    set lcg  [lindex $5val 1]
    set lcf  [lindex $5val 2]
    set fwhm [lindex $5val 3]
    set ang  [lindex $5val 4]
    if { $disp == 0.0 || $disp == "" } { set disp   4.99 ; set flag1 default} else { set flag1 isf }
    if { $lcg  == 0.0 || $lcg  == "" } { set lcg  499.99 ; set flag2 default} else { set flag2 isf }
    if { $lcf  == 0.0 || $lcf  == "" } { set lcf  499.99 ; set flag3 default} else { set flag3 isf }
    if { $fwhm == 0.0 || $fwhm == "" } { set fwhm   5.99 ; set flag4 default} else { set flag4 isf }
    if { $ang  == 0.0 || $ang  == "" } { set ang    0.00 ; set flag5 default} else { set flag5 isf }
    if { $debug_flag || $debug } {
        puts stdout [format "\nMXU disp: Dispersion data:\n \
                              disp = %8.2f ($flag1)\n \
                              lcg  = %8.2f ($flag2)\n \
                              lcf  = %8.2f ($flag3)\n \
                              fwhm = %8.2f ($flag4)\n \
                              ang  = %8.2f ($flag5)\n" \
                            $disp $lcg $lcf $fwhm $ang \
                    ]
    }
    tracer LEAVING
}

#============================
public proc fc { focscale } {
#============================
    tracer ENTERING
    set fosc $focscale
    tracer LEAVING
}

#====================================
public proc coeff { mx1 my1 cx1 cy1 \
                    mx2 my2 cx2 cy2 \
                    mx3 my3 cx3 cy3 \
                    {of_whom ""} } {
#====================================
# Changes coordinate transformation matrices
# of all slit instances created so far,
# or just 'of_whom'
# (avoids repeated calls from every slit, see update_straight method above)

    tracer ENTERING
    if { $of_whom != "" } {
        set who $of_whom
        $of_whom coeff_mc $mx1 $my1 $cx1 $cy1 \
                          $mx2 $my2 $cx2 $cy2 \
                          $mx3 $my3 $cx3 $cy3
        $of_whom set_mc_off [$of_whom c2m 0.0 0.0]
    } else {
        set who all
        for { set i 1 } { $i <= $slit_counter } { incr i } {
            global slit$i
            if { [info exists slit$i] } {
                set slit slit$i
                set of_whom [set $slit]
                $of_whom coeff_mc $mx1 $my1 $cx1 $cy1 \
                                  $mx2 $my2 $cx2 $cy2 \
                                  $mx3 $my3 $cx3 $cy3
                $of_whom set_mc_off [$of_whom c2m 0.0 0.0]
            }
        }
    }
    if { $slit_counter > 0 } {
        set mc_off [$of_whom get_mc_off]
        if { $debug_flag && $verbose_flag } {
            puts stdout [format "coeff for $who: mc_off = %8.2f  %8.2f (mm)" \
                                [lindex $mc_off 0] [lindex $mc_off 1]
                         ]
        }
    } else {
        set who none
        set mc_off [list 0.0 0.0]
        if { $debug_flag && $verbose_flag } {
            puts stdout [format "coeff for $who: mc_off = %8.2f  %8.2f (mm)" \
                                [lindex $mc_off 0] [lindex $mc_off 1]
                         ]
        }
    }
    tracer LEAVING
}

#===========================
public proc label { flag } {
#===========================
# Changes label for all instances : 0 or 1

    tracer ENTERING
    set label $flag
    tracer LEAVING
}

#==================================
public proc transparency { flag } {
#==================================
# Changes transparency for all instances : 0 or 1

    tracer ENTERING
    set transparency $flag
    tracer LEAVING
}

#===================
public proc nos {} {
#===================
# Returns the number of slits 
# The number of all instances, 
# wether they have been deleted in the meanwhile (ON==0)
# or if they are still available (ON==1)

    tracer ENTERING
    tracer LEAVING
    return $slit_counter
}

#===================
public proc nor {} {
#===================
    tracer ENTERING
    tracer LEAVING
    return $ref_slit_cnt
}

#====================
public proc noas {} {
#====================
# Returns the number of all active (non-deleted) mxu slits

    tracer ENTERING
    set ns 0
    for { set i 1 } { $i <= $slit_counter } { incr i 1 } {
        global slit$i
        if { [info exists slit$i] } {
            set slit slit$i
            if { [[set $slit] cget -ON] } {
                incr ns 1
            }
        }
    }
    tracer LEAVING
    return $ns
}

#=====================
public proc noars {} {
#=====================
# Returns the number of all active (non-deleted) reference mxu slits

    tracer ENTERING
    set nrs 0
    for { set i 1 } { $i <= $slit_counter } { incr i 1 } {
        global slit$i
        if { [info exists slit$i] } {
            set slit slit$i
            if { [[set $slit] cget -ON] } {
                if { [[set $slit] cget -ref_slit] } {
                    incr nrs 1
                }
            }
        }
    }
    tracer LEAVING
    return $nrs
}

#===========================================
public proc config_slits { start par val } {
#===========================================
# Changes the configuration of all active slits
# start: starting from that slit number
# par: which parameter: sl = slit length
#                       sw = slit width
#                       so = slit offset
#                       sh = slit shape
#                       pa = slit position angle

    tracer ENTERING
    set sn [incr start 1]
    for { set i $sn } { $i <= $slit_counter } { incr i 1 } {
        global slit$i
        if { [info exists slit$i] } {
            set slit slit$i
            if { [[set $slit] cget -ON] } {
                switch -exact -- $par {
                    sl {
                        [set $slit] configure -slit_length $val
                    }
                    sw {
                        [set $slit] configure -slit_width $val
                    }
                    so {
                        [set $slit] configure -slit_offset $val
                    }
                    sh {
                        [set $slit] configure -shape $val
                    }
                    pa {
                        [set $slit] configure -pos_ang $val
                    }
                }
            }
        }
    }
    tracer LEAVING
}

#-----------------
# Public variables
#-----------------

    #++++++++++++++++++++++
    # Configuration options 
    #++++++++++++++++++++++
  
    #--------------------------------
    public variable slit_number 999 {
    #--------------------------------
        tracer ENTERING " variable slit_number"
        if { $debug } {
            puts stdout " "
            puts stdout "Configured slit $slit_number [$this cget -shape]"
        }
        tracer LEAVING "end of variable slit_number"
    }

    #-------------------------------
    public variable shape STRAIGHT {
    #-------------------------------
        tracer ENTERING " variable shape"
        # slit shape: STRAIGHT, CURVED or CIRCLE
        if { $slit_initialized } {
            # Only for later config, not during slit create
            update_slit
        }
        tracer LEAVING "end of variable shape"
    }

    #---------------------------------
    public variable slit_length 10.0 {
    #---------------------------------
        tracer ENTERING " variable slit_length"

        # From arcsec to mmff
        # Slit length has only a meaning for STRAIGHT
        # For CIRCLE and CURVED, slit_length has no meaning 
        if { [string match *STRAIGHT* [$this cget -shape]] } {
            if { $mmff } {
                # The slit length is given in mm
                set sl_mm $slit_length
                set slit_length [expr $sl_mm * $fosc]
            } else {
                set sl_mm [expr $slit_length / $fosc]
            }
            if { $slit_initialized } {
                update_slit 
                # Updates the mm-focplane (s-array)
                # and converts from mm focplane (s-array) 
                # to canvas (m-array)
            }
        } else {
            # Current slit shape is CIRCLE or CURVED
            # We assume only circle 
            set slit_length [$this cget -slit_width]
            set sl_mm $sw_mm 
        }
        # puts stdout "sl:$slit_length"
        tracer LEAVING "end of variable slit_length"
    }
  
    #-------------------------------
    public variable slit_width 1.0 {
    #-------------------------------
        tracer ENTERING " variable slit_width"

        # From arcsec to mmff
        if { $mmff } {
            # Slit width is given in mm 
            set sw_mm $slit_width
            set slit_width [expr $sw_mm * $fosc]
        } else {
            set sw_mm [expr $slit_width / $fosc]
        }

        if { $slit_initialized } {
            # Only for later config, not during slit create
            update_slit
        }
        tracer LEAVING "end of variable slit_width"
    }

    #--------------------------------
    public variable slit_offset 0.0 {
    #--------------------------------
        tracer ENTERING " variable slit_offset"

        # From arcsec to mmff
        # Slit offset has only a meaning for STRAIGHT
        if { [string match *STRAIGHT* [$this cget -shape]] } {
            if { $mmff } {
                # The slit offset is given in mm
                set so_mm $slit_offset
                set slit_offset [expr $so_mm * $fosc]
            } else {
                set so_mm [expr $slit_offset / $fosc]
            }
            if { $slit_initialized } {
                update_slit 
                # Updates the mm-focplane (s-array)
                # and converts from mm focplane (s-array) 
                # to canvas (m-array)
            }
        } else {
            set so_mm 0
        }
        # puts stdout "so:$slit_offset"
        tracer LEAVING "end of variable slit_offset"
    }

    #----------------------------
    public variable pos_ang 0.0 {
    #----------------------------
        tracer ENTERING " variable pos_ang"

        # From deg to rad 
        set pa [expr $pos_ang * 0.0174532]
        if { [string match *STRAIGHT* [$this cget -shape]] } {
            if { $slit_initialized } {
                update_slit
            }
        }
        tracer LEAVING "end of variable pos_ang"
    }

    #---------------------------
    public variable ref_slit 0 {
    #---------------------------
        tracer ENTERING " variable ref_slit"
        if { $slit_initialized } {
            # Not called by constructor 
            if { [$this cget -ref_slit] } {
                # ref_slit activated 
                incr ref_slit_cnt 1;
            } else {
                # ref_slit de-selected
                incr ref_slit_cnt -1;
            }
        }
        tracer LEAVING "end of variable ref_slit"
    }

    #---------------------
    public variable ON 1 {
    #---------------------
        # puts stdout "ON:$ON"
        # You cannot delete a slit
        # You can only switch it off or on
        # because of renumbering problems 
    }

    #-------------------------------
    public variable target_name NN {
    #-------------------------------
        # puts stdout "tn:$target_name"
    }

    #------------------------
    public variable debug 0 {
    #------------------------
        # For the common public proc
        set debug_flag [$this cget -debug]
    }

    #----------------------------
    public variable verbosity 0 {
    #----------------------------
        # For the common public proc
        set verbose_flag [$this cget -verbosity]
    }

    public variable ff   
    public variable eqx 2000.0
    public variable child;           # to whom I give my gens 

#------------------
# Private variables
#------------------

    private variable alp;                # RA of target in slit, in wcs  
    private variable del;                # DEC of target in slit, in wcs 
                                         # for CURVED : RA,DEC for t=0.5
    private variable oc_alp;             # RA of target optical image 
                                         # distortion correction applied
    private variable oc_del;             # DEC of target, with oc  
    private variable x;                  # slit central position in mm_ff
    private variable y;                  # slit central position in mm_ff 
    private variable oc_x;               # central position in mm_ff, OC applied 
    private variable oc_y;               # central position in mm_ff, OC applied 
    private variable x1;                 # First gridpoint for bezier curved slit 
    private variable y1;                 # First gridpoint for bezier curved slit
    private variable x2;                 # Middle gridpoint for bezier curved slit 
    private variable y2;                 # Middle gridpoint for bezier curved slit
    private variable x3;                 # Last gridpoint for bezier curved slit
    private variable y3;                 # Last gridpoint for bezier curved slit
    private variable s;                  # slit dimension array in mm_ff units 
    private variable m;                  # slit dimension array in canvas units 
    private variable m_canvas;           # to plot 
    private variable m_image;            # to convert 
    private variable slit_initialized 0; # flag 
    private variable sw_mm;              # full slit width  in mm focal plane
    private variable sl_mm;              # full slit length in mm focal plane
    private variable so_mm 0;            # full slit offset in mm focal plane
    private variable pa;                 # slit position angle in rad   
    private variable res 30;             # Bezier resolution 
    private variable add "";             # aperture definition 
    private variable add_mmu "";         # aperture definition 
    private variable mc_off;             # offset for convert distance 
    private variable bf1
    private variable bf2
    private variable bf3
    private variable bf4
    private variable bf5
    private variable bf6

    private common debug_flag 0
    private common verbose_flag 0
    private common MASK_NAID {M 999999}

    private common slit_counter 0;       # number of slits 
    private common ref_slit_cnt 0;       # number of ref_slits 
    private common fosc;                 # focal scale

    # Used when MXU is used with interference filters
    private common disp;                 # dispersion in mm/A
    private common lcg;                  # lambdacenter of grism                
    private common lcf;                  # lambdacenter of interference filter
    private common fwhm;                 # fwhm of interference filter
    private common ang;                  # angle of spectra dispersion to X axis

    # Graphical layout common for all slits 
    private common c1 red; 
    private common c2 yellow; 
    private common c3 red; 
    private common c4 yellow; 
    private common c5 orange; 
    private common c6 white; 
    private common c7 cyan; 
    private common trapa gray25; 

    private common mmff 0;               # if 1 input x,y = mmff else canvas 
    private common label 0;              # if 1 show slit label            
    private common transparency 0;       # if 1 show dispersion stripe shaded

    # degrees to radians conversion constant
    private common deg2rad 0.017453292519943295
    # radians to degrees conversion constant
    private common rad2deg 57.295779513082320876
    }
    # End of the class MXU_SLIT
}
# End of namespace

#
#___oOo___
