#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: ref_targ.tcl 222607 2011-11-21 22:56:40Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2010/11/16  PPRS-23355: Added interactive option
# pbaksai   2010/04/10  PPRS-35032: Catch all 'convert coords' errors
# pbaksai   2010/03/23  PPRS-24729: Typos
# pbaksai   2010/01/24  PPRS-24729: Fixed tracing in verify and removed debug
#                                   variable messages. Added verbosity flags.
# pbaksai   2009/08/13  PPRS-27204: HIT-MS implementation. Fixed bug in verify
# pbaksai   2009/07/28  PPRS-27204: HIT-MS implementation. Allow RT outside
#                       the focalfield when configured by the mask object.
# mkiekebu  2004-06-14  Fixed problem with dialog windows (RH9.0)
# mkiekebu  2002/10/05  Included changes removed previous version
# aaguayo   18/05/02    Removed changes in verify method as for version 2.29
# aaguayo   05/05/01    Modified verify method to be used also when loading 
#                       a setup file
# W.Hummel  23.Feb.99   Purge and purge_single 
# W.Hummel  11.Nov.98   Created 
#
#*******************************************************************************
#   NAME             
#       REF_TARG - Tcl class           
# 
#   NAMESPACE
#       fors
#
#   SYNOPSIS         
#       REF_TARG <instance_name> canvas image ?-eqx <equinox>?
#       
#   PARENT CLASS
#       none 
#
#   DESCRIPTION      
#       Handling of the reference targets for alignment 
#
#   OPTIONS
#       -outside
#           flag to allow RT outside the focal field
#       -debug
#           flag to print detailed messages
#       -verbosity
#           flag to print even more detailed messages
#       -interactive
#           flag to disable interactions with user when in remote control
#
#   PUBLIC METHODS
#       plot  { radius } 
#             plot all reference targets on the canvas
#             2 cyan circle, two black circles, a cyan and a 
#             black number label 
#       get   { i indent }
#       add   {}
#       put   { a d }
#       reset {} 
#             purge the complete list of reference stars
#       purge {}
#             purge only a fraction of reference stars
#       show  {}
#             shows ref-targ numbers and R,DEC in a list_box
#       nort  {}
#             returns the number of reference targets [0..10]
#       test  {}
#       verify {}
#             checks if reference target positions are inside the
#             focal field and are out of the forbidden regions
#
#   PUBLIC DATA MEMBERS
#       ins_path    $env(INS_ROOT)
#       int_path    $env(FIMSROOT)
#       iu_dir      $env(INS_USER)
#       eqx         2000.0
#       outside     0
#       debug       0
#
#   PROTECTED METHODS
#
#   PROTECTED DATA MEMBERS
#
#   PRIVATE METHODS
#       purge_single { j }   
#       put2 { lll }
#
#   PRIVATE DATA MEMBERS
#       num
#       ref_ra,ref_de,rel
#       m_canvas,m_image  
#
#   COMMANDS
#       REF_TARG C_ref_targ  { m_image m_canvas equinox } 
#
#   RETURN VALUES
#
#   CAUTIONS
#       Use FIMS V1.1.5 only with skycat V2.1.3
#
#   EXAMPLES
#
#   SEE ALSO
#
#-------------------------------------------------------------------------------
#        1         2         3         4         5         6         7         8
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#

itk::usual FORS {}

namespace eval fors {

class REF_TARG {

#-----------------------------
constructor { m_c m_i args } {
#-----------------------------
    tracer ENTERING " of $this"
    set m_image  $m_i
    set m_canvas $m_c
    set num 0

    for { set i 1 } { $i <= 10 } { incr i 1 } {
        set ref($i,a) 0.0
        set ref($i,d) 0.0
    }
    eval configure $args
    # Ensure it is a boolean
    if { [$this cget -debug] != 0 } {
        eval configure { -debug 1 }
    }
    if { [$this cget -interactive] != 1 } {
        eval configure { -interactive 0 }
    }
    tracer LEAVING "end of $this"
}

#-----------
destructor {
#-----------
    tracer ENTERING " of $this"
    if { $debug } {
        puts stdout "$this destroyed."
    }
    tracer LEAVING "end of $this"
}

#==============================
public method get { i ident } {
#==============================
# i must be [1..10]
# ident must be 'a' or 'd' 
    tracer ENTERING
    if { $i > $num } {
        tracer LEAVING TOO_MANY
        return
    }
    if { $i < 1 } {
        tracer LEAVING NEGATIVE
        return 
    }
    tracer LEAVING
    return $ref($i,$ident)
}

#=====================
public method add {} {
#=====================
    tracer ENTERING
    if { $num < 10 } {
        info_dialog "Add up to [expr 10-$num] reference targets\n\
                     --isolated point source\n\
                     --bright, not saturated\n\
                     --avoid cyan area\n\
                     --\[Enter\] empty string to stop" .skycat1
        set diff 1
        while { $num < 10 && $diff } {
            set diff 0 
            EnterObject .eo -title "Reference Object #[expr $num+1]" \
                            -image .skycat1.image \
                            -labels {ra dec} \
                            -command [code $this put2]
            tkwait window .eo

        }
    }
    tracer LEAVING
}

#============================
private method put2 { lll } {
#============================
    tracer ENTERING
    $this put [lindex $lll 0] [lindex $lll 1]
    $this plot 20
    set diff 1
    tracer LEAVING
} 

#==========================
public method put { a d } {
#==========================
    tracer ENTERING
    if { $num == 10 } {
        puts stdout "$this put: Error, array full"
    } else {
        set ref([expr {$num+1}],a) $a
        set ref([expr {$num+1}],d) $d
        incr num 1
    }
    tracer LEAVING
}

#=======================
public method purge {} {
#=======================
    tracer ENTERING
    # j might be a list like 5,1,8
    # 2-4 syntax not provided
    set text "Purge Reference Targets \n\n\ "
    if { $num == 0 } {
        append text "\nNothing to Purge" 
    } else {
        for { set i 1 } { $i <= $num } { incr i 1 } {
            append text [format "#%2d   %s   %s\n\ " $i $ref($i,a) $ref($i,d)]
        }
        append text "\nPress \[Ok\] or Enter e.g. 1,$num\n"
    }
    if { $num == 0 } {
        info_dialog $text .skycat1
    } else {
        set wid .skycat1.d         
        catch {destroy $wid}
        set w [InputDialog $wid -title "Warning" \
                                -text $text \
                                -bitmap warning \
                                -modal 1 \
                                -buttons {OK Cancel} \
              ]
        set loc_list [split [$w activate] , ]
        # We first have to purge the highest order number
        set new_list [lsort -integer -decreasing $loc_list]
        for { set i 0 } { $i < [llength $new_list] } { incr i 1 } {
            $this purge_single [lindex $new_list $i]
        }
    }
    tracer LEAVING
} 

#===========================================
private method ldelete_all { w_list value} {
#===========================================
    tracer ENTERING
    # modified ldelete
    # delete list elements by value
    set ix [lsearch -exact $w_list $value]
    set new $w_list
    while { $ix >= 0 } {
        set new [lreplace $w_list $ix $ix]
        set ix [lsearch -exact $new $value]
        set w_list $new
    }
    tracer LEAVING
    return $new
}


#====================================== 
public method verify { {par_load 0} } {
#======================================
# Two tests are performed:
# Test1 : If several reference targets are identical
# Test2 : If the position of the reference targets 
#         are usefull. The cyan forbidden areas must be 
#         drawn on the canvas when calling this method
#         In order to be valid the RT position must  not
#         match the blue border (the CCD rim for HR/SR collimator)
#         and must not match the forbidden cyan area 

    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 1}]

    set text "Verify Positions of \n$num Reference Targets:\n\n"
    set t ""
    if { $num == 0 } {
        set text "No Reference Targets\nVerification Result:\n\n   Failed"
        if { [$this cget -interactive] } {
            info_dialog $text .skycat1
        } else {
            puts stdout $text
        }
        tracer LEAVING ZERO_RT
        return 1
    }

    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "C_ref_targ verify: "
    }
    for { set i 1 } { $i <= $num } { incr i 1 } {
        # Default outside the focal field 
        set inside_mask_flag [$this cget -outside]
        # Default outside forbidden zone
        set not_in_zone_flag 1
        if { [catch { $m_image convert coords $ref($i,a) $ref($i,d) "wcs $eqx" x y canvas } result] } {
            if { $debug && $verbose } {
                puts stdout "C_ref_targ verify: "
                puts stdout [format "        Slit %4d: (ref($i,a), ref($i,d))" $i]
                puts stdout "                  Error converting ($ref($i,a), $ref($i,d)) \[WCS\] to canvas coord."
                puts stdout "                  $result"
            }
        }
        # Test 2 on the forbidden zone 
        set a [$m_canvas find overlapping [expr $x-1] [expr $y-1] \
                                          [expr $x+1] [expr $y+1] \
              ]
        if { $debug && $verbose } {
            puts stdout "        Overlapping to $i: $a"
        }
        foreach c $a {
            set local_tags [ldelete_all [split [$m_canvas gettags $c] " "] ""]
            if { [string compare [lindex $local_tags 0] A] == 0 } {
                # reftarg inside the focal field
                set inside_mask_flag 1
            }
            if { [string compare [lindex $local_tags 2] rtz] == 0 } {
                # reftarg inside the forbidden zone 
                set not_in_zone_flag  0
            }
            if { $debug && $verbose } {
                puts stdout "        $c:"
                puts stdout "        Local tags: >$local_tags<"
                puts stdout "        mask flag:  $inside_mask_flag"
                puts stdout "        zone flag:  $not_in_zone_flag"
            }
        } 
        # Test 2 finished.
        #
        # Test 1 on the duplicate appearance of reference targets 
        set multi ""
        if { $i < $num } {
            for { set j [expr $i + 1] } { $j <= $num } { incr j 1 } {
                $m_image convert coords $ref($i,a) $ref($i,d) "wcs $eqx" lra1 lde1 "deg $eqx"
                $m_image convert coords $ref($j,a) $ref($j,d) "wcs $eqx" lra2 lde2 "deg $eqx"
                set t1 [expr $lra1 -$lra2]
                set t2 [expr $lde1 -$lde2]
                # distance in arcsec 
                set t3 [expr sqrt($t1*$t1 + $t2*$t2)*3600.0 ] 
                if { $t3 < 2.0 } {
                    append multi " = #$j"
                }
                if { $debug && $verbose } {
                    puts stdout "        t1 = $t1"
                    puts stdout "        t2 = $t2"
                    puts stdout "        t3 = $t3"
                }
            }
        }
        # Test 1 finished 
        #
        # Decide what to do with this target
        if { $inside_mask_flag && $not_in_zone_flag } {
            if { $multi != "" } { 
                append text " #$i $multi\n"
                if { $t == "" } {
                    append t $i
                } else {
                    append t ",$i"
                }
            } else {
                append text " #$i Ok\n" 
            }
        } else {
            append text " #$i Not Accepted\n"
            if { $t == "" } {
                append t $i
            } else {
                append t ",$i"
            }
        }
    }

    if { $t == "" } {
        set text "Reference Targets\nVerification Result:\n\n     OK"
        if { [$this cget -interactive] } {
            info_dialog $text .skycat1
        } else {
            puts stdout $text
        }
    } else { 
        append text "\nEnter $t and \nPress \[Ok\] to purge\n"
        # The rest is the same as in the purge method 	
        if { $par_load } {
            # Verify when loading
            set text "There are target references: $t \n In Forbidden Zone!! \n Removing ..."
            if { [$this cget -interactive] } {
                info_dialog $text .skycat1
            } else {
                puts stdout $text
            }
            set loc_list [split $t , ]
            set new_list [lsort -integer -decreasing $loc_list]
            for { set i 0 } { $i < [llength $new_list] } { incr i 1 } {
                $this purge_single [lindex $new_list $i]
            }
            $this plot 1.0
            tracer LEAVING FORBID
            return 0
        } else {   
            # Normal verify - user is aked to purge wrong ref targets	     
            set wid .skycat1.d          
            catch { destroy $wid }
            set w [InputDialog $wid -title "Warning" \
                                    -text $text \
                                    -bitmap warning \
                                    -modal 1 \
                                    -buttons {OK Cancel}]
            set result [$w activate]
            set loc_list [split $result , ]
            # We first have to purge the highest order number
            set new_list [lsort -integer -decreasing $loc_list]
            for { set i 0 } { $i < [llength $new_list] } { incr i 1 } {
                $this purge_single [lindex $new_list $i]
            }
            $this plot 1.0
            if { [string compare $result $t] || $num == 0 } { 
                # if old == new (nothing edited) or no ref targ specified 
                info_dialog "Reference Targets\nVerification Result:\n\n   Failed" .skycat1
                tracer LEAVING FAILED
                return 1
            } else {
                info_dialog "Reference Targets\nVerification Result:\n\n     Ok" .skycat1
                tracer LEAVING SUCCESS
                return 0
            } 
        }
    }
    tracer LEAVING
    return 0
}

#==================================
private method purge_single { j } {
#==================================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 0}]

    if { $j < 1 || $j > $num } {
        if { $debug && $verbose } {
            puts stdout "$this purge_single: $j not in 1..$num"
        }
    } else {
        for { set i $j } { $i < $num } { incr i 1 } {
            set ref($i,a) $ref([expr $i+1],a)
            set ref($i,d) $ref([expr $i+1],d)
        }
        if { $num > 0 } {
            incr num -1
        }
    }
    tracer LEAVING
}

#==============================
public method plot { radius } {
#==============================
    tracer ENTERING

    # Enables verbose debugging locally if -verbosity is set. Put 0 to disable
    set verbose [expr {[$this cget -verbosity] && 1}]
    catch { $m_canvas delete REFTARG } 
    set radius2 [expr {$radius*0.95}]
    set sadius  [expr {$radius/5.0}]
    set sadius2 [expr {$radius2/5.0}]
    if { $debug && $verbose } {
        puts stdout " "
        puts stdout "C_ref_targ plot: "
    }
    for { set i 1 } { $i <= $num } { incr i 1 } {
         if { [catch { $m_image convert coords $ref($i,a) $ref($i,d) "wcs $eqx" x y canvas } result] } {
             if { $debug && $verbose } {
                 puts stdout [format "      Slit %4d: (ref($i,a), ref($i,d))" $i]
                 puts stdout "                 Error converting ($ref($i,a), $ref($i,d)) \[WCS\] to canvas coord."
                 puts stdout "                 $result"
             }
         } else {
             $m_canvas create oval [expr $x-$radius] [expr $y-$radius] \
                                   [expr $x+$radius] [expr $y+$radius] \
                                   -outline cyan \
                                   -tag " REFTARG ref$i "
             $m_canvas create oval [expr $x-$radius2] [expr $y-$radius2] \
                                   [expr $x+$radius2] [expr $y+$radius2] \
                                   -outline black \
                                   -tag " REFTARG ref$i "
             $m_canvas create oval [expr $x-$sadius] [expr $y-$sadius] \
                                   [expr $x+$sadius] [expr $y+$sadius] \
                                   -outline cyan \
                                   -tag " REFTARG ref$i "
             $m_canvas create oval [expr $x-$sadius2] [expr $y-$sadius2] \
                                   [expr $x+$sadius2] [expr $y+$sadius2] \
                                   -outline black \
                                   -tag " REFTARG ref$i "

             $m_canvas create text $x $y \
                                   -fill cyan \
                                   -tag " REFTARG ref$i " \
                                   -anchor w \
                                   -text [ format "  %2d" $i ]
             $m_canvas create text $x $y \
                                   -fill black \
                                   -tag " REFTARG ref$i " \
                                   -anchor e \
                                   -text [ format "%2d  " $i ]
        }
    }
    tracer LEAVING
}

#=======================
public method reset {} {
#=======================
    tracer ENTERING
    for { set i 1 } { $i <= 10 } { incr i 1 } {
        set ref($i,a) 0.0
        set ref($i,d) 0.0
    }
    set num 0
    # Delete them all from the canvas
    $this plot 1.0
    tracer LEAVING
}

#======================
public method show {} {
#======================
    tracer ENTERING
    set text "Reference Targets\n\n\ "
    if { $num > 0 } {
        for { set i 1 } { $i <= $num } { incr i 1 } {
            append text [format "#%2d   %s   %s\n\ " $i $ref($i,a) $ref($i,d)]
        }
    } else {
        append text "no targets selected\n\ "
    }
    info_dialog $text .skycat1
    tracer LEAVING
}

#======================
public method nort {} {
#======================
    tracer ENTERING
    tracer LEAVING
    return $num
}

#======================
public method test {} {
#======================
    tracer ENTERING
    puts stdout [$this nort]
    $this show  
    for { set i 1 } { $i <= 11 } { incr i 1 } {
        $this put 00:00:30.3 -72:34:34.5
        $this show
        puts stdout [$this nort]
        puts stdout "[$this get $i a] [$this get $i d]"
    }
    tracer LEAVING
}

#======================
proc init_ref_targ {} {
#======================
    tracer ENTERING
    REF_TARG ref_targ
    ref_targ test
    ref_targ show
    ref_targ destructor
    tracer LEAVING
}

#-----------------
# public variables
#-----------------
    public variable eqx 2000.0
    public variable mos10ali 0  
    public variable outside 0
    #------------------------
    public variable debug 0 {
    #------------------------
    }
    #----------------------------
    public variable verbosity 0 {
    #----------------------------
    }
    #------------------------------
    public variable interactive 1 {
    #------------------------------
    }

#------------------
# private variables
#------------------
    private variable num 0;  # Number Of Reference Targets
    private variable ref_ra 0.0
    private variable ref_de 0.0
    private variable ref 
    private variable diff
    private variable m_canvas
    private variable m_image

    }
    # End of the class REF_TARG
}
# End of namespace

# fors::REF_TARG::init_ref_targ
#
#___oOo___
