#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: status_panel.tcl 236128 2012-11-30 21:47:34Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2012/11/28  PPRS-48245: Quit Mode bug
#                       - Added show and clear methods
#                       - Updated constructor and destructor so panel is not
#                         destroyed between mode changes, but reused.
# pbaksai   2011/11/22  PPRS-24729: Fixed problem opening panel a second time.
# pbaksai   2009/07/19  PPRS-27204: HIT-MS implementation
# pbaksai   2008/06/23  PPRS-27337: Added camera status
# W.Hummel  02.MAR.99   Adopted from ref-targ class 
#
#*******************************************************************************
#   NAME             
#       StatusPanel - Tcl class         
# 
#   NAMESPACE
#       fors
#
#   SYNOPSIS         
#       StatusPanel <instance_name> -fors_class $this                  
#       
#   PARENT CLASS
#       none 
#
#   DESCRIPTION      
#       Pop up window showing the selected optical components,
#       the session status, the status of several useful flags 
#       and the status of the MXU
#
#   OPTIONS
#
#   PUBLIC METHODS
#       show {}           
#       update {}
#
#   PUBLIC DATA MEMBERS
#       fors_class          
#
#   COMMON METHODS
#
#   PRIVATE METHODS
#       clear {}           
#       create_pan {}           
#       add_label_contents_a  {}
#       layout_dialog {}
#       add_dialog_buttons {}
#       ldelete { list value}
#
#   PRIVATE DATA MEMBERS
#       w_
#
#   COMMANDS
#       MXU_SLIT auto {m_image m_canvas equinox} 
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#       catch {SP update}  
#
#   SEE ALSO
#
#-------------------------------------------------------------------------------
#        1         2         3         4         5         6         7         8
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#

itk::usual FORS {}

# create a class for the application

namespace eval fors {

class StatusPanel {

#-------------------
constructor {args} {
#-------------------
    tracer ENTERING " of $this"
    eval configure $args
    set w_ .statuspanel
    tracer LEAVING "end of $this"
}

#-----------
destructor {
#-----------
    tracer ENTERING " of $this"
    tracer LEAVING "end of $this"
}

#----------------------
public method show {} {
#----------------------
    tracer ENTERING
    if { ![winfo exists $w_] } {
        create_pan 
    }
    update
    catch {wm deiconify $w_}
    tracer LEAVING
}

#------------------------
public method update {} {
#------------------------
    tracer ENTERING
    if { ![winfo exists $w_] } {
        tracer LEAVING NO_PANEL
        return
    }
    if { ![string length $fors_class] } {
        clear
        tracer LEAVING NO_FORS
        return
    }
    $w_.fors config -value [$fors_class cget -instrument] 
    $w_.mode config -value [string toupper [$fors_class cget -modus]]
    $w_.mask config -value [string toupper [$fors_class info variable mask -value]]

    $w_.cam  config -value [$fors_class cget -camera]
    $w_.coll config -value [$fors_class cget -collimat]
    $w_.gris config -value [$fors_class cget -grism]
    $w_.filt config -value [$fors_class cget -filter]

    # if a echelle grism cross disperser grism is in use 

    if { [string match XGRIS* [$w_.gris cget -value]] } {
        set a [split [$w_.gris cget -value] "."]
        $w_.gris config -value "[lindex $a 0].[lindex $a 1]"
        $w_.filt config -value [lindex $a 2]
    }

    if { [string match *open* [$fors_class cget -slit_width]] } {
         $w_.sw config -value [$fors_class cget -slit_width] 
    } else {
         $w_.sw config -value [format "%5.2f" [$fors_class cget -slit_width]]
    }

    $w_.sl config -value [$fors_class cget -slit_length]
    $w_.ll config -value [$fors_class cget -long_width]
    $w_.echs9 config -value [$fors_class info variable echs_flag -value]
 
    if { [$w_.echs9 cget -value] } {
         $w_.echs9 config -value ON
    } else {
         $w_.echs9 config -value OFF
    }

    if { [$fors_class cget -pics] } {
        $w_.pics config -value ON 
    } else {
        $w_.pics config -value OFF
    }
   
    if { [$fors_class info variable sp_mtali -value] } {
        $w_.mtal config -value ON 
    } else {
        $w_.mtal config -value OFF
    }

    $w_.nort config -value [$fors_class info variable sp_nort -value]
    # $w_.open config -value "_oOo_"
    $w_.fina config -value [$fors_class cget -file_name]
    $w_.fofr config -value [$fors_class cget -fors_frame]

    $w_.fits_i config -value [$fors_class info variable sp_instrume -value]
    $w_.fits_o config -value [$fors_class info variable sp_origin -value]

    $w_.llfn config -value [$fors_class info variable last_load_fn -value]
    $w_.lsfn config -value [$fors_class info variable last_save_fn -value]
    $w_.lact config -value [$fors_class info variable last_action -value]

    # -------
    # M  X  U 
    # -------

    if { [string match *mxu*   [$fors_class cget -modus]] ||
         [string match *hitms* [$fors_class cget -modus]] } {
        # number of MXU slits 
        set noms [$fors_class noas]
        set PM [ldelete [split [fors::MXU_SLIT::get_MASK_NAID] " "] ""]

        $w_.m_pid config -value [lindex $PM 0]; # maks label 
        $w_.m_mid config -value [lindex $PM 1]; # mask id 
        $w_.m_tec config -value [fors::MXU_SLIT::get_mmff]; # technical mask  
        $w_.m_rsl config -value [$fors_class noars] 
        # $w_.m_rsl config -value [fors::MXU_SLIT::nor]; # RefSlitNumber
        $w_.m_typ config -value [$fors_class cget -slit_shape]
        $w_.m_nos config -value $noms 
    } else {
        $w_.m_pid config -value ""
        $w_.m_mid config -value ""
        $w_.m_tec config -value 0
        $w_.m_rsl config -value 0
        $w_.m_typ config -value ""
        $w_.m_nos config -value 0
    }
    tracer LEAVING
}

#------------------------
private method clear {} {
#------------------------
    tracer ENTERING
    if { ![winfo exists $w_] } {
        tracer LEAVING NO_PANEL
        return
    }
    $w_.fors config -value none
    $w_.mode config -value [string toupper none]
    $w_.mask config -value [string toupper none]

    $w_.cam  config -value none
    $w_.coll config -value none
    $w_.gris config -value none
    $w_.filt config -value none

    $w_.sw config -value 0

    $w_.sl config -value 0
    $w_.ll config -value 0
    $w_.echs9 config -value OFF
    $w_.pics config -value OFF
    $w_.mtal config -value OFF

    $w_.nort config -value ""
    $w_.fina config -value 0
    $w_.fofr config -value ""

    $w_.fits_i config -value ""
    $w_.fits_o config -value ""

    $w_.llfn config -value ""
    $w_.lsfn config -value ""
    $w_.lact config -value ""

    # -------
    # M  X  U 
    # -------

    $w_.m_pid config -value ""
    $w_.m_mid config -value ""
    $w_.m_tec config -value 0
    $w_.m_rsl config -value 0
    $w_.m_typ config -value ""
    $w_.m_nos config -value 0
    tracer LEAVING
}


#-----------------------------
private method create_pan {} {
#-----------------------------
    tracer ENTERING
    TopLevelWidget $w_
    $w_ add_menubar
    set m [$w_ add_menubutton File]

    $w_ add_menuitem $m command "Exit" \
                                {Exit StatusPanel}  \
                               -state normal  \
                               -command "wm withdraw $w_"
    wm title $w_ "Status Panel"
    wm iconname $w_ "Fims Status"
    layout_dialog 
    wm withdraw $w_
    tracer LEAVING
}


#--------------------------------
private method layout_dialog {} {
#--------------------------------
    tracer ENTERING
    add_dialog_buttons 
    add_label_contents_a 
    tracer LEAVING

}

#-------------------------------------
private method add_dialog_buttons {} {
#-------------------------------------
    tracer ENTERING
    # dialog buttons
    pack [frame $w_.buttons -borderwidth 2 -relief raised] \
         -side bottom \
         -fill x
    pack [button $w_.update -text "UpDate" -command [code $this update]] \
         -side left \
         -expand 1 \
         -pady 2m \
         -in $w_.buttons

    pack [button $w_.ok -text "Icon" -command "wm iconify $w_"] \
         -side left \
         -expand 1 \
         -pady 2m \
         -in $w_.buttons

    pack [button $w_.bell -text "Bell" -command bell] \
         -side left \
         -expand 1 \
         -pady 2m \
         -in $w_.buttons
    tracer LEAVING
}

#---------------------------------------
private method add_label_contents_a {} {
#---------------------------------------
#
# single pack
# 
# Label [Value]
#  
#  lw     vw 

# double pack
#
# Label [Value]  Label [Value]
#
#  lws   vws      vws   vws
    tracer ENTERING

    set lw  8
    set vw 21
    set lws 8
    set vws 5

    pack [frame $w_.inst -borderwidth 2 -relief raised] \
         -side top \
         -fill x

    pack [frame $w_.border1 -borderwidth 2 -relief sunken] \
         -side top \
         -fill x

    pack [frame $w_.keys -borderwidth 2 -relief raised] \
         -side top \
         -fill x

    pack [frame $w_.border2 -borderwidth 2 -relief sunken] \
         -side top \
         -fill x

    pack [frame $w_.mxu_keys -borderwidth 2 -relief raised] \
         -side top \
         -fill x


# part 1
# ------status_panel.tcl
    pack [LabelWidget $w_.head1 -text "Instrument" -labelwidth 10] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.inst

    pack [LabelValue $w_.fors -text "Instrument" \
                              -value "none" \
                              -labelwidth $lw \
                              -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.inst

    pack [frame $w_.double1] \
         -side top \
         -expand 1 \
         -in $w_.inst

    pack [LabelValue $w_.mode -text "Mode" \
                              -value [string toupper none] \
                              -labelwidth $lws \
                              -valuewidth $vws \
         ] \
         [LabelValue $w_.mask -text "FOCF" \
                              -value [string toupper none] \
                              -labelwidth $vws \
                              -valuewidth $vws \
         ] \
         -side left \
         -expand 1  \
         -in $w_.double1

    pack [LabelValue $w_.cam -text "Camera" \
                             -value none \
                             -labelwidth $lw \
                             -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.inst

    pack [LabelValue $w_.coll -text "Collimator" \
                              -value none \
                              -labelwidth $lw \
                              -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.inst

    pack [LabelValue $w_.gris -text "Grism" \
                              -value none \
                              -labelwidth $lw \
                              -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.inst

    pack [LabelValue $w_.filt -text "Filter" \
                              -value none \
                              -labelwidth $lw \
                              -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.inst

# part2 
# -----
    pack [LabelWidget $w_.head2 -text "Session" -labelwidth 10] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.keys

    # --------------- double pack 
    pack [frame $w_.double2a] \
         -side top \
         -expand 1 \
         -in $w_.keys
       
    pack [LabelValue $w_.sw -text "Width/''" \
                            -value 0 \
                            -labelwidth $lws \
                            -valuewidth $vws \
         ] \
         [LabelValue $w_.sl -text "Length" \
                            -value 0 \
                            -labelwidth $vws \
                            -valuewidth $vws \
         ] \
         -side left \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.double2a
    # --------------------- double pack 

    pack [frame $w_.double2] \
         -side top \
         -expand 1 \
         -in $w_.keys

    pack [LabelValue $w_.ll -text "LongSS/''" \
                            -value 0 \
                            -labelwidth $lws \
                            -valuewidth $vws \
         ] \
         [LabelValue $w_.echs9 -text "ECH+9" \
                               -value OFF \
                               -labelwidth $vws \
                               -valuewidth $vws \
         ] \
         -side left \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.double2

    # double pack -----------------------------
    pack [frame $w_.double3] \
         -side top \
         -expand 1 \
         -in $w_.keys

    pack [LabelValue $w_.pics -text "Cen/Slit" \
                              -value "" \
                              -labelwidth $lws \
                              -valuewidth $vws \
         ] \
         [LabelValue $w_.mtal -text "M10ALI" \
                              -value OFF \
                              -labelwidth $vws \
                              -valuewidth $vws \
         ] \
         -side left \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.double3
    # ------------ double pack 

    pack [frame $w_.double4a] \
         -side top \
         -expand 1 \
         -in $w_.keys

    pack [LabelValue $w_.nort -text "Ref Targs" \
                              -value "" \
                              -labelwidth $lws \
                              -valuewidth $vws \
         ] \
         [LabelValue $w_.fofr -text "OptCor" \
                              -value "" \
                              -labelwidth $vws \
                              -valuewidth $vws \
         ] \
         -side left \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.double4a

    pack [LabelValue $w_.fits_i -text "FITS ins" \
                                -value "" \
                                -labelwidth $lw \
                                -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.keys

    pack [LabelValue $w_.fits_o -text "FITS orig" \
                                -value "" \
                                -labelwidth $lw \
                                -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.keys

    pack [LabelValue $w_.fina -text "ProjName" \
                              -value 0 \
                              -labelwidth $lw \
                              -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.keys

    pack [LabelValue $w_.lsfn -text "Last Save" \
                              -value "" \
                              -labelwidth $lw \
                              -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.keys

    pack [LabelValue $w_.llfn -text "Last Load" \
                              -value "" \
                              -labelwidth $lw \
                              -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.keys

    pack [LabelValue $w_.lact -text "Last Act" \
                              -value "" \
                              -labelwidth $lw \
                              -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.keys

#  MXU keys 
# ----------
    pack [LabelWidget $w_.head3 -text "FORS MXU" -labelwidth 10] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.mxu_keys

    pack [LabelValue $w_.m_typ -text "Slit Type" \
                               -value none \
                               -labelwidth $lw \
                               -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.mxu_keys

    pack [LabelValue $w_.m_nos -text "Slit Num" \
                               -value none \
                               -labelwidth $lw \
                               -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.mxu_keys

    pack [LabelValue $w_.m_pid -text "Mask NA" \
                               -value none \
                               -labelwidth $lw \
                               -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.mxu_keys

    pack [LabelValue $w_.m_mid -text "MASK ID" \
                               -value none \
                               -labelwidth $lw \
                               -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.mxu_keys

    pack [LabelValue $w_.m_rsl -text "RefSlit" \
                               -value none \
                               -labelwidth $lw \
                               -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.mxu_keys

    pack [LabelValue $w_.m_tec -text "TechMask" \
                               -value none \
                               -labelwidth $lw \
                               -valuewidth $vw \
         ] \
         -side top \
         -expand 1 \
         -pady 1 \
         -padx 1 \
         -in $w_.mxu_keys

    tracer LEAVING
}


#-------------------------------------
private method ldelete { list value} {
#-------------------------------------
# taken from Welch, p34
    tracer ENTERING
    # delete one list element by value
    set ix [lsearch -exact $list $value]
    if { $ix >= 0 } {
        tracer LEAVING 1
        return [lreplace $list $ix $ix]
    } else {
        tracer LEAVING 2
        return $list
    }
}

public variable fors_class 
private variable w_

}; # End of class StatusPanel

}; # End of namespace fors

#
#___oOo___
