#!/bin/bash
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fims.sh.tpl 259761 2014-09-21 18:09:28Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2014/09/20  CCB-000755: Support for 64 bits and portability.
#                       - Corrected version handling.
# pbaksai   2014/07/28  CCB-000579: Support for 64 bits and portability.
#                       - Start skycat in the background and exit.
# pbaksai   2014/07/25  CCB-000579: Support for 64 bits and portability.
#                       - Added more library linkage info to debug.
# pbaksai   2014/07/10  CCB-000579: Support for 64 bits and portability.
#                       - Added library linkage info to debug.
#                       - Added SHLIB_PATH for HP-UX case.
#                       - Fixed for differences in svn info output between
#                         svn 1.6 and 1.7
#                       - Check that ~/.fims is created for user.
# pbaksai   2014/05/14  CCB-000579: Support for 64 bits and portability.
#                       - Explain what to do if SkyCat not found.
# pbaksai   2014/01/28  CCB-000579: Support for 64 bits and portability.
#                       - Fixed display of version, date, time defaults.
# pbaksai   2013/10/17  CCB-000579: Support for 64 bits and portability.
# pbaksai   2013/01/23  PPRS-23355: Corrections after testing.
#                       - Details.
#                       - Export HOST used by SkyCat_plugin.tcl
# pbaksai   2012/12/29  PPRS-23355: Corrections after testing.
#                       - More checks while looking for local SkyCat version.
# pbaksai   2012/12/28  PPRS-23355: Corrections after testing.
#                       - Construct PATH and LD_LIBRARY_PATH here
#                         for the case installer user is not the same as runtime
#                         user (case in wgsoffX machines).
#                       - Try to find out local SkyCat version.
# pbaksai   2012/12/13  PPRS-23355: Eliminated Tcl/Tk and SkyCat code.
# pbaksai   2012/11/28  PPRS-23355: Corrections after testing.
# pbaksai   2012/11/05  PPRS-23355: Corrections after testing.
# pbaksai   2012/11/04  PPRS-23355: Completed.
# pbaksai   2012/09/22  PPRS-23355: Created.
#
#*******************************************************************************
#   NAME
#       fims.sh - FIMS startup in user environment.
#
#   SYNOPSIS
#       fims.sh [[-l|--log]
#                [-b|--batch]
#                [-s|--screen=<ip:d.s>]
#                [-t|--trace] 
#                [-v|--verbose]
#                [<skycat opts>]
#                [--debug]
#                [--help]
#                [--usage]
#                [--License]
#                [--Version]
#                --]
#               <camera>
#
#   DESCRIPTION
#       This script starts FIMS application in the user's environment.
#
#   PARAMETERS
#        -l|--log         : Enables logging mode.
#        -b|--batch       : Starts FIMS in batch mode. Available only to VLTSW ENG.
#        -s|--screen      : Opens in specified screen. Hidden from normal users.
#        -t|--trace       : Enables tracing mode. Hidden from normal users.
#        -v|--verbose     : Enables verbose mode. Hidden from normal users.
#       --debug           : Enables debugging mode. Hidden from normal users.
#       --help            : Displays a help screen explaining the syntax.
#       --usage           : Displays a help screen explaining the interface.
#       --License         : Displays copyright and license information.
#       --Version         : Displays program version.
#
#   ARGUMENTS
#        <camera>         : [B|R] Character code of the FORS CCD.
#                           No Default.
#        <ip:d.s>         : IP address:display_number.screen_number
#                           Default: $DISPLAY
#       ?<skycat opts>?   : [skycat -?] Command line options passed to skycat.
#                           Default: -with_grid 1 -scrollbars 1 -rtd 0
#
#   INPUTS
#       None
#
#   OUTPUTS
#       <stdout>
#           <exec log>    : Log of execution.
#       <file>
#           <logfile>     : <$PWD>/fims.log
#
#   FILES
#       bash_library.sh
#
#   ENVIRONMENT
#       USER
#       HOST
#       HOME
#       DISPLAY
#       SHELL_DEBUG
#
#   RETURN VALUES
#       0: All OK.
#       1: Cancelled due to insufficient conditions.
#       2: Aborted due to errors.
#       3: Rejected due to incorrect arguments.
#       4: Ended with information display only (help, usage, license, version).
#
#   CAUTIONS
#       None
#
#   EXAMPLES
#       fims.sh -file $HOME/FITS/ngc0815.fits -catalog $HOME/cat/gsc0815.ptl B
#       fims.sh -rtd 1 -with_grid 1 -scrollbars 1 -float_panel 1 --debug -- R
#
#   SEE ALSO
#       None
#
#   BUGS
#       None
#
#-------------------------------------------------------------------------------
#        1         2         3         4         5         6         7         8
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#
# Include BASH library
#
bashlib=`dirname $0`/bash_library.sh
if [ -f "$bashlib" ]; then
    source $bashlib 2>&-
    status=$?
else
    echo "ERROR: $bashlib not found."
    exit 1
fi
if [ $status -ne 0 ]; then
    echo "ERROR: Code $status sourcing $bashlib."
    exit 1
fi

#################################
# Standard Library Overload BEGIN
#
#
# Standard Library Overload END
###############################

###############################
# Standard Initialization BEGIN
#
# Constants
#
# Text formatting for display and log file
LMARGIN=0 ; # Number of columns on the left margin
RMARGIN=0 ; # Number of columns in the right margin
MARGIN=$(printf "%*s" ${LMARGIN}) ; # String of blanks for left margin
#
# Standard Initialization END
#############################

#####################
# Local Library BEGIN
#
#************#
# Parameters #
#************#
#
#
# ValCAMERA: Validates that CAMERA's received value is in the
#            accepted range.
#
ValCAMERA() {
    # Parameters:
    # 1: Value of CAMERA parameter
    # Globals:
    # - CAMERA_LIST: List of parameter values
    # - CAMERA_HELP: List of help lines separated by $LIFS
    # Assigns:
    # - eflag: Error flag TRUE
    # Returns:    nothing
    local _item
    for _item in $CAMERA_LIST; do
        if [ "$*" = "$_item" ]; then
            return
        fi
    done
    eflag=$EC_REJECT
    ParVal CAMERA "$*" "$CAMERA_HELP"
}

#*************#
# Environment #
#*************#
#

#
# Local Library END
###################

############################
# Local Initialization BEGIN
#
# Local constants
#
CAMERA_LIST="\
       B \
       R \
            " # A list of valid values to chose from

CAMERA_HELP="\
       B: FORS Blue CCD$LIFS\
       R: FORS Red CCD$LIFS\
            "

# Some options are kept hidden from users but available to developers:
# batch, debug, trace, verbose, screen
HELP_LINE="\
 [[-l|--log]$LIFS\
  [<skycat opts>]$LIFS\
  [--help] [--usage]$LIFS\
  [--License] [--Version]$LIFS\
  --]$LIFS\
 <camera> $LIFS\
          "

USAGE_LINE="\
 [[-l|--log] $LIFS\
  [<skycat opts>]$LIFS\
  [--help] $LIFS\
  [--usage] $LIFS\
  [--License] $LIFS\
  [--Version] $LIFS\
  --] $LIFS\
 <camera> $LIFS\
 where: $LIFS\
   -l or --log: enables log to file $LIFS\
  --help: displays command line syntax $LIFS\
  --usage: displays this Usage Screen $LIFS\
  --License: displays copyright and license $LIFS\
  --Version: displays the program version $LIFS\
  --: marks end of options, starts positional arguments$LIFS\
    camera: [B|R] Code letter of the FORS CCD $LIFS\
             B: Configure FIMS for FORS Blue CCD $LIFS\
             R: Configure FIMS for FORS Red CCD $LIFS\
             No Default $LIFS\
   ?skycat opts?: command line options passed to skycat $LIFS\
                  (use 'skycat -?' to see all skycat options available) $LIFS\
                 -cat <bool>              * Include ESO/Archive catalog extensions (default). $LIFS\
                 -catalog <name>          * Open a window for the given catalog on startup. $LIFS\
                 -colorramp_height <n>    * height of colorramp window (default: 12). $LIFS\
                 -float_panel <bool>      * put info panel in a popup window (default: 0). $LIFS\
                 -panel_layout <layout>   * panel layout, one of: 'saoimage', 'reverse', 'default'. $LIFS\
                 -panel_orient <orient>   * panel orientation, one of: 'horizontal', 'vertical' $LIFS\
                 -pickobjectorient <v>    * orientation for pick object win: 'horizontal', 'vertical' $LIFS\
                 -min_scale <n>           * minimum scale for magnification menu (default: -10). $LIFS\
                 -max_scale <n>           * maximum scale for magnification menu (default: 20). $LIFS\
                 -remote <bool>           * Use existing skycat process, if available, with Tk send. $LIFS\
                 -debug <bool>            * debug flag: run bg processes in fg. $LIFS\
                 -default_cmap <cmap>     * default colormap. $LIFS\
                 -default_itt <itt>       * default intensity transfer table. $LIFS\
                 -file <file>             * fits file to load ('-' for stdin). $LIFS\
                 -port <port>             * Listen for remote cmds on port (default: 0 = choose port). $LIFS\
                 -rtd <bool>              * Include ESO/VLT Real-Time Features. $LIFS\
                 -scrollbars <bool>       * Display scrollbars (not displayed by default). $LIFS\
                 -shm_data <bool>         * Put image data in sysV shared memory. $LIFS\
                 -shm_header <bool>       * Put image header in sysV shared memory. $LIFS\
                 -usexshm <bool>          * Use X shared mem, if available (default). $LIFS\
                 -use_zoom_view <bool>    * Use a 'view' of the image for the zoom window (default). $LIFS\
                 -verbose <bool>          * Print diagnostic messages. $LIFS\
                 -with_colorramp <bool>   * Display the color bar (default). $LIFS\
                 -with_warp <bool>        * add bindings to move mouse ptr with arrow keys (default: 1). $LIFS\
                 -with_grid <bool>        * Include a WCS grid button (default: 0 = off). $LIFS\
                 -with_pan_window <bool>  * Display the pan window (default). $LIFS\
                 -with_zoom_window <bool> * Display the zoom window (default). $LIFS\
                 -zoom_factor <n>         * zooming factor (default: 4). $LIFS\
          "

LICENSE_LINE="\
  Copyright (C) $(ExpandRange 2013..$yearstamp)$LIFS\
  Pedro Baksai (pbaksai@eso.org)$LIFS\
  $LIFS\
  This program is free software: you can redistribute$LIFS\
  it and/or modify it under the terms of the$LIFS\
  GNU General Public License as published by$LIFS\
  the Free Software Foundation, either version 3 of$LIFS\
  the License, or (at your option) any later version.$LIFS\
  $LIFS\
  This program is distributed in the hope that it$LIFS\
  will be useful, but WITHOUT ANY WARRANTY,$LIFS\
  without even the implied warranty of$LIFS\
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.$LIFS\
  See the GNU General Public License for more details.$LIFS\
  $LIFS\
  You should have received a copy of the$LIFS\
  GNU General Public License along with this program.$LIFS\
  If not, see <http://www.gnu.org/licenses/gpl.html>.$LIFS\
             "

#
# Local variables
#
WORKING_DIR=`pwd`
bflag=0
sflag=0
tflag=0
vflag=0
display=$DISPLAY
CAMERA=U
SCOPTS=""

#
# Local Initialization END
##########################

################
# MAIN PROGRAM #
################

#########################
# Standard Prologue BEGIN
#
StartLog

# Receive options and parameters
#
# Options may be followed by one colon to indicate they have a required argument,
# or by two colons to indicate they have an optional argument.
# All SkyCat options are included and they all require an argument.
options=$(GetOpt -q -o btvl::? -a --longoptions batch,\
                                  --longoptions screen:,\
                                  --longoptions trace,\
                                  --longoptions verbose,\
                                  --longoptions cat:,\
                                  --longoptions catalog:,\
                                  --longoptions colorramp:,\
                                  --longoptions float_panel:,\
                                  --longoptions panel_layout:,\
                                  --longoptions panel_orient:,\
                                  --longoptions pickobjectorient:,\
                                  --longoptions min_scale:,\
                                  --longoptions max_scale:,\
                                  --longoptions remote:,\
                                  --longoptions default_cmap:,\
                                  --longoptions default_itt:,\
                                  --longoptions file:,port:,\
                                  --longoptions rtd:,\
                                  --longoptions scrollbars:,\
                                  --longoptions shm_data:,\
                                  --longoptions shm_header:,\
                                  --longoptions usexshm:,\
                                  --longoptions use_zoom_view:,\
                                  --longoptions with_colorramp:,\
                                  --longoptions with_warp:,\
                                  --longoptions with_grid:,\
                                  --longoptions with_pan_window:,\
                                  --longoptions with_zoom_window:,\
                                  --longoptions zoom_factor:,\
                                  --longoptions debug,\
                                  --longoptions log::,\
                                  --longoptions help,\
                                  --longoptions usage,\
                                  --longoptions License,\
                                  --longoptions Version \
                                  -n $progname -- "$@")
status=$?

# Processes valid options
eval set -- "$options"
while [ $# -gt 0 ]; do
    case $1 in
        -b | --batch)
            bflag=1
            Banner "Batch ON."
            Frame
            ;;
        --debug)
            dflag=1
            Debug "Turned ON."
            Debug "Options received: >$options<"
            # Pass it down to SkyCat
            SCOPTS="$SCOPTS -debug 1"
            ;;
        -l | -log | --log)
            Debug "Processing option $1 with optional argument >$2<"
            lflag=1
            StartLog "$2"
            shift
            ;;
        -s | --screen)
            Debug "Processing option $1 with required argument $2."
            sflag=1
            display=$2
            Warning "FIMS opens in $display screen"
            shift
            ;;
        -t | --trace)
            Debug "Processing option $1 without argument."
            tflag=1
            Banner "Trace ON."
            ;;
        -v | --verbose)
            Debug "Processing option $1 without argument."
            vflag=1
            # Pass it down to SkyCat
            SCOPTS="$SCOPTS -verbose 1"
            Banner "Verbose ON."
            ;;
        --cat | \
        --catalog | \
        --colorramp | \
        --float_panel | \
        --panel_layout | \
        --panel_orient | \
        --pickobjectorient | \
        --min_scale | \
        --max_scale | \
        --remote | \
        --default_cmap | \
        --default_itt | \
        --file | \
        --port | \
        --rtd | \
        --scrollbars | \
        --shm_data | \
        --shm_header | \
        --usexshm | \
        --use_zoom_view | \
        --with_colorramp | \
        --with_warp | \
        --with_grid | \
        --with_pan_window | \
        --with_zoom_window | \
        --zoom_factor)
            Debug "Processing option $1 with required argument $2."
            # SkyCat passthrough:
            # Cumulate options and their arguments
            SCOPTS="$SCOPTS $1 $2"
            shift
            ;;
        --help | -\?)
            Debug "Processing option $1 without argument."
            Help Syntax "$HELP_LINE"
            Exit $EC_INFO
            ;;
        --usage)
            Debug "Processing option $1 without argument."
            Help Usage "$USAGE_LINE"
            Exit $EC_INFO
            ;;
        --License)
            Debug "Processing option $1 without argument."
            Help License "$LICENSE_LINE"
            Exit $EC_INFO
            ;;
        --Version)
            Debug "Processing option $1 without argument."
            Help Version "$(Version)"
            Exit $EC_INFO
            ;;
        (--)
            Debug "End of options detected."
            shift
            break
            ;;
        (--*)
            Error "invalid long option $1"
            eflag=$EC_REJECT
            ;;
        (-?)
            Error "invalid short option $1"
            eflag=$EC_REJECT
            ;;
        (*)
            Debug "Argument $1 detected."
            break
            ;;
    esac
    shift
done

# Catches invalid options
if [ $status -ne 0 ]; then
    Warning "code $status: unrecognized or invalid option(s) ignored"
fi

# Processes valid non-option arguments
shift $((OPTIND-1))
case $# in
    (0)
        # Use default CAMERA
        ;;
    (*)
        CAMERA=$1
        shift
        ;;
esac

# Catches remaining non-option arguments
while [ $# -gt 0 ]; do
    case $1 in
        (*)
            Warning "invalid argument(s) $1 ignored"
            ;;
    esac
    shift
done

#
# Validate argument values
#
ValCAMERA $CAMERA

#
# Validate environment variables
#

#
# Exit if error
#
if [ $eflag -ne 0 ]; then
    HelpAd
    Exit $eflag
fi

#
# Set debug level
#
dflag=$(( dflag || dbg ))

#
# Set log level
#
lflag=$(( lflag || log ))

#
# Standard Prologue END
#######################

######################
# Main Algorithm BEGIN
#
if [ $dflag -eq 1 ]; then
    export FIMS_DEBUG=ON
fi
if [ $sflag -eq 1 ]; then
    export DISPLAY=$display
fi
if [ $tflag -eq 1 ]; then
    export FIMS_TRACE=ON
fi
if [ $vflag -eq 1 ]; then
    export FIMS_VERBOSE=ON
fi

# Default options for SkyCat
if [ -z "$SCOPTS" ]; then
    SCOPTS="-with_grid 1 -scrollbars 1 -rtd 0"
fi

# Strip quotes from string coming from shell
# and replace double hyphens with -
SCOPTS=`echo $SCOPTS | sed -e s/\"//g -e s/--/-/g`

case $CAMERA in
    B)
        Tell "Configuring FIMS for FORS Blue..."
        ;;
    R)
        Tell "Configuring FIMS for FORS Red..."
        ;;
    *)
        Error "Invalid camera $CAMERA."
        Explain "Camera should be B (Blue) or R (Red)."
        Exit 1
esac

# Receive required versions from installer script
TCLTK_REQVER=@@TCLTKREQVER@@
SKYCAT_REQVER=@@SKYCATREQVER@@

# Receive installation paths from installer script
FIMS_BASE_PATH=@@FIMSBASEPATH@@
BINPATH=@@BINPATH@@
LIBPATH=@@LIBPATH@@
BIN_PATH=`echo ${BINPATH} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
LIB_PATH=`echo ${LIBPATH} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`

# Extract binpath if present, then add it again. This way, if it is not present
# then it gets added, else it is not copied. Same for libpath.
CMD="echo $PATH | `echo $BIN_PATH: | awk '{printf(\"sed -e s/%s//g\",$1)}'`"
Debug "CMD BIN = >$CMD<"
SUBPATH=`eval $CMD`
Debug "SUBPATH BIN = >$SUBPATH<"
export PATH=$BINPATH:$SUBPATH

if [ -n "$SHLIB_PATH" ]; then
    # HP-UX library path
    LD_LIBRARY_PATH="$SHLIB_PATH"
fi

CMD="echo $LD_LIBRARY_PATH | `echo $LIB_PATH: | awk '{printf(\"sed -e s/%s//g\",$1)}'`"
Debug "CMD LIB = >$CMD<"
SUBPATH=`eval $CMD`
Debug "SUBPATH LIB = >$SUBPATH<"
export LD_LIBRARY_PATH=$LIBPATH:$SUBPATH
export SHLIB_PATH=$LIBPATH:$SUBPATH

export HOST=$ws
export FORS_CAM=$CAMERA
export TCLLIBPATH=@@TCLPATH@@
export VLTROOT=@@VLTPATH@@
export INSROOT=@@INSPATH@@
export INSUSER=@@USER@@
export FIMSROOT=$FIMS_BASE_PATH
export ISF_DIR=$FIMS_BASE_PATH/../config
export PREP_DIR=$HOME/$INSUSER/PREP
export LOG_DIR=$HOME/$INSUSER/LOG
export SET_DIR=$HOME/$INSUSER/SET
ESO_PLUGIN=${FIMS_BASE_PATH}/lib/libesoPlugins.tcl/GraphFeatures_plugin.tcl
FC_PLUGIN=${FIMS_BASE_PATH}/ext/lib/libfcPlugins.tcl/FindingChart_plugin.tcl
FIMS_PLUGIN=${FIMS_BASE_PATH}/lib/libfimsPlugins.tcl/SkyCat_plugin.tcl
export SKYCAT_PLUGIN=$FC_PLUGIN:$ESO_PLUGIN:$FIMS_PLUGIN
export SKYSEARCH_PLUGIN=${FIMS_BASE_PATH}/lib/libfimsPlugins.tcl/SkySearch_plugin.tcl
# export SKYCAT_CONFIG=file:${FIMS_BASE_PATH}/../config/skycat.cfg
export SKYCAT_CONFIG=file:${HOME}/${INSUSER}/skycat.cfg
export SKYCAT_LOGO=${FIMS_BASE_PATH}/../bitmap/skycat-logo.gif
export TCS_ENVNAME=wt0tcs
unset MODROOT

if [ $dflag -eq 1 ]; then
    Debug "HOST             = >$HOST<"
    Debug "FIMS_BASE_PATH   = >$FIMS_BASE_PATH<"
    Debug "BINPATH          = >$BINPATH<"
    Debug "LIBPATH          = >$LIBPATH<"
    Debug "ESO_PLUGIN       = >$ESO_PLUGIN<"
    Debug "FC_PLUGIN        = >$FC_PLUGIN<"
    Debug "FIMS_PLUGIN      = >$FIMS_PLUGIN<"
    Debug "SKYSEARCH_PLUGIN = >$SKYS_PLUGIN<"
    Debug "SKYCAT_CONFIG    = >$SKYCAT_CONFIG<"
    Debug "SKYCAT_LOGO      = >$SKYCAT_LOGO<"
    Debug "TCLLIBPATH       = >$TCLLIBPATH<"
    Debug "VLTROOT          = >$VLTROOT<"
    Debug "TCLROOT          = >$TCLTK_ROOT<"
    Debug "INSROOT          = >$INSROOT<"
    Debug "INSUSER          = >$INSUSER<"
    Debug "PREP_DIR         = >$PREP_DIR<"
    Debug "LOG_DIR          = >$LOG_DIR<"
    Debug "SET_DIR          = >$SET_DIR<"
    Debug "PATH             = >$PATH<"
    Debug "LD_LIBRARY_PATH  = >$LD_LIBRARY_PATH<"
    Debug "SHLIB_PATH       = >$SHLIB_PATH<"
fi

# Verification of environment
if [ $bflag -eq 1 -a ! -f $FIMS_BASE_PATH/bin/fims_ec ]; then
    Error "Batch mode not available"
    eflag=2
fi
for d in $PREP_DIR $LOG_DIR $SET_DIR; do
    test -d $d || mkdir -p $d > /dev/null 2>&1
    chmod -f 777 $d > /dev/null 2>&1
done

if [ $eflag -ne 0 ]; then
    Exit $eflag
fi

# Find out component versions
if [ -a $FIMS_BASE_PATH/../doc/FIMSPACK_svn.txt ]; then
    SVN_INFO=`cat $FIMS_BASE_PATH/../doc/FIMSPACK_svn.txt`
else
    SVN_INFO="Path: unknown URL: unknown Repository Root: unknown Repository UUID: 00000000-0000-0000-0000-000000000000 Revision: 000000 Node Kind: directory Schedule: normal Last Changed Author: unknown Last Changed Rev: 000000 Last Changed Date: 0000-00-00 00:00:00 +0000 (Dow, 00 Mon 0000)"
fi

if [ -a $FIMS_BASE_PATH/../doc/tcltk_version.txt ]; then
    TCLTK_VER=`cat $FIMS_BASE_PATH/../doc/tcltk_version.txt`
    Debug "TCLTK_VER = >$TCLTK_VER< from $FIMS_BASE_PATH/../doc/tcltk_version.txt"
else
    TCLTK_VER=$(echo "puts [info tclversion]" | tclsh 2>/dev/null)
    Debug "TCLTK_VER = >$TCLTK_VER< from tclsh"
fi
if [ -z $TCLTK_VER ]; then
    Warning "TclTk installation could not be found in this system."
    Explain "Try installing FIMS with -r option."
    TCLTK_VER='0.0'
    Debug "TCLTK_VER = >$TCLTK_VER< not found."
fi

# Exports the SkyCat version required by FIMS, in case the one in this
# package gets replaced.
if [ -a $FIMS_BASE_PATH/../doc/skycat_version.txt ]; then
    SKYCAT_BASE=$FIMS_BASE_PATH/ext
    Debug "SKYCAT_BASE = >$SKYCAT_BASE<"
    SKYCAT_VER=`cat $FIMS_BASE_PATH/../doc/skycat_version.txt`
    Debug "SKYCAT_VER = >$SKYCAT_VER< from $FIMS_BASE_PATH/../doc/skycat_version.txt"
else
    # Try to find installed SkyCat version
    SKYCAT_BASE=""
    prg=`which skycat 2>/dev/null`
    Debug "which skycat = >$prg<"
    while [ -h "$prg" ]; do
        ls=`ls -ld "$prg"`
        link=`expr "$ls" : '.*-> \(.*\)$'`
        if expr "$link" : '/.*' > /dev/null; then
            prg="$link"
        else
            prg=`dirname "$prg"`/"$link"
        fi
    done
    if [ -n "$prg" ]; then
        dir=`dirname "$prg"`/..
        SKYCAT_BASE=`(cd $dir; pwd)`
    fi
    libs=`echo $LD_LIBRARY_PATH | sed s/:/' '/g`
    if [ -n "$SKYCAT_BASE" ]; then
        libs="$libs $SKYCAT_BASE/lib"
        Debug "SKYCAT_BASE = >$SKYCAT_BASE<"
    fi
    files=""

    for l in $libs ; do
        if [ -d $l ]; then
            files="$files `find $l -name skycatConfig.sh`"
        fi
    done
    files=`echo $files`

    v=""
    for f in $files ; do
        v=`grep skycat_VERSION $f`
    done
    if [ -n "$v" ]; then
        SKYCAT_VER=`echo $v | sed s/skycat_VERSION=//g | sed s/[\']//g`
        Debug "SKYCAT_VER = >$SKYCAT_VER< from skycatConfig.sh"
    fi
fi
if [ -z $SKYCAT_VER ]; then
    Warning "SkyCat installation could not be found in this system."
    Explain "Try installing FIMS with -r option."
    SKYCAT_VER='0.0.0'
    Debug "SKYCAT_VER = >$SKYCAT_VER< not found."
fi
export SKYCAT_VER

if [ $dflag -eq 1 ]; then
    WISH_PATH=`which wish${TCLTK_VER} 2>/dev/null`
    MOS_PATH=`which fsmosaic 2>/dev/null`
    SKY_PATH="${SKYCAT_BASE}/lib/libskycat${SKYCAT_VER}.so"
    LDD_WISH=`ldd -d $WISH_PATH 2>/dev/null`
    LDD_MOS=`ldd -d $MOS_PATH 2>/dev/null`
    LDD_SKY=`ldd -d $SKY_PATH 2>/dev/null`
    Debug "Library linkage  = >WISH: "
    Debug "$LDD_WISH<"
    Debug "Library linkage  = >MOSAIC: "
    Debug "$LDD_MOS<"
    Debug "Library linkage  = >SKYCAT: "
    Debug "$LDD_SKY<"
fi

# Check against required versions
if [ ${TCLTK_VER//./} -lt ${TCLTK_REQVER//./} ]; then
    Warning "TclTk version $TCLTK_VER is older than required by FIMS ($TCLTK_REQVER)"
fi
if [ ${SKYCAT_VER//./} -lt ${SKYCAT_REQVER//./} ] ; then
    Warning "SkyCat version $SKYCAT_VER is older than required by FIMS ($SKYCAT_REQVER)"
fi

if [ -z "$MOD_VERSION" ]; then
    MOD_DATE=`echo ${SVN_INFO/Path:*Last Changed Date: /} | awk '{printf("%s", $1)}'`
    MOD_TIME=`echo ${SVN_INFO/Path:*Last Changed Date: /} | awk '{printf("%s", $2)}'`
    MOD_VERSION=`echo ${SVN_INFO/Path:*Last Changed Rev: /} | awk '{printf("%s", $1)}'`
    export MOD_VERSION
    Debug "MOD_VERSION = >$MOD_VERSION<"
fi
M_V=${MOD_VERSION:-000000}
S_V=$SKYCAT_VER
line1=`echo "" | awk -v MV=$M_V -v SV=$S_V '{ printf("FIMS  %8s |----plugin----> | SkyCat %8s |\n", MV, SV) }'`
line2=`echo "" | awk -v MD=${MOD_DATE:-0000-00-00} -v MT=${MOD_TIME:-00:00:00} '{ printf("last checkin : %10s GD %8s UT", MD, MT) }'`

echo " "
Frame
Banner "  W E L C O M E     T O     F I M S"
Banner " -----------------------------------"
Banner
Banner " +----------------+                +-----------------+"
Banner " | $line1"
Banner " +----------------+                +-----------------+"
Filler
Banner " +------------------------------------------+"
Banner " | $line2 |"
Banner " | now is       : `date -u +%Y-%m-%d` GD `date -u +%H:%M:%S` UT |"
Banner " | FIMS requires: SkyCat ${SKYCAT_REQVER} & TclTk ${TCLTK_REQVER}  |"
Banner " +------------------------------------------+"
Banner " INPUT : $PREP_DIR"
Banner " OUTPUT: $SET_DIR"
Banner " LOGS  : $LOG_DIR"
Filler
Banner "skycat ${SCOPTS}..."
Frame
# Restore ^Z signal
trap SIGTSTP
if [ $bflag -eq 1 ]; then
    test -z "$INTROOT" && (Error "FIMS is not available in batch mode in this installation.") && Exit $EC_ABORT
    fims_ec $SCOPTS &
else
    test -z "$SKYCAT_BASE" && (Error "SkyCat tool not found.") && Exit $EC_ABORT
    skycat $SCOPTS &
fi
#
# Main Algorithm END
####################

#########################
# Standard Epilogue BEGIN
#
Exit $EC_QUIET
#
# Standard Epilogue END
#######################
#
#___FIMS___
