#!/bin/bash
#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: fims_install.sh 259761 2014-09-21 18:09:28Z pbaksai $"
#
# who       when        what
# --------  ----------  --------------------------------------------------------
# pbaksai   2014/09/22  CCB-000579: Support for 64 bits and portability.
#                       - Corrected TCLTK_ROOT.
# pbaksai   2014/09/20  CCB-000579: Support for 64 bits and portability.
#                       - Upgraded qfits library in fsmosaic.
#                       - Corrected version handling.
#                       - Corrected path handling.
# pbaksai   2014/07/25  CCB-000579: Support for 64 bits and portability.
#                       - Check errors with CheckStatus.
#                       - Check support for 32 bits libraries when needed.
# pbaksai   2014/07/10  CCB-000579: Support for 64 bits and portability.
#                       - Do not use tar xf - -C (not portable).
#                       - Check that package supports system architecture.
#                       - Check status of all compilations.
#                       - Check and search for required libraries.
# pbaksai   2014/05/14  CCB-000579: Support for 64 bits and portability.
#                       - Redirect configure output to compilation log file.
#                       - Use portable tar command.
#                       - Added soft link to X11 includes.
# pbaksai   2014/01/28  CCB-000579: Support for 64 bits and portability.
#                       - Use portable ls_nobackup option.
# pbaksai   2013/10/17  CCB-000579: Support for 64 bits and portability.
#                       - Reintegrated Tcl/Tk and SkyCat code that gets
#                         compiled with -cr option.
# pbaksai   2013/01/23  PPRS-23355: Install pre-compiled SkyCat and Tcl/Tk with
#                       option -r (required).
# pbaksai   2012/12/28  PPRS-23355: Construction of PATH and LD_LIBRARY_PATH
#                       moved to fims.sh.tpl for the case installer user is not
#                       the same as runtime user (case in wgsoffX machines).
# pbaksai   2012/12/13  PPRS-23355: Eliminated Tcl/Tk and SkyCat code.
# pbaksai   2012/11/28  PPRS-23355: Corrections after testing.
# pbaksai   2012/11/05  PPRS-23355: Corrections after testing.
# pbaksai   2012/11/04  PPRS-23355: Completed.
# pbaksai   2012/09/22  PPRS-23355: Created.
#
#*******************************************************************************
#   NAME
#       fims_install.sh - Package installation script for FIMS.
#
#   SYNOPSIS
#       fims_install.sh [[-d|--debug]
#                        [-l|--log]
#                        [-c|--compile]
#                        [-r|--required]
#                        [-u|--uninstall]
#                        [--help]
#                        [--License]
#                        [--Usage]
#                        [--Version]
#                        --]
#                       [<path>]
#
#
#   DESCRIPTION
#       This script installs FIMS package distribution in user's environment.
#
#   PARAMETERS
#        -d|--debug       : Enables debugging mode.
#        -l|--log         : Enables logging mode.
#        -c|--compile     : Recompiles all package components in user's
#                           environment.
#        -r|--required    : Installs pre-compiled required packages:
#                           Tcl/Tk, SkyCat
#        -u|--uninstall   : Removes FIMS package installation.
#       --help            : Displays a help screen explaining the syntax.
#       --License         : Displays copyright and license information.
#       --Usage           : Displays a help screen explaining the interface.
#       --Version         : Displays program version.
#
#   ARGUMENTS
#       ?<path>?          : Path to the installation directory.
#                           Default: current directory.
#
#   INPUTS
#
#   OUTPUTS
#       <stdout>
#           <exec log>    : Log of execution.
#       <file>
#           <logfile>     : <$PWD>/fims_install.log
#           <script>      : <path>/fims<OS><bits>.uninstall
#           <package>     : <path>/fims<OS><bits>.tar.gz
#
#   FILES
#       bash_library.sh
#
#   ENVIRONMENT
#       USER
#       HOST
#       HOME
#       SHELL_LOG
#       SHELL_DEBUG
#       VLTROOT
#       INTROOT
#
#   RETURN VALUES
#       0: All OK.
#       1: Cancelled due to insufficient conditions.
#       2: Aborted due to errors.
#       3: Rejected due to incorrect arguments.
#       4: Ended with information display only (help, usage, license, version).
#
#   CAUTIONS
#       None
#
#   EXAMPLES
#       None
#
#   SEE ALSO
#       None
#
#   BUGS
#       None
#
#-------------------------------------------------------------------------------
#        1         2         3         4         5         6         7         8
#23456789 123456789 123456789 123456789 123456789 123456789 123456789 1234567890
#
# Include BASH library
#
bashlib=`dirname $0`/bash_library.sh
if [ -f "$bashlib" ]; then
    source $bashlib 2>&-
    status=$?
else
    echo "ERROR: $bashlib not found."
    exit 1
fi
if [ $status -ne 0 ]; then
    echo "ERROR: Code $status sourcing $bashlib."
    exit 1
fi

#################################
# Standard Library Overload BEGIN
#
#
# Standard Library Overload END
###############################

###############################
# Standard Initialization BEGIN
#
#
# Standard Initialization END
#############################

#####################
# Local Library BEGIN
#
#************#
# Parameters #
#************#
#

#*************#
# Environment #
#*************#
#

#
# SafeLink: Makes a symbolic link from source to destination with backup.
#
SafeLink() {
    # Parameters:
    # 1: Source path
    # 2: Destination path
    # Globals:
    # - PACK_NAME: Base name of the package
    # - upath: Path to uninstaller script
    # Assigns:    nothing
    # Returns:
    # 0: All OK
    # 1: Error linking source to destination
    local _source=$1
    local _dest=$2
    Debug "Source     : >$_source<"
    Debug "Destination: >$_dest<"
    if [ -z "$_source" -o -z "$_dest" ]; then
        Warning "could not link, source and/or destination are/is blank"
        Explain "Source     : >$_source<"
        Explain "Destination: >$_dest<"
        return 1
    fi
    # Remove existing destination link to package item
    # so that it does not get backed up again.
    if [ -h $_dest ]; then
        if [ "`ls -laA$ls_nobackup $_dest 2>&- | grep $PACK_NAME`" != "" ]; then
            rm -f $_dest 2>&-
        fi
    fi
    # If a backup exists, move it to original destination 
    if [ -h ${_dest}.fims ]; then
        mv -f ${_dest}.fims $_dest 2>&-
    fi
    # If destination exists, make a backup with .fims suffix
    if [ -h $_dest -o -f $_dest -o -d $_dest ]; then
        mv -f $_dest ${_dest}.fims 2>&-
    fi
    # Link destination if exists
    Debug "ln -f -s $_source $_dest"
    # ln --suffix=.fims -b -f -s $_source $_dest 2>&-
    ln -f -s $_source $_dest 2>&-
    if [ $? -ne 0 ]; then
        Warning "Error: ln -f -s $_source $_dest failed"
        Explain "Symbolic link from $_source to $_dest was not established."
    fi
    # Register in uninstaller macro
    if [ -f $upath ]; then
        # Remove link installed by package
        Debug "echo rm -f $_dest 2>&- >> $upath"
        echo "rm -f $_dest 2>&-" >> $upath
        # Restore backup made by ln -b above
        Debug "echo mv -f ${_dest}.fims $_dest 2>&- >> $upath"
        echo "mv -f ${_dest}.fims $_dest 2>&-" >> $upath
    fi
    return 0
}
#
# Local Library END
###################

############################
# Local Initialization BEGIN
#
# Local constants
#
HELP_LINE="\
 [[-d|--debug] [-l|--log]$LIFS\
  [-c|--compile]$LIFS\
  [-r|--required]$LIFS\
  [-u|--uninstall]$LIFS\
  [--help] [--License]$LIFS\
  [--Usage] [--Version]$LIFS\
  --]$LIFS\
 [<path>] $LIFS\
          "

USAGE_LINE="\
 [[-d|--debug] $LIFS\
  [-l|--log] $LIFS\
  [-c|--compile] $LIFS\
  [-r|--required] $LIFS\
  [-u|--uninstall] $LIFS\
  [--help] $LIFS\
  [--License] $LIFS\
  [--Usage] $LIFS\
  [--Version] $LIFS\
  --] $LIFS\
 [<path>] $LIFS\
 where: $LIFS\
   -d or --debug: enables debugging mode $LIFS\
   -l or --log: enables log to file $LIFS\
   -c or --compile: recompiles FIMS executables 
   -r or --required: installs pre-compiled packages: Tcl/Tk, SkyCat 
   -u or --uninstall: removes FIMS package 
  --help: displays command line syntax $LIFS\
  --License: displays copyright and license $LIFS\
  --Usage: displays this Usage Screen $LIFS\
  --Version: displays the program version $LIFS\
  --: marks end of options, starts positional arguments$LIFS\
   ?path?: path to the installation directory $LIFS\
           Default is current directory $LIFS\
          "

LICENSE_LINE="\
  Copyright (C) $(ExpandRange 2013..$yearstamp)$LIFS\
  Pedro Baksai (pbaksai@eso.org)$LIFS\
  $LIFS\
  This program is free software: you can redistribute$LIFS\
  it and/or modify it under the terms of the$LIFS\
  GNU General Public License as published by$LIFS\
  the Free Software Foundation, either version 3 of$LIFS\
  the License, or (at your option) any later version.$LIFS\
  $LIFS\
  This program is distributed in the hope that it$LIFS\
  will be useful, but WITHOUT ANY WARRANTY,$LIFS\
  without even the implied warranty of$LIFS\
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.$LIFS\
  See the GNU General Public License for more details.$LIFS\
  $LIFS\
  You should have received a copy of the$LIFS\
  GNU General Public License along with this program.$LIFS\
  If not, see <http://www.gnu.org/licenses/gpl.html>.$LIFS\
             "

#
# Local variables
#
TCLTK_REQVER=8.4
SKYCAT_REQVER=3.1.2
PERIOD=95
INSTRUMENT=FORS2
PACK_NAME=fimsLIN32
INSUSER=.fims
INSROOT=$HOME
PREP_DIR=$INSROOT/$INSUSER/PREP
LOG_DIR=$INSROOT/$INSUSER/LOG
SET_DIR=$INSROOT/$INSUSER/SET
UNPACK_DIR=`pwd`
INSTALL_DIR=""
cflag=0
rflag=0
uflag=0
ask=0

#
# Local Initialization END
##########################

################
# MAIN PROGRAM #
################

#########################
# Standard Prologue BEGIN
#
StartLog

# Receive options and parameters
#
options=$(GetOpt -q -o crudl::? -a --longoptions compile,required,uninstall,debug,log::,help,License,Usage,Version -n $progname -- "$@")
status=$?

# Processes valid options
eval set -- "$options"
while [ $# -gt 0 ]; do
    case $1 in
        -d | -debug | --debug)
            dflag=1
            Debug "Turned ON."
            Debug "Options received: >$options<"
            ;;
        -l | -log | --log)
            Debug "Processing option $1 with optional argument $2"
            lflag=1
            StartLog "$2"
            shift
            ;;
        -c | --compile)
            Debug "Processing option $1 without argument."
            cflag=1
            ;;
        -r | --required)
            Debug "Processing option $1 without argument."
            rflag=1
            ;;
        -u | --uninstall)
            Debug "Processing option $1 without argument."
            uflag=1
            ;;
        --help | -\?)
            Debug "Processing option $1 without argument."
            Help Syntax "$HELP_LINE"
            Exit $EC_INFO
            ;;
        --License)
            Debug "Processing option $1 without argument."
            Help License "$LICENSE_LINE"
            Exit $EC_INFO
            ;;
        --Usage)
            Debug "Processing option $1 without argument."
            Help Usage "$USAGE_LINE"
            Exit $EC_INFO
            ;;
        --Version)
            Debug "Processing option $1 without argument."
            Help Version "$(Version)"
            Exit $EC_INFO
            ;;
        (--)
            Debug "End of options detected."
            shift
            break
            ;;
        (--*)
            Error "Invalid long option $1"
            eflag=$EC_REJECT
            ;;
        (-?)
            Error "Invalid short option $1"
            eflag=$EC_REJECT
            ;;
        (*)
            Debug "Argument $1 detected."
            break
            ;;
    esac
    shift
done

# Catches invalid options
if [ $status -ne 0 ]; then
    Warning "Code $status: Unrecognized or invalid option(s) ignored"
fi

# Processes valid non-option arguments
shift $((OPTIND-1))
case $# in
    (0)
        # Use default INSTALL_DIR
        INSTALL_DIR=$UNPACK_DIR
        ask=1
        ;;
    (*)
        INSTALL_DIR=$1
        GREP=`echo $INSTALL_DIR | grep "^/"`
        if [ $? -ne 0 ]; then
            INSTALL_DIR=${PWD}/${INSTALL_DIR}
        fi
        if [ ! -d $INSTALL_DIR ]; then
            mkdir -p $INSTALL_DIR 2>/dev/null
            if [ $? -ne 0 ]; then
                Tell "Cannot create this directory!"
                Tell "Please choose another one..."
                INSTALL_DIR=$UNPACK_DIR
                ask=1
            else
                Tell "Created directory $INSTALL_DIR\n"
            fi
        elif [ ! -w $INSTALL_DIR ]; then
            Tell "You do not appear to have write access to"
            Tell "$INSTALL_DIR."
            Tell "Please choose another directory for installation."
            INSTALL_DIR=$UNPACK_DIR
            ask=1
        fi
        shift
        ;;
esac

# Catches remaining non-option arguments
while [ $# -gt 0 ]; do
    case $1 in
        (*)
            Warning "invalid argument $1 ignored"
            ;;
    esac
    shift
done

#
# Validate argument values
#

#
# Validate environment variables
#

#
# Exit if error
#
if [ $eflag -ne 0 ]; then
    HelpAd
    Exit $eflag
fi

#
# Set debug level
#
dflag=$(( dflag || dbg ))

#
# Set log level
#
lflag=$(( lflag || log ))

#
# Standard Prologue END
#######################

######################
# Main Algorithm BEGIN
#
Frame
Banner "BEGIN  $progname by ${user}@${ws} on ${datestamp}T${timestamp}"
Frame

# Check working directory
if [ ! -f "$progname" ]; then
    Error   "Please run this script from inside the package directory."
    Explain "cd /some/path/$PACK_NAME/"
    Exit $EC_CANCEL
fi

# Choice of installation directory
if [ $ask -ne 0 ]; then
    # If it is asking then INSTALL_DIR=UNPACK_DIR
    Doing "Choose Installation Directory:"
    Tell  "The default is the current directory."
fi
while [ $ask -eq 1 ]; do
    Tell "Please enter the installation directory now"
    echo
    answer="$(GetAnswer 'Installation dir ' \
                        $INSTALL_DIR \
                        'Type the full path to the installation directory' \
                        0\
             )"
    if [ -z "$answer" ]; then
        ask=2
    else
        absolut=`echo $answer | grep "^/"`
        if [ $? -ne 0 ]; then
            answer=${PWD}/$answer
        fi
        if [ ! -d "$answer" ]; then
            if [ $uflag -eq 0 ]; then
                makedir="$(GetAnswer 'Do you want to create '$answer' ?' \
                                     'Y' \
                                     'Answer Y(es) or N(o)' \
                                     0 \
                          )"
                if [ -z "$makedir" ]; then
                    makedir=Y
                fi
                if [ "$makedir" = "Y" ]; then
                    mkdir -p "$answer" 2>/dev/null
                    if [ $? -ne 0 ]; then
                        Tell "Cannot create this directory! Please choose another one..."
                    else
                        INSTALL_DIR="$answer"
                        ask=3
                        Tell "Created directory $INSTALL_DIR"
                    fi
                fi
            else
                Tell "$PACK_NAME package was not installed there! Please choose another one..."
            fi
        elif [ ! -w "$answer" ]; then
            Tell "You do not appear to have write access to"
            Tell "$answer"
            Tell "Please choose another installation directory."
        else
            INSTALL_DIR="$answer"
            ask=4
            Tell "OK with $INSTALL_DIR"
        fi
    fi
done
if [ $ask -ne 0 ]; then
    Done "Choosing Installation Directory."
    echo " "
fi

if [ $uflag -ne 0 ]; then
    Doing "Un-Installing FIMS package from $INSTALL_DIR"
    if [ -d ${INSTALL_DIR}/${PACK_NAME} ]; then
        cd ${INSTALL_DIR}/${PACK_NAME}
    else
        Error   "$PACK_NAME package was not installed there."
        Explain "${INSTALL_DIR}/${PACK_NAME} not found."
        Exit $EC_CANCEL
    fi
    if [ -f ${PACK_NAME}.uninstall ]; then
        if [ $dflag -ne 0 ]; then
            cat ${PACK_NAME}.uninstall
        fi
        # Execute uninstaller macro
        # Remove package directory
        # Remove package compressed files and installer
        # Leave only files distributed to users, if any
        cmd="
            chmod 755 ${PACK_NAME}.uninstall ;
            ./${PACK_NAME}.uninstall ;
            mv -f ${PACK_NAME}.uninstall ${PACK_NAME}.uninstall.done ;
            cd ../. ;
            rm -rf ${INSTALL_DIR}/${PACK_NAME} ;
            rm -f ${INSTALL_DIR}/${PACK_NAME}.tar.gz ;
            rm -f ${UNPACK_DIR}/${progname} ;
            rm -f ${UNPACK_DIR}/bash_library.sh ;
            rm -f ${UNPACK_DIR}/${PACK_NAME}.tar.gz ;
            rm -f /tmp/*_compilation_warnings.log ;
        "
        Waitfor "uninstalling ${PACK_NAME}..." "${cmd}" 1
        status=$?
        CheckStatus $status $status $LINENO $EC_END X "" 1
    else
        Error   "package is just unpacked, not installed"
        Explain "${PACK_NAME}.uninstall not found in"
        Explain "${INSTALL_DIR}/${PACK_NAME}"
        Explain "First install the package with:"
        Explain "~$> $progname $INSTALL_DIR"
        Exit $EC_CANCEL
    fi
    Done "Un-Installing FIMS package."
    Exit $EC_END
fi

Frame
Banner "       ** Welcome to the FIMS installation script **"
Frame
Banner "PLEASE NOTE: This package is to be used for period $PERIOD"
Banner "              of observations for the ESO telescopes."
Banner "              Current date $datestamp belongs to period $(Period)."
Filler
Banner "The Finding Chart plug-in included in this package may be"
Banner "used for any service mode program."
Banner "The FIMS tool in this package is specific to programs for"
Banner "$INSTRUMENT."
Frame

# Check package components
Doing "Checking package is present..."
if [ ! -f "${PACK_NAME}.tar.gz" ]; then
    Error   "Package ${PACK_NAME}.tar.gz not found"
    Explain "in $UNPACK_DIR"
    Explain "Make sure you have unpacked FIMS"
    Explain "distribution in current directory."
    Explain "Please execute gzip -d -c ${PACK_NAME}*.tar.gz | tar xf - "
    Explain "and then ./$progname"
    Exit $EC_CANCEL
else
    Tell "Package found here." 1
    os=`echo "${PACK_NAME}" | grep "${OSTAG}"`
    if [ $? -ne 0 ]; then
            Warning "Package ${PACK_NAME}.tar.gz does not seem to support this operating system."
            Explain "This is a $UNAME ($OSTAG) system."
    fi
fi
Done "Checking package is present."
echo " "

if [ $cflag -ne 0 ]; then
    Doing "Checking if C and C++ compilers are installed..."
    CC_EXEC=gcc
    CPP_EXEC=g++
    CCv=`$CC_EXEC -v  2>&1 | grep "$CC_EXEC version" | awk '{printf("%s",$3)}'`
    if [ -z "$CCv" ]; then
        Error "C compiler not found."
        Explain "Cannot re-compile modules."
        Exit $EC_ABORT
    else
        CC_PATH=`wich $CC_EXEC 2>/dev/null`
        Tell "C compiler version $CCv found in:"
        Tell "$CC_PATH" 1
        CPPv=`$CPP_EXEC -v  2>&1 | grep "$CC_EXEC version" | awk '{printf("%s",$3)}'`
        if [ -z "$CPPv" ]; then
            Error "C++ compiler not found."
            Explain "Cannot re-compile modules."
            Exit $EC_ABORT
        else
            CPP_PATH=`wich $CPP_EXEC 2>/dev/null`
            Tell "C++ compiler version $CPPv found in:"
            Tell "$CPP_PATH" 1
        fi
    fi
    Done "Checking C and C++ compilers."
    echo " "
fi

if [ $rflag -eq 0 ]; then
    Doing "Checking TCL interpreter is available..."
    TCL_EXEC=tclsh
    TCLv=$(echo "puts [info tclversion]" | $TCL_EXEC 2>/dev/null)
    if [ -z "$TCLv" ]; then
        Error "Tcl interpreter not found."
        Explain "Please run the installation again with -r option"
        Explain "to install Tcl/Tk from this package."
        eflag=$EC_ABORT
    else
        TCL_PATH=`which $TCL_EXEC 2>/dev/null`
        TCLTK_ROOT=$(dirname $(which $TCL_EXEC 2>/dev/null) 2>/dev/null)/..
        Tell "Tcl interpreter version $TCLv found in:"
        Tell "$TCL_PATH" 1
    fi
    Done "Checking TCL interpreter."
    echo " " 
    Doing "Checking SkyCat tool is available..."
    SKY_EXEC=skycat
    SKY_PATH=`which $SKY_EXEC 2>/dev/null`
    if [ -z "$SKY_PATH" ]; then
        Error "SkyCat tool not found."
        Explain "Please run the installation again with -r option"
        Explain "to install SkyCat from this package."
        eflag=$EC_ABORT
    else
        Tell "SkyCat tool found in:"
        Tell "$SKY_PATH" 1
    fi
    Done "Checking SkyCat tool."
    echo " " 
else
    TCLTK_ROOT=${INSTALL_DIR}/${PACK_NAME}/${BITS}b/ext
fi
test $eflag -eq 0 || Exit $eflag

tclroot=${TCLTK_ROOT}/lib/tcl[0-9].*

Doing "Checking PERL interpreter is available..."
PERL_EXEC=perl
PERLv=`$PERL_EXEC -v 2>/dev/null`
PERLv=`echo ${PERLv/This*[( ]v/} | sed -e 's/)/ /' | awk '{printf("%s\n",$1)}'`
if [ -z "$PERLv" ]; then
    Tell "Perl interpreter not found in your path."
    Tell "You can either decide to specify the path"
    Tell "to your Perl interpreter, or to use FIMS"
    Tell "without Perl (you won't be able to produce"
    Tell "Postscript files for MXU masks)"
    echo
    answer="$(GetAnswer 'Would you like to specify the path for '$PERL_EXEC' ?' \
                        'N' \
                        'Answer Y(es) or N(o)' \
                        0 \
             )"
    if [ -z "$answer" -o "$answer" = "N" -o "$answer" = "Q" ]; then
        Tell "FIMS will work without PERL."
    else
        answer=""
        default="/usr/bin/$PERL_EXEC"
        while [ -z "$answer" ]; do
            echo
            answer="$(GetAnswer 'Please enter the absolut path to '$PERL_EXEC \
                                $default \
                                'Type the absolut path to '$PERL_EXEC' in your system \
                                 starting with /' \
                                0 \
                     )"
            absolut=`echo $answer | grep "^/"`
            if [ $? -ne 0 ]; then
                Tell "Please enter an absolute path starting with /"
                default=$answer
                answer=""
            fi
        done
        perlname=`echo $answer | awk '{i=split($1,a,"/"); if (i > 0) printf("%s",a[i]); else print("")}'`
        if [ "x$perlname" == "x$PERL_EXEC" ]; then
            perlname=""
            $answer -v > /dev/null 2>&1
            status=$?
        else
            perlname="/$PERL_EXEC"
            $answer/$PERL_EXEC -v > /dev/null 2>&1
            status=$?
        fi
        if [ $status -eq 0 ]; then
            PERL_PATH=$answer
            Tell "FIMS will use $answer$perlname"
        else
            Tell "It did not work. Sorry."
            Tell "Nevermind. FIMS will work without PERL."
        fi
    fi    
else
    PERL_PATH=`which $PERL_EXEC 2>/dev/null`
    Tell "Perl interpreter version $PERLv found in: "
    Tell "$PERL_PATH" 1
fi
Done  "Checking PERL interpreter."
echo " "

Doing "Installing FIMS package in $INSTALL_DIR"
if [ -f "${UNPACK_DIR}/${PACK_NAME}.tar.gz" ]; then
    Doing "Unpacking ${UNPACK_DIR}/${PACK_NAME}.tar.gz" 1
    Tell  "to ${INSTALL_DIR}" 1
    cmd="
         rm -rf ${INSTALL_DIR}/${PACK_NAME} ;
         rm -rf ${INSTALL_DIR}/${PACK_NAME}.uninstall.done ;
         cd ${INSTALL_DIR} ;
         gzip -d -c ${UNPACK_DIR}/${PACK_NAME}.tar.gz | $tar xf - > /dev/null ;
    "
    Waitfor "uncompressing ${PACK_NAME}..." "${cmd}" 1
    status=$?
    CheckStatus $status $status $LINENO $EC_ABORT X "" 1
    Done "Unpacking." 1
    echo " "
    test $eflag -eq 0 || Exit $eflag
    Doing "Checking that package supports host architecture..." 1
    if [ ! -d ${INSTALL_DIR}/${PACK_NAME}/${BITS}b ]; then
        Error   "Package ${PACK_NAME}.tar.gz does not support this architecture."
        Explain "This is a ${BITS} bits system."
        Exit $EC_ABORT
    fi
    Done "Checking architecture." 1
    echo " "
else
    Error   "Package ${PACK_NAME}.tar.gz not found"
    Explain "in $UNPACK_DIR"
    Exit $EC_ABORT
fi

if [ "X$OSTAG" == "XLIN" -a "X$BITS" == "X64" ]; then
    Doing "Checking support for 32 bits libraries..." 1
    LD_LIB=/lib/ld-linux.so
    if [ -f ${LD_LIB}* ]; then 
        Tell "32 bits libraries are supported." 1
    else
        bits=$(GetELF "${INSTALL_DIR}/${PACK_NAME}/${BITS}b/lib/libpafchk.so")
        if [ $cflag -eq 0 -a X"$bits" = "X32" ]; then
            Error "There is no support for 32 bits libraries and this package needs it."
            Explain "You need to re-compile this package in your system."
            Explain "Run again $progname with options -cr"
            Exit $EC_ABORT
        fi
    fi
    Done "Checking support for 32 bits libraries." 1
    echo " "
fi

# Create uninstaller macro
upath=${INSTALL_DIR}/${PACK_NAME}/${PACK_NAME}.uninstall
echo "# FIMS Un-Installer created ${datestamp}T${timestamp}" > $upath
echo "cd ${INSTALL_DIR}/${PACK_NAME}" >> $upath

# Compile FIMS tool
if [ $cflag -ne 0 ]; then
    cd ${INSTALL_DIR}/${PACK_NAME}/src
    Doing "Compiling autopos module..." 1
    Doing "Uncompressing autopos source code..." 2
    cmd="gzip -d -c autopos-*.tar.gz | $tar xf - > /dev/null ;"
    Waitfor "uncompressing autopos source..." "${cmd}" 2
    status=$?
    CheckStatus $status $status $LINENO $EC_ABORT X "" 2
    Done "Uncompressing autopos source code." 2
    echo " "
    Doing "Compiling autopos source code..." 2
    clf="/tmp/autopos_compilation_warnings.log"
    csf="/tmp/autopos_make_status.txt"
    cmd="
        cd ${INSTALL_DIR}/${PACK_NAME}/src/autopos/src ;
        make -f makefile.novlt all > $clf ;
        echo \$? > $csf ;
    "
    Waitfor "autopos compilation..." "${cmd}" 2 "$clf"
    status1=$?
    status2=`cat $csf`
    status=$((status1 || status2))
    CheckStatus $status ${status1}.$status2 $LINENO $EC_ABORT I "Look at $clf for causes." 2
    Done "Compiling autopos source code." 2
    echo " "
    # Copy binaries to bin
    Doing "Installing autopos binary..." 2
    rm -f ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/AP
    cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/autopos/bin/AP ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/AP > /dev/null 2>&1
    status=$?
    if [ $status -ne 0 ]; then
        Warning "Could not cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/autopos/bin/AP ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/AP"
    fi
    Done "Installing autopos binary." 2
    echo " "
    # Clean up
    Doing "Cleaning up autopos..." 2
    make -f makefile.novlt -s clean > /dev/null 2>&1
    Done "Cleaning up autopos." 2
    cd ${INSTALL_DIR}/${PACK_NAME}/src
    Done "Compiling autopos module." 1
    echo " "

    Doing "Compiling fims module..." 1
    Doing "Uncompressing fims source code..." 2
    cmd="gzip -d -c fims-*.tar.gz | $tar xf - > /dev/null ;"
    Waitfor "uncompressing fims source..." "${cmd}" 2
    status=$?
    CheckStatus $status $status $LINENO $EC_ABORT X "" 2
    Done "Uncompressing fims source code." 2
    echo " "
    Doing "Compiling fims source code..." 2
    if [ -n "${VLTROOT}" ]; then
        # When building in a VLTSW environment,
        # the FIMS batch software is installed
        makefile=Makefile
    else
        makefile=makefile.novlt
    fi
    clf="/tmp/fims_compilation_warnings.log"
    csf="/tmp/fims_make_status.txt"
    cmd="
        cd ${INSTALL_DIR}/${PACK_NAME}/src/fims/src ;
        make -f $makefile all > $clf ;
        echo \$? > $csf ;
    "
    Waitfor "fims compilation..." "${cmd}" 2 "$clf"
    status1=$?
    status2=`cat $csf`
    status=$((status1 || status2))
    CheckStatus $status ${status1}.$status2 $LINENO $EC_ABORT I "Look at $clf for causes." 2
    Done "Compiling fims source code." 2
    echo " "
    Doing "Installing fims binaries, configurations, and libraries..." 2
    # Copy binaries to bin
    for b in `ls -1A$ls_nobackup $ls_ignore ../bin`; do
        rm -f ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/$b
        cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/fims/bin/$b ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/$b > /dev/null 2>&1
        status=$?
        if [ $status -ne 0 ]; then
                Warning "Could not cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/fims/bin/$b ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/$b"
        fi
    done
    # Copy library to lib
    for l in libesoPlugins.tcl libfimsPlugins.tcl; do
        rm -rf ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/lib/$l
        cp -rp ${INSTALL_DIR}/${PACK_NAME}/src/fims/lib/$l ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/lib/. > /dev/null 2>&1
        status=$?
        if [ $status -ne 0 ]; then
                Warning "Could not cp -rp ${INSTALL_DIR}/${PACK_NAME}/src/fims/lib/$l ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/lib/."
        fi
    done
    # Copy configuration files to config
    for c in `ls -1A$ls_nobackup $ls_ignore ../config`; do
        rm -f ${INSTALL_DIR}/${PACK_NAME}/config/$c
        cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/fims/config/$c ${INSTALL_DIR}/${PACK_NAME}/config/. > /dev/null 2>&1
        status=$?
        if [ $status -ne 0 ]; then
                Warning "Could not cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/fims/config/$c ${INSTALL_DIR}/${PACK_NAME}/config/."
        fi
    done
    Done "Installing fims binaries, configurations, and libraries." 2
    echo " "
    # Clean up
    Doing "Cleaning up fims..." 2
    make -f $makefile -s clean > /dev/null 2>&1
    Done "Cleaning up fims." 2
    cd ${INSTALL_DIR}/${PACK_NAME}/src
    Done "Compiling fims module." 1
    echo " "

    Doing "Compiling fsmosaic module..." 1
    Doing "Uncompressing fsmosaic source code..." 2
    cmd="gzip -d -c fsmosaic-*.tar.gz | $tar xf - > /dev/null ;"
    Waitfor "uncompressing fsmosaic source..." "${cmd}" 2
    status=$?
    CheckStatus $status $status $LINENO $EC_ABORT X "" 2
    Done "Uncompressing fsmosaic source code." 2
    echo " "
    Doing "Compiling fsmosaic source code..." 2
    clf="/tmp/fsmosaic_compilation_warnings.log"
    csf1="/tmp/fsmosaic_configure_status.txt"
    csf2="/tmp/fsmosaic_make_status.txt"
    cmd="
        cd ${INSTALL_DIR}/${PACK_NAME}/src/fsmosaic ;
        ./configure --prefix=$PWD --disable-shared > $clf ;
        echo \$? > $csf1 ;
        make >> $clf ;
        echo \$? > $csf2 ;
    "
    Waitfor "fsmosaic compilation..." "${cmd}" 2 "$clf"
    status1=$?
    status2=`cat $csf1`
    status3=`cat $csf2`
    status=$((status1 || status2 || status3))
    CheckStatus $status ${status1}.${status2}.$status3 $LINENO $EC_ABORT I "Look at $clf for causes." 2
    Done "Compiling fsmosaic source code." 2
    echo " "
    # Copy binaries to bin
    Doing "Installing fsmosaic binary..." 2
    rm -f ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/fsmosaic
    cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/fsmosaic/fsmosaic/fsmosaic ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/fsmosaic > /dev/null 2>&1
    status=$?
    if [ $status -ne 0 ]; then
        Warning "Could not cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/fsmosaic/fsmosaic/fsmosaic ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/fsmosaic"
    fi
    Done "Installing fsmosaic binary." 2
    echo " "
    # Clean up
    Doing "Cleaning up fsmosaic..." 2
    make -s clean > /dev/null 2>&1
    Done "Cleaning up fsmosaic." 2
    cd ${INSTALL_DIR}/${PACK_NAME}/src
    Done "Compiling fsmosaic module." 1
    echo " "

    Doing "Compiling pafchk module..." 1
    Doing "Uncompressing pafchk source code..." 2
    cmd="gzip -d -c pafchk-*.tar.gz | $tar xf - > /dev/null ;"
    Waitfor "uncompressing pafchk source..." "${cmd}" 2
    status=$?
    CheckStatus $status $status $LINENO $EC_ABORT X "" 2
    Done "Uncompressing pafchk source code." 2
    echo " "
    Doing "Compiling pafchk source code..." 2
    clf="/tmp/pafchk_compilation_warnings.log"
    csf="/tmp/pafchk_make_status.txt"
    cmd="
        cd ${INSTALL_DIR}/${PACK_NAME}/src/pafchk/src ;
        make -f makefile.novlt all > $clf ;
        echo \$? > $csf ;
    "
    Waitfor "pafchk compilation..." "${cmd}" 2 "$clf"
    status1=$?
    status2=`cat $csf`
    status=$((status1 || status2))
    CheckStatus $status ${status1}.$status2 $LINENO $EC_ABORT I "Look at $clf for causes." 2
    Done "Compiling pafchk source code." 2
    echo " "
    # Copy binaries to bin
    Doing "Installing pafchk binaries and libraries..." 2
    for b in pafchkCreate pafchkVerify; do
        rm -f ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/$b
        cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/pafchk/bin/$b ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/. > /dev/null 2>&1
        status=$?
        if [ $status -ne 0 ]; then
            Warning "Could not cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/pafchk/bin/$b ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/."
        fi
    done
    # Copy library to libs
    for l in libpafchk.a libpafchk.so; do
        rm -f ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/lib/$l
        cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/pafchk/lib/$l ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/lib/. > /dev/null 2>&1
        status=$?
        if [ $status -ne 0 ]; then
            Warning "Could not cp -pf ${INSTALL_DIR}/${PACK_NAME}/src/pafchk/lib/$l ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/lib/."
        fi
    done
    Done "Installing pafchk binaries and libraries." 2
    echo " "
    # Clean up
    Doing "Cleaning up pafchk..." 2
    make -f makefile.novlt -s clean > /dev/null 2>&1
    Done "Cleaning up pafchk." 2
    cd ${INSTALL_DIR}/${PACK_NAME}/src
    Done "Compiling pafchk module." 1
    echo " "

    # Compile required external packages
    if [ $rflag -ne 0 ]; then
        Doing "Installing external TclTk package..." 1
        Tell "This may take some minutes. Please be patient." 1
        Doing "Uncompressing tcltk source code..." 2
        cmd="gzip -d -c tcltk-*.tar.gz | $tar xf - > /dev/null ;
             rm -f tcltk-*.tar.gz ;
        "
        Waitfor "uncompressing tcltk source..." "${cmd}" 2
        status=$?
        CheckStatus $status $status $LINENO $EC_ABORT X "" 2
        Done "Uncompressing tcltk source code." 2
        echo " "
        Doing "Compiling tcltk source code..." 2
        clf="/tmp/tcltk_compilation_warnings.log"
        csf="/tmp/tcltk_make_status.txt"
        cmd="
            cd ${INSTALL_DIR}/${PACK_NAME}/src/tcltk-* ;
            make prefix=${TCLTK_ROOT} > $clf ;
            echo \$? > $csf ;
        "
        Waitfor "tcltk compilation..." "${cmd}" 2 "$clf"
        status1=$?
        status2=`cat $csf`
        status=$((status1 || status2))
        CheckStatus $status ${status1}.$status2 $LINENO $EC_ABORT I "Look at $clf for causes." 2
        Done "Compiling tcltk source code." 2
        cd ${INSTALL_DIR}/${PACK_NAME}/src
        Done "Installing external TclTk package." 1
        echo " "

        Doing "Installing external SkyCat package..." 1
        Tell "This may take some minutes. Please be patient." 1
        Doing "Uncompressing skycat source code..." 2
        cmd="gzip -d -c skycat-*.tar.gz | $tar xf - > /dev/null ;
             rm -f skycat-*.tar.gz ;
        "
        Waitfor "uncompressing skycat source..." "${cmd}" 2
        status=$?
        CheckStatus $status $status $LINENO $EC_ABORT X "" 2
        Done "Uncompressing skycat source code." 2
        echo " "
        Doing "Finding X11 headers..." 2
        rm -f ${INSTALL_DIR}/${BITS}b/ext/include/X11 2>&-
        if [ -d /usr/X11R6/include/X11 ]; then
            ln -f -s /usr/X11R6/include/X11 ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/ext/include/X11 2>&-
            status=$?
        elif [ -d /usr/X11/include/X11 ]; then
            ln -f -s /usr/X11/include/X11 ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/ext/include/X11 2>&-
            status=$?
        elif [ -d /usr/include/X11 ]; then
            ln -f -s /usr/include/X11 ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/ext/include/X11 2>&-
            status=$?
        else
            status=1
        fi
        if [ $status -ne 0 ]; then
            Tell "...not found" 2
        else
            Tell "...found" 2
        fi
        Done "finding X11 headers." 2
        echo " "
        Doing "Compiling skycat source code..." 2
        clf="/tmp/skycat_compilation_warnings.log"
        csf1="/tmp/skycat_configure_status.txt"
        csf2="/tmp/skycat_make_status.txt"
        cmd="
            cd ${INSTALL_DIR}/${PACK_NAME}/src/skycat-* ;
            ./configure --prefix=${INSTALL_DIR}/${PACK_NAME}/${BITS}b/ext --with-tcl=${TCLTK_ROOT}/lib --with-tk=${TCLTK_ROOT}/lib --with-blt=${TCLTK_ROOT}/lib --enable-merge > $clf ;
            echo \$? > $csf1 ;
            make all install >> $clf ;
            echo \$? > $csf2 ;
        "
        Waitfor "skycat compilation..." "${cmd}" 2 "$clf"
        status1=$?
        status2=`cat $csf1`
        status3=`cat $csf2`
        status=$((status1 || status2 || status3))
        CheckStatus $status ${status1}.${status2}.$status3 $LINENO $EC_ABORT I "Look at $clf for causes." 2
        Done "Compiling skycat source code." 2
        cd ${INSTALL_DIR}/${PACK_NAME}/src
        Done "Installing external SkyCat package." 1
        echo " "
    fi
fi

if [ $eflag -ne 0 ]; then
    Exit $eflag
fi

Doing "Installing scripts, libraries and binaries..." 1
if [ -n "${VLTROOT}" ]; then
    vltroot=${VLTROOT}
else
    vltroot=${INSTALL_DIR}/${PACK_NAME}
fi
# Create local links to libraries, executables and scripts
if [ -n "${INTROOT}" ]; then
    # If it is installed in a VLTSW directory structure, replace some libraries
    # and binaries in INTROOT. Tcl/Tk should be installed.
    if [ -d ${INTROOT}/bin -a -w ${INTROOT}/bin ]; then
        binpath=${INTROOT}/bin
    fi
    if [ -d ${INTROOT}/lib -a -w ${INTROOT}/lib ]; then
        libpath=${INTROOT}/lib
    fi
fi
if [ -z "$binpath" ]; then
    mkdir -p ${HOME}/binfims
    binpath=${HOME}/binfims
fi
if [ -z "$libpath" ]; then
    mkdir -p ${HOME}/libfims
    libpath=${HOME}/libfims
fi
cd ${INSTALL_DIR}/${PACK_NAME}
if [ -n "$binpath" -a -w $binpath ]; then
    Doing "Linking bin from $binpath..." 2
    for b in `ls -1A$ls_nobackup ${PWD}/${BITS}b/bin`; do
        SafeLink ${PWD}/${BITS}b/bin/$b ${binpath}/$b
    done
    if [ $rflag -ne 0 ]; then
        Tell "Adding links to external pre-compiled binaries..." 2
        for eb in `ls -1A$ls_nobackup ${PWD}/${BITS}b/ext/bin`; do
            SafeLink ${PWD}/${BITS}b/ext/bin/$eb ${binpath}/$eb
        done
    fi
    Done "Linking bin." 2
    echo " "
else
    Error   "could not define links to executables"
    Explain "Cannot complete install."
    Exit $EC_ABORT
fi
    
if [ -n "$libpath" -a -w $libpath ]; then
    Doing "Linking lib from $libpath..." 2
    for l in `ls -1A$ls_nobackup ${PWD}/${BITS}b/lib`; do
        SafeLink ${PWD}/${BITS}b/lib/$l ${libpath}/$l
    done
    if [ $rflag -ne 0 ]; then
        Tell "Adding links to external pre-compiled libraries..." 2
        for el in `ls -1A$ls_nobackup ${PWD}/${BITS}b/ext/lib`; do
            SafeLink ${PWD}/${BITS}b/ext/lib/$el ${libpath}/$el
        done
    fi
    if [ ! -f ${PWD}/${BITS}b/ext/lib/libstdc* ]; then
        # Link all the libstdc++ libraries we can find in this system
        lpath_list=`whereis libstdc++.so.* | awk -F: '{printf("%s\n",$2)}'`
        for lp in $lpath_list; do
            ln=`basename $lp`
            SafeLink $lp ${libpath}/$ln
        done
        if [ -z "$lpath_list" ]; then
            Warning "Could not find any libstdc++ to link into your environment."
        fi
    fi
    Done "Linking lib." 2
    echo " "
else
    Error   "could not define links to libraries"
    Explain "Cannot complete install."
    Exit $EC_ABORT
fi

mkdir -p ${HOME}/bin
cp -pf ${UNPACK_DIR}/bash_library.sh ${PWD}/${BITS}b/bin/. > /dev/null 2>&1
SafeLink ${PWD}/${BITS}b/bin/bash_library.sh ${HOME}/bin/bash_library.sh
SafeLink ${PWD}/${BITS}b/bin/fims.sh ${HOME}/bin/fims.sh

# Add extra cleaning of bin and lib links to the uninstaller macro
if [ -n "$binpath" ]; then
    echo "rm -f \`ls -laA$ls_nobackup $binpath 2>&- | grep $PACK_NAME | awk '{printf(\"$binpath/%s \",\$9)}'\`" >> $upath
fi
if [ -n "$libpath" ]; then
    echo "rm -f \`ls -laA$ls_nobackup $libpath 2>&- | grep $PACK_NAME | awk '{printf(\"$libpath/%s \",\$9)}'\`" >> $upath
fi
Done "Installing scripts, libraries and binaries." 1
echo " "
Doing "Updating PERL scripts..." 1
chmod u+w ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/gerber2ps.perl > /dev/null 2>&1
rm -f gerber2ps.perl.new > /dev/null 2>&1
cat ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/gerber2ps.perl | awk '{if(NR==1) printf("#!" A "\n"); else print $0}' A=$PERL_PATH > gerber2ps.perl.new > /dev/null 2>&1
mv -f gerber2ps.perl.new ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/gerber2ps.perl > /dev/null 2>&1

chmod u+w ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/gerber2eps.perl > /dev/null 2>&1
rm -f gerber2eps.perl.new > /dev/null 2>&1
cat ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/gerber2eps.perl | awk '{if(NR==1) printf("#!" A "\n"); else print $0}' A=$PERL_PATH > gerber2eps.perl.new > /dev/null 2>&1
mv -f gerber2eps.perl.new ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/gerber2eps.perl > /dev/null 2>&1
chmod 555 ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/*.perl > /dev/null 2>&1
Done "Updating PERL scripts" 1
echo " "
Doing "Updating launch script..." 1
# Jejeje, 27 backslashes to replace one / by \\\/ 
# (3 backslashes followed by a slash)
# giving a substitution like /my/full/path/to/fims/ for
# \\\/my\\\/full\\\/path\\\/to\\\/fims
# which is needed to pass this path string through sed again without the / being
# mistaken as pattern delimiters.
FIMS_PATH=`echo ${INSTALL_DIR}/${PACK_NAME}/${BITS}b | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
INS_PATH=`echo ${INSROOT} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
VLT_PATH=`echo ${vltroot} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
TCL_PATH=`echo ${tclroot} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
BIN_PATH=`echo ${binpath} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
LIB_PATH=`echo ${libpath} | sed s/[/]/\\\\\\\\\\\\\\\\\\\\\\\\\\\//g`
Debug "FIMS_ROOT   = >$FIMS_PATH<"
Debug "INSROOT     = >$INS_PATH<"
Debug "VLTROOT     = >$VLT_PATH<"
Debug "TCLPATH     = >$TCL_PATH<"
Debug "BINPATH     = >$BIN_PATH<"
Debug "LIBPATH     = >$LIB_PATH<"
Debug "INSUSER     = >$INSUSER<"
sed -e s/@@FIMSBASEPATH@@/$FIMS_PATH/ \
    -e s/@@INSPATH@@/$INS_PATH/ \
    -e s/@@VLTPATH@@/$VLT_PATH/ \
    -e s/@@TCLPATH@@/$TCL_PATH/ \
    -e s/@@BINPATH@@/$BIN_PATH/ \
    -e s/@@LIBPATH@@/$LIB_PATH/ \
    -e s/@@USER@@/$INSUSER/ \
    -e s/@@TCLTKREQVER@@/$TCLTK_REQVER/ \
    -e s/@@SKYCATREQVER@@/$SKYCAT_REQVER/ \
    ${INSTALL_DIR}/${PACK_NAME}/src/fims.sh.tpl > ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/fims.sh

chmod 555 ${INSTALL_DIR}/${PACK_NAME}/${BITS}b/bin/fims.sh
Done "Updating launch scripts." 1
echo " "
Doing "Copying configuration files..." 1
# Create FIMS user directories
if [ ! -d "${INSROOT}/${INSUSER}" ]; then
    mkdir -p ${INSROOT}/${INSUSER} > /dev/null 2>&1
fi
if [ ! -d "$PREP_DIR" ]; then
    mkdir -p $PREP_DIR > /dev/null 2>&1
    chmod -f 666 $PREP_DIR/* > /dev/null 2>&1
fi
if [ ! -d "$LOG_DIR" ]; then
    mkdir -p $LOG_DIR > /dev/null 2>&1
    chmod -f 666 $LOG_DIR/* > /dev/null 2>&1
fi
if [ ! -d "$SET_DIR" ]; then
    mkdir -p $SET_DIR > /dev/null 2>&1
    chmod -f 666 $SET_DIR/* > /dev/null 2>&1
fi

# Backup skycat.cfg
if [ -f "${INSROOT}/${INSUSER}/skycat.cfg" ]; then
    mv -f ${INSROOT}/${INSUSER}/skycat.cfg ${INSROOT}/${INSUSER}/skycat.cfg.$datestamp.$timestamp > /dev/null 2>&1
    echo "mv -f ${INSROOT}/${INSUSER}/skycat.cfg.$datestamp.$timestamp ${INSROOT}/${INSUSER}/skycat.cfg" >> $upath
fi
if [ -f "${INSROOT}/.skycat/skycat.cfg" ]; then
    mv -f ${INSROOT}/.skycat/skycat.cfg ${INSROOT}/.skycat/skycat.cfg.$datestamp.$timestamp > /dev/null 2>&1
    echo "mv -f ${INSROOT}/.skycat/skycat.cfg.$datestamp.$timestamp ${INSROOT}/.skycat/skycat.cfg.orig" >> $upath
fi
# Copy configuration files
cp -pf ${INSTALL_DIR}/${PACK_NAME}/config/skycat.cfg ${INSROOT}/${INSUSER}/. > /dev/null 2>&1
cp -pf ${INSTALL_DIR}/${PACK_NAME}/config/mxu2.???.standard.fad ${SET_DIR}/. > /dev/null 2>&1
# Maybe some .fits, .cat, .fims for a beginners test
# can be copied to PREP_DIR, SET_DIR
Done "Copying configuration." 1
# Remove TclTk and SkyCat version files if not -r
if [ $rflag -eq 0 ]; then
    rm -f ${INSTALL_DIR}/${PACK_NAME}/doc/tcltk_version.txt > /dev/null 2>&1
    rm -f ${INSTALL_DIR}/${PACK_NAME}/doc/skycat_version.txt > /dev/null 2>&1
fi
Done "Installing FIMS package."
echo " "
Frame
Explain " "
Explain "Execute FIMS tool with:"
Explain "~$> ~/bin/fims.sh [B|R]"
Explain "     B: Configure for Blue CCD"
Explain "     R: Configure for Red CCD"
Explain "Have Fun With FIMS! "
Explain " "
Frame
rm -f ${UNPACK_DIR}/${PACK_NAME}.tar.gz > /dev/null 2>&1
#
# Main Algorithm END
####################

#########################
# Standard Epilogue BEGIN
#
Exit $EC_END
#
# Standard Epilogue END
#######################
#
#___FIMS___
